
#include "GraphWinAGDopt.h"
#include <AGD/SubgraphPlanarizer.h>


int agd_opt_demo_main()
{
	GraphWinAGDopt *GW = new GraphWinAGDopt;

	GW->open();

	delete GW;

	return 0;
}


GraphWinAGDopt::GraphWinAGDopt() : GraphWinAGD()
{
	gw_add_simple_call (*this, callPlanAugOpt, "opt. planar biconnected", augment_menu);
	gw_add_simple_call (*this, callPlanSubOpt, "maximum planar", subgraph_menu);

	opt_ps_augment = 0;
	opt_pd_augment = 0;
	opt_mm_augment = 0;

}

void GraphWinAGDopt::optionsPlanarization()
{
	int layouter = planarization_layouter;
	int planarizer = planarization_planarizer;

	panel P ("Options for algorithm Planarization");

	P.choice_item ("Planar Subgraph:", planarizer, "approx.", "maximum planar");
	P.choice_item ("Layout:", layouter, "Orthogonal", "Mixed-Model");

	P.button ("Ok");
	int cancel = P.button ("Cancel");

	if (P.open (get_window()) != cancel) {
		planarization_layouter = layouter;
		switch (layouter) {
		case 0:
			planarization.set_planar_layouter(quod);
			break;
		case 1:
			planarization.set_planar_layouter(mixed);
			break;
		}
		planarization_planarizer = planarizer;
		SubgraphPlanarizer SP;
		switch (planarizer) {
		case 0:
			SP.set_subgraph(plan_sub);
			break;
		case 1:
			SP.set_subgraph(plan_sub_opt);
			break;
		}
		planarization.set_planarizer(SP);
	}
}

void GraphWinAGDopt::optionsStraight()
{
	bool   size_opt   = straight.size_opt();
	bool   max_outer  = straight.max_outer();
	double base_ratio = straight.base_ratio();
	int    augment = opt_ps_augment;

	panel P ("Options for algorithm PlanarStraight");

	P.bool_item ("size_opt:",   size_opt);
	P.bool_item ("max_outer:",  max_outer);
	P.real_item ("base_ratio:", base_ratio);
	P.choice_item ("augmentation:", augment, "heuristic", "optimal");

	P.button ("Ok");
	int cancel = P.button ("Cancel");

	if (P.open (get_window()) != cancel) {
		straight.size_opt   (size_opt);
		straight.max_outer  (max_outer);
		straight.base_ratio (base_ratio);
		if (augment != opt_ps_augment) {
			opt_ps_augment = augment;
			switch (augment) {
			case 0:
				straight.set_augmenter(PlanAug());
				break;
			case 1:
				straight.set_augmenter(PlanAugOpt());
				break;
			}
		}
	}
}

void GraphWinAGDopt::optionsPlanarDraw()
{
	bool   size_opt   = planar_draw.size_opt();
	bool   side_opt   = planar_draw.side_opt();
	bool   max_outer  = planar_draw.max_outer();
	double base_ratio = planar_draw.base_ratio();
	int    augment    = opt_pd_augment;

	panel P ("Options for algorithm PlanarDraw");

	P.bool_item ("size_opt:",   size_opt);
	P.bool_item ("side_opt:",   side_opt);
	P.bool_item ("max_outer:",  max_outer);
	P.real_item ("base_ratio:", base_ratio);
	P.choice_item ("augmentation:", augment, "heuristic", "optimal");

	P.button ("Ok");
	int cancel = P.button ("Cancel");

	if (P.open (get_window()) != cancel) {
		planar_draw.size_opt   (size_opt);
		planar_draw.side_opt   (side_opt);
		planar_draw.max_outer  (max_outer);
		planar_draw.base_ratio (base_ratio);
		if (augment != opt_pd_augment) {
			opt_pd_augment = augment;
			switch (augment) {
			case 0:
				planar_draw.set_augmenter(PlanAug());
				break;
			case 1:
				planar_draw.set_augmenter(PlanAugOpt());
				break;
			}
		}
	}
}

void GraphWinAGDopt::optionsMixedModel()
{
	int    co_type           = (int)mixed.co_type();
	bool   prefer_nodes      = mixed.prefer_nodes();
	double base_ratio        = mixed.base_ratio();
	bool   do_postprocessing = mixed.do_postprocessing();
	int    augment           = opt_mm_augment;

	panel P ("Options for algorithm Mixed-Model");

	P.choice_item ("co_type:", co_type, "triconnected", "biconnected");
	P.bool_item ("prefer_nodes:",  prefer_nodes);
	P.real_item ("base_ratio:", base_ratio);
	P.bool_item ("do_postprocessing:", do_postprocessing);
	P.choice_item ("augmentation:", augment, "heuristic", "optimal");

	P.button ("Ok");
	int cancel = P.button ("Cancel");

	if (P.open (get_window()) != cancel) {
		mixed.co_type           ((CanonicalOrderType)(co_type));
		mixed.prefer_nodes      (prefer_nodes);
		mixed.base_ratio        (base_ratio);
		mixed.do_postprocessing (do_postprocessing);
		if (augment != opt_mm_augment) {
			opt_mm_augment = augment;
			switch (augment) {
			case 0:
				mixed.set_augmenter(PlanAug());
				break;
			case 1:
				mixed.set_augmenter(PlanAugOpt());
				break;
			}
		}
	}
}

