//-------------------------------------------------------------------
//
//                         AGDLayout.h
//
//    author: Carsten Gutwenger
//    date:   august 1997
//-------------------------------------------------------------------


#ifndef _AGD_AGD_LAYOUT_H
#define _AGD_AGD_LAYOUT_H


#include <AGD/LayoutBuffer.h>


/*{\Manpage {AGDLayout} {} {Layouts} {A} }*/

/*{\Mbaseclasses

\begin{hierarchy}{8}{4}
\hdclass{0}{3}{5}{LayoutInterface}
\arhook{5}{3.5}{6}
\hdclass{1.5}{1.5}{5}{LayoutBuffer}
\arhook{6.5}{2}{7.5}
\hdclass{3}{0}{5}{AGDLayout}
\end{hierarchy}
}*/

class AGDLayout : public LayoutBuffer {

/*{\Mdefinition
An instance |\Mvar| of type |\Mname| is a layout interface that stores the
graphical attributes of a particular graph $G$. It can be used, e.g., if
the layout of $G$ shall not be visualized immediately.
}*/

public:

	/*{\Mcreation A 5.5}*/

	AGDLayout(const leda_graph& G);
	/*{\Mcreate creates an instances |\Mvar| of type |\Mname| initialized
	to a layout for graph $G$.}*/

	~AGDLayout() { }


	/*{\Moperations}*/

	double get_width  (leda_node v) const {
		return _width[v];
	}
	/*{\Mop returns the width of the graphics of node |v|.}*/

	void set_width  (leda_node v, double new_w) {
		_width[v] = new_w;
	}
	/*{\Mop sets the width of the graphics of node |v| to |new_w|.}*/

	double get_height (leda_node v) const {
		return _height[v];
	}
	/*{\Mop returns the height of the graphics of node |v|.}*/

	void set_height (leda_node v, double new_h) {
		_height[v] = new_h;
	}
	/*{\Mop sets the height of the graphics of node |v| to |new_h|.}*/

private:
	leda_node_map<double> _width, _height;
};


#endif
