//-------------------------------------------------------------------
//
//                         AGD_Key.h
//
//
//    author: Carsten Gutwenger
//    date:   july 1997
//-------------------------------------------------------------------


#ifndef _AGD_AGD_KEY_H
#define _AGD_AGD_KEY_H



#include <LEDA/slist.h>


// define hash function for strings (used by Keymapper)
inline int Hash (const leda_string& s)
{
	int i, sum;

	for (i = 0, sum = 0; i < s.length(); i++)
		sum += (int)(s[i]);
	return sum;
}

class Keymapper;

/*{\Manpage {AGD_Key} {} {Key Structs} {K} }*/

class AGD_Key {
	friend Keymapper;

/*{\Mdefinition
An instance |\Mvar| of type |\Mname| is a pair $(s,i)$ of a string
$s$ and an index $i$. We define the type |agd_key| to be a pointer
to an instance of type |\Mname| and call such a pointer a key.
}*/

private:
	const leda_string str;
	const int _id;

	slist_item pos_in_list;

public:

	/*{\Mcreation}*/

	AGD_Key(const AGD_Key& K) : str(K.str), _id(K._id) { }

	AGD_Key(const leda_string& s, int id) : str (s), _id (id) { }
	/*{\Mcreate creates an instances |\Mvar| of type |\Mtype| initialized
	to the pair $(s,|id|)$.}*/


	/*{\Moperations}*/

	const leda_string& info() const {
		return str;
	}
	/*{\Mop returns the string $s$ if $\Mvar=(s,i)$.}*/

	int id() const {
		return _id;
	}
	/*{\Mop returns the index $i$ if $\Mvar=(s,i)$.}*/

	LEDA_MEMORY(AGD_Key)
};

typedef AGD_Key *agd_key;

inline ostream& operator<< (ostream& os, const agd_key& k) {
	os << (k->info());
	return os;
}
/*{\Mbinopfunc writes |s| to the output stream |os| if $k$ points
to $(s,i)$.}*/


// overloaded compare function for keys
inline int compare (const agd_key& k1, const agd_key& k2)
{
	return compare (k1->id(), k2->id());
}


#endif
