//-------------------------------------------------------------------
//
//                           DPoint.h
//
//    author: Carsten Gutwenger
//    date:   june 1997
//-------------------------------------------------------------------


#ifndef _AGD_DPOINT_H
#define _AGD_DPOINT_H



class istream;
class ostream;


/*{\Manpage {DPoint} {} {Double Points} {dp} }*/

class DPoint {

/*{\Mdefinition
An instance |\Mvar| of type |\Mname| is a point $(a,b)$, where $a$ and
$b$ are doubles. We call $a$ the x-coordinate of |\Mvar| and $b$ the
y-coordinate of |\Mvar|.}*/

private:
	double the_x, the_y;

public:

	/*{\Mcreation}*/

	DPoint() { the_x = the_y = 0.0; }
	/*{\Mcreate  creates an instance |\Mvar| of type |\Mname| initialized to
	the point $(0,0)$.}*/

	DPoint (double x, double y) { the_x = x; the_y = y; }
	/*{\Mcreate  creates an instance |\Mvar| of type |\Mname| initialized to
	the point $(x,y)$.}*/

	DPoint (const DPoint& p) { the_x = p.the_x; the_y = p.the_y; }
	/*{\Mcreate  creates an instance |\Mvar| of type |\Mname| initialized to
	the point |dp|.}*/


	/*{\Moperations}*/

	double xcoord() const { return the_x; }
	/*{\Mop returns the x-coordinate of |\Mvar|.}*/

	double ycoord() const { return the_y; }
	/*{\Mop returns the y-coordinate of |\Mvar|.}*/

	void xcoord (double x) { the_x = x; }
	/*{\Mop sets the x-coordinate of |\Mvar| to $x$.}*/

	void ycoord (double y) { the_y = y; }
	/*{\Mop sets the y-coordinate of |\Mvar| to $y$.}*/

	DPoint& inc_x (double dx) {
		the_x += dx;
		return *this;
	}
	/*{\Mop increments the x-coordinate of |\Mvar| by |dx|. Returns
	a reference to |\Mvar|.}*/

	DPoint& inc_y (double dy) {
		the_y += dy;
		return *this;
	}
	/*{\Mop increments the y-coordinate of |\Mvar| by |dy|. Returns
	a reference to |\Mvar|.}*/

	bool operator== (const DPoint& p) const {
		return (the_x == p.the_x && the_y == p.the_y);
	}
	/*{\Mbinop Test for equality.}*/

	bool operator!= (const DPoint& p) const {
		return (the_x != p.the_x || the_y != p.the_y);
	}
	/*{\Mbinop Test for inequality.}*/

	DPoint operator+ (const DPoint& dp1) const {
		return DPoint(the_x+dp1.the_x, the_y+dp1.the_y);
	}
	/*{\Mbinop returns the addition (of coordinates).}*/

	DPoint operator- (const DPoint& dp1) const {
		return DPoint(the_x-dp1.the_x, the_y-dp1.the_y);
	}
	/*{\Mbinop returns the subtraction (of coordinates).}*/

	DPoint& operator+= (const DPoint& dp1) {
		the_x += dp1.the_x;
		the_y += dp1.the_y;
		return *this;
	}
	/*{\Mbinop adds |dp1| to |\Mvar|. Returns
	a reference to |\Mvar|.}*/

	DPoint& operator-= (const DPoint& dp1) {
		the_x -= dp1.the_x;
		the_y -= dp1.the_y;
		return *this;
	}
	/*{\Mbinop subtracts |dp1| from |\Mvar|. Returns
	a reference to |\Mvar|.}*/


	LEDA_MEMORY(DPoint)
};

ostream& operator<< (ostream& O, const DPoint& dp);
/*{\Mbinopfunc writes |dp| to output stream $O$.}*/

istream& operator>> (istream& I, DPoint& dp);
/*{\Mbinopfunc reads the coordinates of |dp| (two $double$ numbers)
from input stream $I$.}*/



#endif
