//-------------------------------------------------------------------
//
//                        HierarchyLayout.h
//
//
//    author: Carsten Gutwenger
//    date:   february 1998
//-------------------------------------------------------------------


#ifndef _AGD_HIERARCHY_LAYOUT_H
#define _AGD_HIERARCHY_LAYOUT_H

#include <AGD/AGDModule.h>


class Hierarchy;
class GraphCopy;
class LayoutInterface;

/*{\Manpage {HierarchyLayoutModule} {} {Hierarchy Layout Modules} {L} }*/

/*{\Mbaseclasses

\begin{hierarchy}{7.5}{2.5}
\hdclass{0.5}{1.5}{5.5}{AGDModule}
\arhook{6}{2}{7}
\hdclass{0}{0}{7.5}{HierarchyLayoutModule}
\end{hierarchy}
}*/

class HierarchyLayoutModule : public AGDModule {

/*{\Mdefinition
Modules of type |\Mname| compute a layout of a node hierarchy |H|.
|H| is a hierarchy of a graph |GC|, which is a copy of a graph |G|.
|GC| may contain dummy nodes that are not copies of nodes in |G|.
These dummy nodes have degree two and represent edge bends in
the final drawing.

\headerline{Implementing Hierarchy Layout Modules}

An implementation of a hierarchy layout module
must override the method call(|GC|,$H$,|bbox|,|A|), which gets as
input a graph copy |GC| and a node hierarchy |H| of |GC|. It
returns the layout in |A| and the bounding box of the computed layout
in |bbox|.

\headerline{Input and Output Parameters}

\begin{tabular}{lcc}
input parameter: & graph & $|in_graph|=1$ \\
 & hierarchy & $|in_hierarchy|=2$ \\
output parameter: & layout & $|out_layout|=1$
\end{tabular}

The integer constants |in_graph|, |in_layer| and |out_layer| can be used to denote
the positions of the input and output parameters.
}*/

public:

	// the input and output parameters of a hierarchy layout module
	static const int in_graph, in_hierarchy;
	static const int out_layout;

	/*{\Minitialization L 5}*/

	HierarchyLayoutModule () : AGDModule(2,1) { }

	/*{\Mcreate initializes a hierarchy layout module.}*/

	virtual ~HierarchyLayoutModule () { }


	/*{\Moperations 1.2 3}*/

	virtual void call (const GraphCopy& GC, const Hierarchy& H,
		LayoutInterface& A) = 0;
	/*{\Moppure implements the module call for graph $GC$ and hierarchy
	$H$. Assigns the layout to $A$.}*/
};


#endif
