//-------------------------------------------------------------------
//
//                         IPolyline.h
//
//    author: Carsten Gutwenger
//    date:   march 1997
//-------------------------------------------------------------------


#ifndef _AGD_IPOLYLINE_H
#define _AGD_IPOLYLINE_H


#include <AGD/IPoint.h>
#include <AGD/impl/misc.h>
#include <LEDA/list.h>


class ISegment;


/*{\Manpage {IPolyline} {} {Integer Polylines} {ipl} }*/

/*{\Mbaseclasses

\begin{hierarchy}{6.5}{2.5}
\hdclass{0}{1.5}{5}{list{\tt <}IPoint{\tt >}}
\arhook{5}{2}{6}
\hdclass{1.5}{0}{5}{IPolyline}
\end{hierarchy}
}*/

class IPolyline : public leda_list<IPoint> {

/*{\Mdefinition
An instance |\Mvar| of type |\Mname| is a list
$|ip|_1,\ldots,|ip|_n$ of integer points $|ip|_i$.
}*/

public:

	/*{\Mcreation}*/

	IPolyline () : leda_list<IPoint>() { }
	/*{\Mcreate creates an instance |\Mvar| of type |\Mname|
	initialized to an empty list.}*/

	IPolyline (const IPolyline& ipl) : leda_list<IPoint>(ipl) { }

	virtual ~IPolyline() { }

	/*{\Moperations}*/

	IPoint& get_point (int i);
	/*{\Mop returns a reference to $|ip|_i$ if |\Mvar| is
	$|ip|_1,\ldots,|ip|_n$. \precond $1\leq i\leq n$}*/

	IPoint& get_rev_point (int i);
	/*{\Mop returns a reference to $|ip|_{n-i+1}$ if |\Mvar| is
	$|ip|_1,\ldots,|ip|_n$. \precond $1\leq i\leq n$}*/

	ISegment get_segment (int i) const;
	/*{\Mop returns the integer segment $[|ip|_i,|ip|_{i+1}]$ if
	|\Mvar| is $|ip|_1,\ldots,|ip|_n$. \precond $1\leq i< n$}*/

	ISegment get_rev_segment (int i) const;
	/*{\Mop returns the integer segment $[|ip|_{n-i+1},|ip|_{n-i}]$ if
	|\Mvar| is $|ip|_1,\ldots,|ip|_n$. \precond $1\leq i< n$}*/

	void reverse () {
		reverse_list (*((leda_list<IPoint> *)(this)));
	}
	/*{\Mop reverses the list of integer points.}*/
};

/*{\Mimplementation
All operations for
|list<IPoint>| are also available for |\Mname|.
}*/


#endif
