//-------------------------------------------------------------------
//
//                          Segment.h
//
//    author: Carsten Gutwenger
//    date:   march 1997
//-------------------------------------------------------------------


#ifndef _AGD_ISEGMENT_H
#define _AGD_ISEGMENT_H


#include <AGD/IPoint.h>


/*{\Manpage {ISegment} {} {Integer Line Segments} {is} }*/

class ISegment {

/*{\Mdefinition
An instance |\Mvar| of type |\Mname| is a line segment $[p,q]$,
where $p$ and $q$ are integer points. We call $p$ the source point
and $q$ the target point of the segment.
}*/

public:

	/*{\Mcreation}*/

	ISegment () { }
	/*{\Mcreate creates an instance |\Mvar| of type |\Mname|
	initialized to the segment $[(0,0),(0,0)]$.}*/

	ISegment (const IPoint& p, const IPoint& q) {
		the_p = p; the_q = q;
	}
	/*{\Mcreate creates an instance |\Mvar| of type |\Mname|
	initialized to the segment $[p,q]$.}*/


	/*{\Moperations}*/

	IPoint& source() {
		return the_p;
	}

	const IPoint& source() const {
		return the_p;
	}
	/*{\Mop returns a reference to the source point of |\Mvar|.}*/

	IPoint& target() {
		return the_q;
	}

	const IPoint& target() const {
		return the_q;
	}
	/*{\Mop returns a reference to the target point of |\Mvar|.}*/

	bool horizontal() const {
		return (the_p.ycoord() == the_q.ycoord());
	}
	/*{\Mop returns |true| iff |\Mvar| is horizontal, i.e
	the y-coordinate of the source point equals the
	y-coordinate of the target point.}*/

	bool vertical() const {
		return (the_p.xcoord() == the_p.ycoord());
	}
	/*{\Mop returns |true| iff |\Mvar| is vertical, i.e
	the x-coordinate of the source point equals the
	x-coordinate of the target point.}*/

		
private:
	IPoint the_p, the_q;
};


#endif
