//-------------------------------------------------------------------
//
//                         Keymapper.h
//
//
//    author: Carsten Gutwenger
//    date:   july 1997
//-------------------------------------------------------------------


#ifndef _AGD_KEYMAPPER_H
#define _AGD_KEYMAPPER_H

#include <AGD/AGD_Key.h>
#include <LEDA/h_array.h>


// the Keymapper assigns int id's 0,1,... to keys

/*{\Manpage {Keymapper} {} {Key Mapper} {M} }*/

class Keymapper {

/*{\Mdefinition
An instance |\Mvar| of type |\Mname| is an injective mapping from strings
to keys. We use |dom(\Mvar)| to denote the set of used strings, and |\Mvar(s)|
to denote the value of |\Mvar| for string s. 
}*/

private:
	leda_h_array<leda_string,agd_key> stringToKey;
	leda_slist<agd_key> _keys;


public:

	/*{\Mcreation}*/

	Keymapper ();
	/*{\Mcreate creates an instances |\Mvar| of type |\Mtype|.
	Sets |dom(\Mvar)| to the empty set.}*/

	~Keymapper ();

	
	/*{\Moperations}*/

	int number_of_keys() {
		return _keys.length();
	}
	/*{\Mop returns the number of defined keys.}*/

	agd_key ask_key (const leda_string& s) const {
		return stringToKey [s];
	}
	/*{\Mop returns |\Mvar(s)| if $|s|\in |dom(\Mvar)|$, |nil|
	otherwise.}*/

	agd_key get_key (const leda_string& s);
	/*{\Mop returns |\Mvar(s)|. Creates a new key and adds |s| to
	|dom(\Mvar)| if $|s|\notin |dom(\Mvar)|$.}*/

	leda_string get_string (agd_key k) const {
		return k->info();
	}
	/*{\Mop returns the string |s| with $|\Mvar(s)|=k$.}*/

	// Iteration

	agd_key first_key() const;

	agd_key next_key (agd_key k) const;

};

/*{\Mtext
\bigskip
\headerline{Iteration}
{\bf forall\_keys}($k,M$) 
$\{$  ``the keys defined in |M| are successively assigned to $k$''  $\}$
}*/

#define forall_keys(k,M) \
	for(k=M.first_key(); k != nil; k = M.next_key(k))


#endif
