//-------------------------------------------------------------------
//
//                       MedianHeuristic.h
//
//
//    author: Carsten Gutwenger
//            Michael Krger
//    date:   february 1998
//-------------------------------------------------------------------


#ifndef _AGD_MEDIAN_HEURISTIC_H
#define _AGD_MEDIAN_HEURISTIC_H


#include <AGD/TwoLayerCrossMin.h>


/*{\Manpage {MedianHeuristic} {} {Median Heuristic} {M} }*/

/*{\Mbaseclasses

\begin{hierarchy}{8}{4}
\hdclass{0}{3}{5.5}{AGDModule}
\arhook{5.5}{3.5}{6.5}
\hdclass{1}{1.5}{6}{TwoLayerCrossMin}
\arhook{7}{2}{8}
\hdclass{2.5}{0}{6}{MedianHeuristic}
\end{hierarchy}
}*/

class MedianHeuristic : public TwoLayerCrossMin {

/*{\Mdefinition
The class |\Mname| implements the two layer crossing minimization heuristic
{\it Median}. It reorders the nodes on the changeable layer according
to the median weight~\cite{EW86}
\[ Median(v) = \left\{\begin{array}{cl}
\frac{\D pos(w_{\lfloor\frac{n}{2}\rfloor})+pos(w_{\lceil\frac{n}{2}\rceil})}{\D2} & \mbox{ if }n\geq1\\
0.0 & \mbox{ otherwise}
\end{array}\right. \]
where $v$ is a node on the changeable layer, and $v_1,\ldots,v_n$
is the sequence of neighbors of $v$ on the fixed layer ordered
according to the position |pos|.
}*/

public:

	/*{\Mcreation}*/

	MedianHeuristic() { }
	/*{\Mcreate  creates an instance |\Mvar| of type |\Mname|.}*/

	~MedianHeuristic() { }


	/*{\Moperations}*/

	void call (const leda_graph& G, Layer& L);
	/*{\Mop calls |\Mvar| for graph $G$ and layer $L$. Reorders the nodes
	on layer $L$.}*/

	void init (const leda_graph& G, const Hierarchy& /*H*/) {
		weight.init(G);
	}

	void cleanup () {
		weight.init();
	}

	AGDModule *clone() const {
		return new MedianHeuristic();
	}

private:

	leda_node_array<double> weight;
};


#endif
