
#line 3 "OrthoShape.lw"
#ifndef ORTHOSHAPE_H
#define ORTHOSHAPE_H

#include <AGD/PlaneGraphCopy.h>

#include <AGD/GridLayoutModule.h>

class ostream;

/*{\Manpage {OrthoShape} {} {Orthogonal Shape Description} {H} }*/

class OrthoShape {

  /*{\Mdefinition
  The class |\Mname| serves as \ldots. An instance |\Mvar| \ldots
  }*/

private:

  enum direction { N, E, S, W, unexplored };
  inline direction opp(direction d) { return (direction)((d+6) % 4); }
  inline direction next(direction d) { return (direction)((d+5) % 4); }
  inline direction prev(direction d) { return (direction)((d+3) % 4); }
 
  PlaneGraphCopy* _G; 
  leda_face _f_0;
  leda_edge_map<int> _a;
  leda_edge_map<leda_string> _s;

  leda_edge_map<int> _zeroes;
  leda_edge_map<direction> _dir;
  int _number_of_bends;

public:

  /*{\Mcreation}*/

  OrthoShape::OrthoShape() { init(); }

  //OrthoShape(leda_graph& G);
  /*{\Mcreate creates an instance |\Mvar| of type |\Mname|
  }*/
  
  OrthoShape(PlaneGraphCopy& G, leda_face f_0);
  /*{\Mcreate creates an instance |\Mvar| of type |\Mname| and 
     initializes the underlying graph with |G|. The external face
     is set to |f_0|.
  }*/

  OrthoShape(leda_graph& G, leda_face f_0);
  /*{\Mcreate \ldots}*/

  ~OrthoShape() { }


  /*{\Moperations}*/

  //void init();

  void init();
  
  bool check(leda_string& msg) const;
  /*{\Mop tests if |\Mvar| is a {\em legal} orthogonal shape desription,
     i.e., if there is an orthogonal grid embedding which realizes the
     given shape. If the test fails, |msg| contains the reason.
  }*/

  void print_edge(leda_edge e) const { _G->print_edge(e); }

  void print_node(leda_node e) const { _G->print_node(e); }

  void print_face(leda_face e) const { _G->print_face(e); }

  leda_edge first_edge() const { return _G->first_edge(); }
  
  leda_edge succ_edge(leda_edge f) const { return _G->succ_edge(f); }

  leda_edge face_cycle_succ(leda_edge f) const { 
    return _G->face_cycle_succ(f); 
  }
	
  leda_face first_face() const { return _G->first_face(); }
  
  leda_face succ_face(leda_face f) const { return _G->succ_face(f); }

  leda_node first_node() const { return _G->first_node(); }
  
  leda_node succ_node(leda_node f) const { return _G->succ_node(f); }

  int size(leda_face f) const { return (_G->adj_edges(f)).size(); }

  leda_edge reversal(leda_edge e) const { return _G->reversal(e); }

  leda_edge original(leda_edge e) const { return _G->original(e); }

  leda_face face_of(leda_edge e) const { return _G->face_of(e); }

  int number_of_nodes() const { return _G->number_of_nodes(); }

  int number_of_edges() const { return _G->number_of_edges(); }

  int number_of_faces() const { return _G->number_of_faces(); }

  leda_face& get_f_0() { return _f_0; }

  leda_face get_const_f_0() const { return _f_0; }

  leda_graph& get_graph() { return *_G; }
  /*{\Mop \ldots}*/

  leda_graph &get_const_graph() const { return *_G; }

  leda_node original(leda_node v) const { return _G->original(v); }

  leda_face expand(leda_node v) { return _G->expand(v); }

  leda_node collapse(leda_face f, leda_node w) { return _G->collapse(f, w); }

  leda_edge split(leda_edge e);

  bool north(leda_edge e) const { return (_dir[e] == N); }

  bool west(leda_edge e) const { return (_dir[e] == W); }

  int get_angle(const leda_edge e) const { return _a[e]; }
  /*{\Mop returns the angle edge |e| forms with its successor.
  }*/

  direction get_dir(const leda_edge e) const { return _dir[e]; }

  void set_angle(const leda_edge e, int a) { _a[e] = a; }
  /*{\Mop \ldots}*/

  void set_string(const leda_edge e, const char c, int n);
  /*{\Mop \ldots}*/

  leda_string get_string(const leda_edge e) const { return _s[e]; }
  /*{\Mop \ldots}*/

  leda_edge_map<int> get_angles() const { return _a; }

  leda_edge_map<leda_string> get_strings() const { return _s; }

  void add_bend(const leda_edge e);

  int number_of_bends() const { return _number_of_bends; }
  /*{\Mop returns the number of bends}*/

  void normalize();

  void dissect();

  void orientate();
  void orientate(leda_edge e, direction d);

  void assign_coords(GridLayout& L);

  LEDA_MEMORY(OrthoShape)

protected:

  int ones(const leda_edge e) const { return _s[e].length() - _zeroes[e]; }

};

ostream& operator<< (ostream& O, const OrthoShape& H);
/*{\Mbinopfunc writes |H| to output stream $O$.}*/

#endif

