//-------------------------------------------------------------------
//
//                      PSInterface.h
//
//    author: David Alberts
//    date:   December 1997
//-------------------------------------------------------------------


#include <AGD/LayoutInterface.h>


#ifndef _AGD_PS_INTERFACE_H
#define _AGD_PS_INTERFACE_H


/*{\Manpage {PSInterface} {} {Interface to PostScript} {A} }*/

/*{\Mbaseclasses

\begin{hierarchy}{7}{2.5}
\hdclass{0}{1.5}{5.5}{LayoutInterface}
\arhook{5.5}{2}{6.5}
\hdclass{1}{0}{6}{PSInterface}
\end{hierarchy}
}*/

class PSInterface : public LayoutInterface {

/*{\Mdefinition
  An instance |\Mvar| of type |\Mname| is the interface to the
  graphical attributes in a PostScript$^{TM}$ file. When a layout
  algorithm is called with an instance of |\Mname| as a parameter,
  then the drawing is saved to a file in PostScript$^{TM}$ format.
  The file name can be specified in the constructor of |\Mname|.
  The generated file can be used as Encapsulated PostScript$^{TM}$,
  too.
}*/

public:

  /*{\Mcreation}*/

  PSInterface(const leda_graph &G, leda_string name="unnamed.ps");
  /*{\Mcreate creates an instance |\Mvar| of type |\Mname| initialized
       to an interface to |ps|.}*/

  ~PSInterface() { }


  /*{\Moperations}*/

  double get_x(leda_node v) const { return xcoord[v]; }
  /*{\Mop returns the x-coordinate of the position of node |v|.}*/

  void set_x(leda_node v, double new_x) { xcoord[v] = new_x; }
  /*{\Mop sets the x-coordinate of the position of node |v|.}*/

  double get_y(leda_node v) const { return ycoord[v]; }
  /*{\Mop returns the y-coordinate of the position of node |v|.}*/

  void set_y(leda_node v, double new_y) { ycoord[v] = new_y; }
  /*{\Mop sets the y-coordinate of the position of node |v|.}*/

  DPolyline get_bend_points(leda_edge e) const { return bends[e]; }
  /*{\Mop returns the bend points of edge |e|.}*/

  void set_bend_points(leda_edge e, const DPolyline& l) { bends[e] = l; }
  /*{\Mop sets the bend points of edge |e|.}*/

  double get_width(leda_node v) const { return node_width[v]; }
  /*{\Mop returns the width of the graphics of node |v|.}*/

  void set_width(leda_node v, double new_w) { node_width[v] = new_w; }
  /*{\Mop sets the width of the graphics of node |v|.}*/

  double get_height(leda_node v) const { return node_height[v]; }
  /*{\Mop returns the height of the graphics of node |v|.}*/

  void set_height(leda_node v, double new_h) { node_height[v] = new_h; }
  /*{\Mop sets the height of the graphics of node |v|.}*/

  DPoint get_source_anchor(leda_edge e) const { return sanch[e]; }
  /*{\Mop returns the source anchor position of edge |e|.}*/

  void set_source_anchor(leda_edge e, const DPoint& pos) { sanch[e] = pos; }
  /*{\Mop sets the source anchor position of edge |e| to |pos|.}*/

  DPoint get_target_anchor(leda_edge e) const { return tanch[e]; }
  /*{\Mop returns the target anchor position of edge |e|.}*/

  void set_target_anchor(leda_edge e, const DPoint& pos) { tanch[e] = pos; }
  /*{\Mop sets the target anchor position of edge |e| to |pos|.}*/

  void init(const leda_graph &G, leda_string name);
  void update();

private:

  void _init(const leda_graph &G, const leda_string &name);

  leda_string              filename;
  leda_node_map<double>    xcoord;      // node coordinates
  leda_node_map<double>    ycoord;
  leda_node_map<double>    node_width;
  leda_node_map<double>    node_height; 
  leda_edge_map<DPolyline> bends;       // edge bends
  leda_edge_map<DPoint>    sanch;       // source and target anchors
  leda_edge_map<DPoint>    tanch;

};

#endif
