//-------------------------------------------------------------------
//
//                        RankAssignment.h
//
//
//    author: Carsten Gutwenger
//    date:   february 1998
//-------------------------------------------------------------------


#ifndef _AGD_RANK_ASSIGNEMENT_H
#define _AGD_RANK_ASSIGNEMENT_H

#include <AGD/AGDModule.h>


/*{\Manpage {RankAssignment} {} {RankAssignment Modules} {R} }*/

/*{\Mbaseclasses

\begin{hierarchy}{6.5}{2.5}
\hdclass{0}{1.5}{5.5}{AGDModule}
\arhook{5.5}{2}{6.5}
\hdclass{1.5}{0}{5.5}{RankAssignment}
\end{hierarchy}
}*/

class RankAssignment : public AGDModule {

/*{\Mdefinition
Modules of type |\Mname| partition the nodes of a graph $G$ into
levels. Each node $v$ is assigned a non-negative rank $i$, what
means that $v$ belongs to the $i$th level. 

Such a ranking can
be used to build a node hierarchy of $G$. The hierarchical
drawing method by Sugiyama builds a node hierarchy in a first
step and computes a layout that represents the hierarchy, i.e.,
all nodes on the same level have the same y-coordinate.

\headerline{Input and Output Parameters}

\begin{tabular}{lcc}
input parameter: & graph & $|in_graph|=1$ \\
output parameter: & ranking & $|out_ranking|=1$
\end{tabular}

The integer constants |in_graph| and |out_ranking| can be used to denote
the positions of the input and output parameters.
}*/

public:

	// the input and output parameters of a rank assignment module
	static const int in_graph;
	static const int out_ranking;

	/*{\Minitialization}*/

	RankAssignment () : AGDModule(1,1) { }

	/*{\Mcreate initializes a rank assignment module.}*/

	virtual ~RankAssignment () { }


	/*{\Moperations 1.2 3}*/

	bool check (const leda_graph& G, leda_string& message);
	/*{\Mop returns |true| if $G$ satisfies the precondition of |\Mvar|.
	Otherwise, |false| is returned, and |message| contains the
	error message.}*/

	virtual void call (const leda_graph& G, leda_node_array<int>& rank) = 0;
	/*{\Moppure implements the module call. Returns the rank for each
	node $v\in G$ in |rank[v]|.}*/
};


#endif
