//-------------------------------------------------------------------
//
//                        CompOrder2.h
//
//    author: Carsten Gutwenger
//    date:   february 1997
//-------------------------------------------------------------------


#ifndef _AGD_IMPL_COMP_ORDER_2_H
#define _AGD_IMPL_COMP_ORDER_2_H


#include <stdio.h>
#include <LEDA/face_array.h>
#include <AGD/CanonicalOrder.h>


//
// auxiliary structures
//
// pair of face f and list_item it
//

struct face_item {
	face_item ()
		{ f = nil; it = nil; }
	face_item (face f_ini, list_item it_ini)
		{ f = f_ini; it = it_ini; }
	face f;
	list_item it;
};

ostream& operator<< (ostream& out, face_item);
istream& operator>> (istream& in, face_item);

//
// pair of node v and list_item it
//

struct nd {
	nd ()
		{ }
	nd (node v_ini, list_item it_ini = NULL)
		{ v = v_ini; it = it_ini; }
	node v;
	list_item it;
};

ostream& operator<< (ostream& out, nd);
istream& operator>> (istream& in, nd);

enum NT { type_face, type_node, type_edge };

#define INIT_VAR(x,var,val) \
   var[x] = (val);               \
   set_update(x);

#define DEC_VAR(x,var)      \
   var[x] --;                    \
   set_update(x);

#define INC_VAR(x,var)      \
   var[x] ++;                    \
   set_update(x);

class CompOrder2 {
public:
	CompOrder2 (graph &G, face F, double base_ratio);

	face F_out()
		{ return Fout; }
	face left (edge e)
		{ return g->face_of(e); }
	face right (edge e)
		{ return g->face_of(g->reversal(e)); }
	face right (node v) {
		return left(next_succ[v]);
	}
	face left (node v) {
		return right(prev_pred[v]);
	}
	int virte (node v);
	node next (node v) {
		return the_next[v];
	}
	node prev (node v) {
		return the_prev[v];
	}
	bool cutv (face f)
		{ return (virt_src [f] != nil); }

	bool is_poss_face (face f) {
		return (f != F_out() && outv[f] >= 3 && outv[f] == oute[f]+1);
	}
	bool is_poss_node (node v) {  // precond.: v on C_k'
		//return (v != v1 && v != v2 && cutf[v] <= 1 && cutf[v] == virte(v) && numsf[v] == 0 && deg[v] >= 3);
		return (on_base[v] == false && cutf[v] <= 1 && cutf[v] == virte(v) && numsf[v] == 0 && deg[v] >= 3);
	}
	bool is_poss_virt (node v) {  // precond.: v on C_k'
		return virt_edge[v] && ((deg[v] == 2 && v != v_left) || (deg[next(v)] == 2 && next(v) != v_right));
	}

	bool get_possible();
	NT next_poss()
		{ return next_type; }

	void set_V1 (OrderSet &V) {
		int i;
		edge e;

		V = OrderSet (base_length, nil, nil);
		for (i = 1, e = e_left; i <= base_length; i++, e = g->face_cycle_pred(e))
			V[i] = target(e);
	}

	void init_possibles();
	void remove_next_face (OrderSet &V);
	void remove_next_node (OrderSet &V);
	void remove_next_virt (OrderSet &V);

	void do_update();

private:
	void edge_to_contour (edge e);
	void virt_to_contour (node v, node w, edge e_n_s, edge e_p_p);
	void virt_to_contour (node v, node w);

	void mark (node v, bool val);
	node get_face_cl(face f);
	void set_outv (node v);
	void set_seqp (node cl, node cr);
	void del_outer_node(node v);
	void dec_seqp(node v);
	void put_on_outer (node v, face f);
	void del_outer_ref (face f);
	void get_adj_faces (node v, list<face>& L);
	void get_adj_nodes (node v, list<node>& L);

	void set_update (face f);
	void set_update (node v);

	graph *g;
	face Fout;
	//node v1, v2;
	//edge e21;
	edge e_left, e_right;
	node v_left, v_right;
	int base_length;

	NT next_type;
	face next_f;
	node next_v;
	node next_e;

	// variables of nodes
	node_array<int>       deg;      // Grad von v in G_k'
	node_array<int>       cutf;     // Anzahl Flchen von v mit cutv(F)=true
	node_array<int>       numsf;    // Anzahl Flchen F von v mit outv(F)>seqp(F)+1
	node_array<bool>      on_outer; // true <=> v auf C_k'
	node_array<bool> on_base;

	node_array<list_item> v_link;	// Item in poss_nodes, das v enthlt, bzw. nil
	node_array<list_item> link_virt;
	node_array<bool>	v_update;
	node_array<list<face_item> > in_out_nodes;

	// variables of faces
	face_array<int>		outv,		// Anzahl Knoten von F auf C_k'
						oute,		// Anzahl Kanten von F auf C_k'
						seqp;		// Anzahl aufeinanderfolgender Paare c_i,c_i+1 von F
	face_array<node>	virt_src;	// falls F virt. Kante e enthlt, dann source(e), sonst nil
	face_array<list_item>	f_link;	// link ist das Item in possible_faces, das diese Flche enthlt,
									// falls sie in possible_faces ist, sonst nil
	face_array<bool>	f_update;
	face_array<bool>    f_mark;		// markieren bei Berechnung von seqp
	face_array<bool>    is_sf;		// true <=> outv(F) < seqp(F)+1
	face_array<list<nd> > outer_nodes;	// Liste aller Knoten diesr Face auf der Auenflche

	// represantation of the contour
	node_array<node>	the_next, the_prev;
	node_array<edge>	next_succ, prev_pred;
	node_array<bool>	virt_edge;	// virt_edge[c_i] = true <=> (c_i,c_i+1) virtuelle Kante

	// lists of possible faces, nodes and edges
	list<face>     poss_faces;    // Flchen mit outv(F)>= 3 und outv(F)=oute(F)+1
	list<node>     poss_nodes;    // Knoten mit cutf(v) <= 1, cutf(v) = virte(v), numsf(v)=0 und deg(v)>= 3
	list<node>     poss_virt;     // Knoten v bezeichne virtuelle Kante e = (v,next[v]), die erfllt:
								 // (deg[v]=2 und v!=v1) oder (deg[next[v]]=2 und next[v]!=v2)
	list<node>     update_nodes;
	list<face>     update_faces;
	list<edge>     rem_edges;
};


#endif
