//-------------------------------------------------------------------
//
//                          IOPoints.h
//
//    author: Carsten Gutwenger
//    date:   june 1997
//-------------------------------------------------------------------


#ifndef _AGD_IMPL_IOPOINTS_H
#define _AGD_IMPL_IOPOINTS_H


#include <LEDA/graph.h>


///////////////////////////////////////////////////////////
//                 auxiliary classes
///////////////////////////////////////////////////////////

class InOutPoint {
public:
	InOutPoint ()
		{ dx = dy = 0; e = NULL; }
	InOutPoint (leda_edge e1)
		{ e = e1; dx = dy = 0; }
	int  dx, dy;
	leda_edge e;
};

ostream& operator<< (ostream& out, InOutPoint);
istream& operator>> (istream& in, InOutPoint);

struct pair_v_d {
	pair_v_d() { v = NULL; d = 0; }
	pair_v_d(leda_node v_ini, int d_ini) {
		v = v_ini; d = d_ini;
	}

	leda_node v;
	int d;
};

ostream& operator<< (ostream& out, const pair_v_d);
istream& operator>> (istream& in, pair_v_d);

#if defined(_MSC_VER)
ostream& operator<< (ostream& out, const leda_list<pair_v_d>&);
istream& operator>> (istream& in, leda_list<pair_v_d>&);
int compare (const leda_list<pair_v_d>, const leda_list<pair_v_d>);
#endif


// represantation of in- and outpoint lists

class IOPoints {
private:
	leda_graph *g;
	leda_node_map<leda_list<InOutPoint> > in, out;
	leda_edge_map<InOutPoint *> the_point_of;

public:
	IOPoints () { }
	IOPoints (leda_graph& G) {
		g = &G;
		in.init (G);
		out.init (G);
		the_point_of.init (G);
	}
	~IOPoints () { }

	void init (leda_graph& G) {
		g = &G;
		in.init (G);
		out.init (G);
		the_point_of.init (G);
	}

	void clear () {
		//in = out = node_array<list<InOutPoint> >();
		 //in.init(); out.init();
		//the_point_of = edge_array<InOutPoint *>();
		 //the_point_of.init();
	}

	void append_in_point (leda_edge e) {
		leda_node v = source(e);
		the_point_of [e] = &(in[v] [in[v].append (InOutPoint(e))]);
	}
	void append_out_point (leda_edge e) {
		leda_node v = source(e);
		the_point_of [e] = &(out[v] [out[v].append (InOutPoint(e))]);
	}
	void push_in_point (leda_edge e) {
		leda_node v = source(e);
		the_point_of [e] = &(in[v] [in[v].push (InOutPoint(e))]);
	}
	void push_out_point (leda_edge e) {
		leda_node v = source(e);
		the_point_of [e] = &(out[v] [out[v].push (InOutPoint(e))]);
	}

	void set_out_coord (leda_node v, list_item it, int x, int y) {
		out[v][it].dx = x;
		out[v][it].dy = y;
	}
	void set_in_coord (leda_node v, list_item it, int x, int y) {
		in[v][it].dx = x;
		in[v][it].dy = y;
	}

	int& out_dx (leda_node v, list_item it) {
		return out[v][it].dx;
	}
	int& out_dy (leda_node v, list_item it) {
		return out[v][it].dy;
	}
	int& in_dx (leda_node v, list_item it) {
		return in[v][it].dx;
	}
	int& in_dy (leda_node v, list_item it) {
		return in[v][it].dy;
	}

	const leda_list<InOutPoint>& in_points (leda_node v) {
		return in [v];
	}
	const leda_list<InOutPoint>& out_points (leda_node v) {
		return out [v];
	}
	const InOutPoint& in_points (leda_node v, list_item it) {
		return in [v] [it];
	}
	const InOutPoint& out_points (leda_node v, list_item it) {
		return out [v] [it];
	}

	InOutPoint& point_of (leda_edge e) {
		return *(the_point_of [e]);
	}

	void change_edge (leda_node v, leda_edge e_new) {
		out[v].Pop();
		append_in_point (e_new);
	}

};

#endif
