
#include <LEDA/graph.h>
#include <AGD/CanonicalOrder.h>


class io_point {
public:
	io_point() {
		e = nil; dx = dy = 0;
	}

	io_point(edge e1) {
		e = e1; dx = dy = 0;
	}

	edge e;
	int dx, dy;
};

ostream& operator<< (ostream& os, const io_point& p);
istream& operator>> (istream& is, io_point& p);

class mm_ordering {
public:
	mm_ordering(const graph& G);

	int in(node v) const;
	int out(node v) const;

	const list<io_point>& inpoints(node v) const;
	const list<io_point>& outpoints(node v) const;

	node first_in_neighbour(node v) const;
	node last_in_neighbour(node v) const;

	edge left_out(node v) const;
	edge sec_left_out(node v) const;
	edge right_out(node v) const;
	edge sec_right_out(node v) const;

	void append_outpoint(edge e);
	void push_inpoint(edge e);

	void init (int m) {
		_V = array<OrderSet>(1,m);
	}

	const OrderSet& operator[](int i) const {
		return _V[i];
	}

	OrderSet& operator[](int i) {
		return _V[i];
	}

	node operator()(int i, int j) const {
		return (_V[i])[j];
	}

private:
	node_map<list<io_point> > _inpoints, _outpoints;
	node_map<int> _rank;
	array<OrderSet> _V;
};


inline int mm_ordering::in(node v) const
{
	return _inpoints[v].length();
}

inline int mm_ordering::out(node v) const
{
	return _outpoints[v].length();
}

inline const list<io_point>& mm_ordering::inpoints(node v) const
{
	return _inpoints[v];
}
inline const list<io_point>& mm_ordering::outpoints(node v) const
{
	return _outpoints[v];
}

inline node mm_ordering::first_in_neighbour(node v) const
{
	return target(_inpoints[v].head().e);
}

inline node mm_ordering::last_in_neighbour(node v) const
{
	return target(_inpoints[v].tail().e);
}

inline edge mm_ordering::left_out(node v) const
{
	return _outpoints[v].head().e;
}

inline edge mm_ordering::sec_left_out(node v) const
{
	const list<io_point> &L = _outpoints[v];
	return L[L.succ(L.first())].e;
}

inline edge mm_ordering::right_out(node v) const
{
	return _outpoints[v].tail().e;
}

inline edge mm_ordering::sec_right_out(node v) const
{
	const list<io_point> &L = _outpoints[v];
	return L[L.pred(L.last())].e;
}
