/*****************************************************************

            Filename    :  vbc_node.h

	    Version     :  01.1997

	    Author      :  Sebastian Leipert

	    Language    :  C++

            Purpose     :  Class for managing nodes.
	    
******************************************************************/


#ifndef _AGD_IMPL_VBC_NODE_H
#define _AGD_IMPL_VBC_NODE_H

#include <AGD/impl/TreeLayout/def.glb>
#include <iostream.h>


class vbc_node {
   friend ostream &operator<<(ostream &, const vbc_node &);
public:
   vbc_node(int = 0, double = 1.0, double = 1.0, double = 1.0, int = 1, int = 1,
        double = 0, double = 0, double = 0, double = 0,
        vbc_node * = 0,vbc_node * = 0,vbc_node * = 0,vbc_node * = 0,vbc_node * = 0);
   ~vbc_node();

   vbc_node*   get_parent() const;
   vbc_node*   get_firstchild() const;
   vbc_node*   get_leftsibling() const;
   vbc_node*   get_rightsibling() const;
   vbc_node*   get_leftneighbor() const;

   int     get_nmber() const;
   int     get_colour() const;
   int     get_underlying_colour() const;
   double  get_radius() const;
   double  get_height() const;
   double  get_width() const;
   double  get_Xcoord() const;
   double  get_Ycoord() const;
   double  get_prelim() const;
   double  get_modifier() const;
   char*   get_information();
   char*   get_shortinfo();

   void    set_parent(vbc_node *);
   void    set_firstchild(vbc_node *);
   void    set_leftsibling(vbc_node *);
   void    set_rightsibling(vbc_node *);
   void    set_leftneighbor(vbc_node *);
   void    set_nmber(int);
   void    set_colour(int);
   void    set_underlying_colour(int);
   void    set_height(double);
   void    set_width(double);
   void    set_Xcoord(double);
   void    set_Ycoord(double);
   void    set_prelim(double);
   void    set_modifier(double);
   void    set_information(char*);
   void    set_shortinfo(char*);

   void    add_information(char*);
   void    add_shortinfo(char*);

   int     is_leaf() const;
   int     has_left_sib() const;
   int     has_right_sib() const;
   int     has_child() const;
   
   int     is_displayed() const;
   void    display(int);

   int     is_highlighted() const;
   void    highlight(int);

private:

   int     nmber;
   int     displayed;
   int     highlighted;
   int     colour;
   int     underlying_colour;
   
   double  radius;
   double  height;
   double  width;
   double  x;
   double  y;
   double  prelim_x;
   double  mod;

   char    information[INFOSIZE];
   char    shortinfo[SHORTSIZE];
   
   vbc_node*   parent;
   vbc_node*   firstchild;
   vbc_node*   leftsib;
   vbc_node*   rightsib;
   vbc_node*   leftneighb;
   

};


#endif
