//-------------------------------------------------------------------
//
//                         tr_literal.h
//
//
//    author: Carsten Gutwenger
//    date:   july 1997
//-------------------------------------------------------------------



#ifndef _AGD_TR_LITERAL_H
#define _AGD_TR_LITERAL_H


#include <AGD/AGD_Key.h>



/*{\Manpage {tr_literal} {} {Tagged Rule Literals} {l} }*/

class tr_literal {

/*{\Mdefinition
An instance |\Mvar| of the data type |\Mname| is a pair $(k,i)$
of a key $k$ and an index $i$. Tagged rule literals are used
by tagged rules.
}*/

public:

	/*{\Mcreation}*/

	tr_literal () {
		key = nil, index = 1;
	}
	/*{\Mcreate creates an instances |\Mvar| of type |\Mtype|
	initialized to $(|nil|,1)$.}*/

	tr_literal (agd_key k) {
		key = k; index = 1;
	}
	/*{\Mcreate creates an instances |\Mvar| of type |\Mtype|
	initialized to $(k,1)$.}*/

	tr_literal (agd_key k, int i) {
		key = k; index = i;
	}
	/*{\Mcreate creates an instances |\Mvar| of type |\Mtype|
	initialized to $(k,i)$.}*/

	agd_key key;
	int index;


	// Input & Output (currently dummy only)

	friend istream& operator>> (istream& is, tr_literal& ) {
		return is;
	}

	friend ostream& operator<< (ostream& os, const tr_literal& l) {
		os << "(" << l.key << "," << l.index << ")";
		return os;
	}

};


#endif
