//-------------------------------------------------------------------
//
//                     CanonicalOrder.cpp
//
//    author: Carsten Gutwenger
//    date:   february 1997
//-------------------------------------------------------------------


#include <AGD/CanonicalOrder.h>
#include <AGD/impl/CompOrder.h>
#include <AGD/impl/CompOrder2.h>
#include <AGD/impl/misc.h>

#include <LEDA/b_stack.h>


int compare (const list_item&, const list_item&)
{ return 0; }


CanonicalOrder::CanonicalOrder (graph &g,
	CanonicalOrderType co_type,
	face F, bool prefer_nodes, double base_ratio)
{
	list<OrderSet> c_order;

	_g = &g;

	switch (co_type) {
	case co_triconnected:
		comp_tric_co (g, c_order, F, prefer_nodes, base_ratio);
		break;
	case co_biconnected:
		comp_bic_co (g, c_order, F, base_ratio);
		break;
	}

	int i, m = c_order.length();
	list_item it;
	V = array<OrderSet> (1, m);

	i = 1;
	forall_items (it, c_order) {
		V[i++] = c_order[it];
	}

	init_rank (g);
}


LmcOrder::LmcOrder (graph &g, CanonicalOrderType co_type,
	face F, bool prefer_nodes, double base_ratio)
{
	list<OrderSet> c_order;

	_g = &g;

	switch (co_type) {
	case co_triconnected:
		comp_tric_co (g, c_order, F, prefer_nodes, base_ratio);
		break;
	case co_biconnected:
		comp_bic_co (g, c_order, F, base_ratio);
		break;
	}

	node_array< list<OrderSet *> > cr_sets (g);
	b_stack<node>	Outerface_Stack (g.number_of_nodes());

	list_item it;
	node	  cr;
	int 	  i, j, m = c_order.length();

	V = array<OrderSet> (1, m);
	forall_items (it, c_order) {
		cr = c_order [it].right();
		if (cr != nil)
			cr_sets [cr].append (& c_order [it]);
	}

	OrderSet &V1 = c_order [c_order.first()];
	for (j = V1.len(); j >= 2; j--)
		Outerface_Stack.push ( V1[j] );

	V [1] = V1;

	i = 2;
	while (!Outerface_Stack.empty()) {
		cr = Outerface_Stack.top();
		if (cr_sets [cr].empty())
			Outerface_Stack.pop();
		else {
			V [i] = *(cr_sets [cr].pop());
			for (j = len(i); j >= 1; j--)
				Outerface_Stack.push ( (V[i])[j] );
			i++;
		}
	}

	init_rank (g);
}


void CanonicalOrder::init_rank (graph& g)
{
	int i, j;

	the_rank.init (g);

	for (i = 1; i <= length(); i++) {
		for (j = 1; j <= V[i].len(); j++) {
			the_rank [(V[i])[j]] = i;
		}
	}
}


int CanonicalOrder::get_base_chain (graph& G, face F, double base_ratio,
	edge& e_left, edge& e_right)
{
	int len;
	node_array<bool> mark;

	e_right = find_max_base_chain (G, F, len);
	len = Max (2, Min (len, (int)(base_ratio*G.size(F)+0.5)));

	//int i;

	//e_right = e_start;
	e_left = e_right;
	for (int i = 2; i < len; i++)
		e_left = G.face_cycle_succ (e_left);

	return len;
}


class QType {
public:
	QType (edge e, int i) {
		start = e;
		limit = i;
	}
	QType () {
		start = nil;
		limit = 0;
	}

	edge start;
	int  limit;
};

ostream& operator<< (ostream& O, const QType)
{
	return O;
}

istream& operator>> (istream& I, QType)
{
	return I;
}

#if defined(_MSC_VER)
ostream& operator<< (ostream&, const list<int>&);
istream& operator>> (istream&, list<int>&);
int compare (const list<int>&, const list<int>&);
#endif

edge CanonicalOrder::find_max_base_chain (graph& G,
	face F, int& length)
{
	edge e_start, e, e2;
	int p = G.size(F);

	node_array<int> num(G,-1);

	int i = 0, j, k, d;
	list_item it, it_limit;

	forall_face_edges (e, F)
		num[source(e)] = i++;

	array<list<int> > diag (0, p-1);
	forall_face_edges (e, F) {
		i = num[source(e)];
		for (e2 = G.cyclic_adj_pred(e); e2 != G.cyclic_adj_succ(e);
		e2 = G.cyclic_adj_pred(e2)) {
			j = num[target(e2)];
			if (j != -1)
				diag[i].append (j);
		}
	}

	list<QType> Q;
	array<list_item> pos_in_Q (0, p-1);
	init (pos_in_Q, (list_item)nil);

	length = 0;
	bool first_run = true;
	e = G.first_face_edge(F);
	i = num[source(e)];
	do {
		if (pos_in_Q[i] != nil) {
			e2 = Q.head().start;
			d = (i-num[source(e2)]+p) % p +1;
			if (d > length || (d == length && index(source(e2)) < index(source(e_start)))) {
				length = d;
				e_start = e2;
			}
			it_limit = pos_in_Q[i];
			do {
				it = Q.first();
				pos_in_Q[Q[it].limit] = nil;
				Q.pop();
			} while (it != it_limit);
		}

		if (diag[i].empty())
			j = (i-2+p) % p;
		else {
			int m = p;
			forall (k, diag[i]) {
				d = (k-i+p)%p;
				if (d < m) {
					m = d;
					j = k;
				}
			}
			j = (j-1+p) % p;
			if (!first_run) {
				pos_in_Q[Q[Q.last()].limit] = nil;
				Q[Q.last()].limit = j;
				pos_in_Q[j] = Q.last();
			}
		}

		if (first_run)
			pos_in_Q[j] = Q.append(QType(e,j));

		e = G.face_cycle_succ(e);
		i = num[source(e)];
		if (i == 0) first_run = false;
	} while (!Q.empty());

	return e_start;
}


///////////////////////////////////////////////////////////////////////////////////////////////////
// Berechnung der kanonischen Ordnung fr dreizusammenhngende Graphen (Kant)

void CanonicalOrder::comp_tric_co (graph &G, list<OrderSet> &c_order,
	face F_out, bool prefer_nodes, double base_ratio)
{
	node_array<edge>	e_pred (G), // Fr Auenflche c_1,...,c_q ist e_pred[c_i] = (c_i,c_i-1)
						e_succ (G); //   e_succ[c_i] = (c_i,c_i+1)

	edge e_left, e_right;			// definiert Grundseite
	int base_length, i;

	node		vk, u, cl, cr;
	edge		e, e2;
	face		f, Fk;
	OrderSet	V(1);

	// Definition eines Objektes der Hifsklasse CompOrder (Aufruf des Konstruktors)
	CompOrder	cpo (G, prefer_nodes);

	base_length = get_base_chain (G, F_out, base_ratio, e_left, e_right);

	// Initialisierung von e_pred und e_succ fr Auenflche von G
	for (e = G.face_cycle_succ (e_left); e != e_right; e = G.face_cycle_succ(e)) {
		e_succ [source(e)] = e;
		e_pred [target(e)] = G.reversal(e);
	}
	e_pred [target(e_left)] = e_succ [source(e_right)] = nil;

	// Aktualisiere outv, oute
	cpo.init_outer_nodes (F_out, e_left, e_right);
	cpo.init_outer_edges (F_out);

	// einziger mglicher Knoten ist anfangs vn
	int l = (G.size(F_out) - base_length + 1) / 2;
	for (e = e_left, i = 1; i <= l; i++)
		e = G.face_cycle_succ(e);
	cpo.init_possible (target(e));

	// For k := K downto 2
	while (cpo.is_possible()) {
		// Whle nchsten mglichen Knoten oder Flche
		cpo.get_next_possible (vk, Fk);
		if (cpo.is_node ()) {  // Falls Knoten
			V = OrderSet(1,e_pred [vk],e_succ [vk]);
			//V = OrderSet (1);  // Erzeuge V-Menge mit einem Element {vk}
			//V.left  (cl = target (e_pred [vk]));   // left- und rightvertex
			//V.right (cr = target (e_succ [vk]));
			V [1] = vk;
			c_order.push (V);   // Fge die neue Menge am Anfang der c_order-Liste ein
			cl = target (e_pred [vk]);
			cr = target (e_succ [vk]);
		} else {  // Falls Flche
			V = OrderSet (cpo.get_outv(Fk)-2);  // Erzeuge V-Menge {z1,...,zl} mit l = outv[Fk]-2 (cl und cr bleiben)
			// Bestimme Anfang und Ende der Kette (cl,cr) :
			// Suche bel. Knoten mit Grad 2 von Fk auf Ck und wandere nach links, solange Grad = 2
			cl = cpo.get_outer_deg_2 (Fk, G, e_pred, e_succ);
			while (cl != target(e_left) && e_pred[cl] == G.cyclic_adj_pred(e_succ[cl]))
				cl = target(e_pred[cl]);
			cr = target(e_succ[cl]);
			int i = 1;
			// Wandere von cl ber Knoten vom Grad 2 nach rechts bis zu Knoten mit Grad > 2 (= cr)
			while (cr != source(e_right) && e_pred[cr] == G.cyclic_adj_pred(e_succ[cr])) {
				V [i++] = cr;        // trage Knoten (mit Grad 2) in V_Menge ein
				cr = target(e_succ[cr]);
			}
			// Eniedrige sepf fr die beiden Knoten cl, cr, die auf der Auenflche bleiben (da Fk Sep.-Flche war)
			cpo.dec_sepf (cl); cpo.dec_sepf (cr);
			V.left  (cl);
			V.right (cr);
			V.left_edge(G.reversal(e_succ[cl]));
			V.right_edge(G.reversal(e_pred[cr]));
			//V = OrderSet (cpo.get_outv(Fk)-2,G.reversal(e_succ[cl]),G.reversal(e_pred[cr]));
			c_order.push (V);   // Fge die neue Menge am Anfang der c_order-Liste ein
		}
		// Aktualisiere e_succ[cl] und e_pred[cr]
		e_succ [cl] = G.cyclic_adj_pred (e_succ [cl]);
		e_pred [cr] = G.cyclic_adj_succ (e_pred [cr]);
		// Durchlaufe alle Knoten u auf C_k-1 zwischen cl und cr
		cpo.inc_oute (G.face_of(e_pred [cr]));	// (Diese Kante wird in unterer Schleife nicht betrachtet)
		cpo.inc_visited (cl);  // da cl und cr Nachbar in Vk haben, erhhe visited
		cpo.inc_visited (cr);
		e = e_succ [cl];
		for (u = target (e); u != cr; u = target (e)) {
			// Fr neue Kante auf Auenflche erhhe oute
			cpo.inc_oute (G.face_of(G.reversal(e)));

			e_pred [u] = G.reversal (e);
			e = G.face_cycle_succ (e);

			if (target (e) == vk) {   // Falls Kante von vk nach u
				cpo.inc_visited (u);     // dann erhhe visited[u]
				e = G.cyclic_adj_pred (e);
			}
			e_succ [u] = e;

			for (e2 = e_pred[u]; e2 != e_succ[u]; e2 = G.cyclic_adj_succ (e2)) {
				// aktualisiere outv[f], outer_nodes[f], sepf[v]
				f = G.face_of(e2);
				cpo.add_outer_node (u, f);
			}

		}
		if (!cpo.is_node())
			// Falls (cl,cr) in G ist, dann F' sei andere Flche von (cl,cr)
			// Teste ob (cl,cr) einzige Kante von F' auf der Auenflche ist
			if (target (e_succ [cl]) == cr &&
				cpo.is_only_edge (G.face_of(G.reversal(e_succ [cl]))))
			{
				cpo.dec_sepf (cl); cpo.dec_sepf (cr);  // dann erniedrige sepf[cl] und sepf[cr] nochmals
			}
		cpo.do_update ();  // Fhre Update durch
	}
	V = OrderSet (base_length);   // Trage noch Menge V1 = {v1,v2} nach
	for (e = e_left, i = 1; i <= base_length; e = G.face_cycle_pred(e), i++) {
		V[i] = target(e);
	}
	c_order.push (V);
}


///////////////////////////////////////////////////////////////////////////////////////////////////
// Berechnung der kanonischen Ordnung fr zweizusammenhngende Graphen

void CanonicalOrder::comp_bic_co (graph &G, list<OrderSet> &c_order,
	face F_out, double base_ratio)
{
	CompOrder2 cpo(G, F_out, base_ratio);

	cpo.init_possibles();

	while (cpo.get_possible())
	{
		switch (cpo.next_poss()) {
		case type_face:
			c_order.push (OrderSet());
			cpo.remove_next_face (c_order[c_order.first()]);
			break;
		case type_node:
			c_order.push (OrderSet());
			cpo.remove_next_node (c_order[c_order.first()]);
			break;
		case type_edge:
			c_order.push (OrderSet());
			cpo.remove_next_virt (c_order[c_order.first()]);
			break;
		}

		cpo.do_update();
	}
	c_order.push (OrderSet(2));
	cpo.set_V1(c_order[c_order.first()]);
}


face CanonicalOrder::max_face (graph& G)
{
	int max = 0;
	face f, f_max = nil;

	forall_faces (f, G) {
		if (G.size(f) >= max) {
			max = G.size(f);
			f_max = f;
		}
	}
	return f_max;
}


void CanonicalOrder::push (int k, node v, node tgt)
{
	OrderSet W (len(k)+1, V[k].left_edge(), V[k].right_edge());

	for (int i = 1; i <= len(k); i++)
		W [i] = (V[k])[i];
	W [len(k)+1] = v;
	W.left (left(k));
	W.right (right(k));

	V[k] = W;
	the_rank[v] = k;

	node last = W[len(k)-1];
	OrderSet &Vi = V[rank(tgt)];
	if (Vi.left() == last)
		Vi.left(v);
	if (Vi.right() == last)
		Vi.right(v);
}
