//-------------------------------------------------------------------
//
//                        CompOrder2.cpp
//
//    author: Carsten Gutwenger
//    date:   february 1997
//-------------------------------------------------------------------


#include <AGD/impl/CompOrder2.h>


void CompOrder2::edge_to_contour (edge e)
{
	node v = source(e), w = target(e);

	the_next  [v] = w;
	the_prev  [w] = v;
	next_succ [v] = g->cyclic_adj_pred(e);
	prev_pred [w] = g->cyclic_adj_succ(g->reversal(e));
	virt_edge [v] = false;
}

void CompOrder2::virt_to_contour (node v, node w, edge e_n_s, edge e_p_p)
{
	the_next  [v] = w;
	the_prev  [w] = v;
	next_succ [v] = e_n_s;
	prev_pred [w] = e_p_p;
	virt_edge [v] = true;
}

void CompOrder2::virt_to_contour (node v, node w)
{
	the_next  [v] = w;
	the_prev  [w] = v;
	virt_edge [v] = true;
}

void CompOrder2::put_on_outer (node v, face f)
{
	list_item it;

	it = outer_nodes[f].append (nd(v));
	(outer_nodes[f])[it].it = in_out_nodes[v].append(face_item(f, it));
}

void CompOrder2::del_outer_ref (face f)
{
	list<nd> &L = outer_nodes[f];
	nd x;

	while (!L.empty()) {
		x = L.pop();
		in_out_nodes[x.v].del_item(x.it);
	}
}

int CompOrder2::virte (node v) {
	int num = 0;

	if (on_outer[v] == true) {
		if (virt_edge[v] == true) num++;
		if (v != v_left && virt_edge[prev(v)] == true) num++;
	}
	return num;
}

CompOrder2::CompOrder2 (graph &G, face F, double base_ratio)
{
	node v, w;
	edge e, e2;
	face f;

	g = &G;
	v_link  .init(G, nil);
	link_virt.init (G, nil);

	//G.compute_faces();
	Fout = F;

	base_length = CanonicalOrder::get_base_chain (G, Fout, base_ratio, e_left, e_right);

	deg      	.init (G);
	cutf     	.init (G, 0);
	numsf    	.init (G, 0);
	on_outer 	.init (G, false);
	the_next    .init (G);
	the_prev    .init (G);
	next_succ   .init (G);
	prev_pred   .init (G);
	virt_edge	.init (G, false);
	v_update   	.init (G, false);
	in_out_nodes.init (G);
	outv        .init (G, 0);
	oute        .init (G, 0);
	seqp        .init (G, 0);
	virt_src    .init (G, nil);
	f_link      .init (G, nil);
	f_update    .init (G, false);
	f_mark      .init (G, false);
	is_sf       .init (G, false);
	outer_nodes .init (G);
	on_base     .init (G, false);

	forall_nodes(v, G)
		deg[v] = outdeg(v);

	// Initialisiere on_base[v]
	for (e = e_right; e != e_left; e = g->face_cycle_succ(e))
	  on_base [source(e)] = true;
	on_base [source(e_left)] = on_base [target(e_left)] = true;

	v_left = target(e_left);
	v_right = source(e_right);

	bool first = true;

	e = e_left;
	while (first || e != e_right) {
		v = target(e);
		forall_out_edges (e2, v) {
			f = g->face_of(e2);
			if (f != Fout) {
				outv [f] ++;
				put_on_outer (v, f);
			}
		}
		e = g->face_cycle_succ(e);
		first = false;
	}

	forall_face_edges (e, Fout)
		if (on_base[source(e)] == false || on_base[target(e)] == false) {
			oute [g->face_of(g->reversal(e))] ++;
		}

/*	forall_face_edges (e, F_out()) {
		if (e != e21) oute [right(e)]++;
		v = G.source(e);
		forall_out_edges (e2, v) {
			f = left(e2);
			if (f != Fout) {
				outv [f]++;
				put_on_outer (v, f);
			}
		}
	}
*/
	on_outer [v_left] = true;
	prev_pred[v_left] = next_succ[v_right] = nil;
	the_prev[v_left] = the_next[v_right] = nil;
	for (e = G.face_cycle_succ(e_left); e != e_right; e = G.face_cycle_succ(e)) {
		v = source(e); w = target(e);
		on_outer [w] = true;
		edge_to_contour (e);
		mark (v, true);
		forall_out_edges (e2, w) {
			f = left(e2);
			if (f_mark [f] == true) seqp[f]++;
		}
		mark (v, false);
	}

	//set_seqp (v1, v2);
	//update_nodes.clear();

	for (v = v_left; v != nil; v = next(v))
		forall_out_edges (e, v) {
			f = left(e);
			if ((is_sf [f] = (outv[f] > seqp[f]+1)) == true) numsf[v]++;
		}
}

void CompOrder2::init_possibles()
{
	node v;
	face f;

	forall_faces (f, (*g)) {
		if (is_poss_face(f))
			f_link [f] = poss_faces.append(f);
	}
	for (v = next(v_left); v != v_right; v = next(v))
		if (is_poss_node(v))
			v_link [v] = poss_nodes.append(v);
}

bool CompOrder2::get_possible()
{
	if (!poss_faces.empty()) {
		next_type = type_face;
		next_f = poss_faces.pop();
		return true;
	} else if (!poss_nodes.empty()) {
		next_type = type_node;
		next_v = poss_nodes.pop();
		return true;
	} else if (!poss_virt.empty()) {
		next_type = type_edge;
		next_e = poss_virt.pop();
		link_virt[next_e] = nil;
		return true;
	} else return false;
}

void CompOrder2::mark (node v, bool val)
{
	//edge e;
	face f;
	list<face> L;

	get_adj_faces (v, L);
	forall (f, L)
	//forall_out_edges (e, v)
		//f_mark [left(e)] = val;
        f_mark [f] = val;
}

node CompOrder2::get_face_cl(face f)
{
	node v;
	edge e;

	if (cutv (f)) {
		v = virt_src [f];
	}
	else {
		//forall (e, f->edges)
		forall_face_edges (e, f)
			if (on_outer[v = source(e)] == true && deg[v] == 2) break;
	}
	while (v != v_left && deg[v] == 2) v = prev(v);
	return v;
}

void CompOrder2::get_adj_faces (node v, list<face>& L)
{
	edge e,
	  //e_start = (v != v_left) ? prev_pred[v] : g->cyclic_adj_succ(g->reversal(e21)),
	  e_start = (v != v_left) ? prev_pred[v] : g->cyclic_adj_succ(g->reversal(e_left)),
	  //e_end   = (v != v_right) ? next_succ[v] : g->cyclic_adj_pred(e21);
	  e_end   = (v != v_right) ? next_succ[v] : g->cyclic_adj_pred(e_right);

	L.clear();
	L.append (right(e_start));
	if (deg[v] >= 3) {
		for (e = e_start; e != e_end; e = g->cyclic_adj_succ(e))
			L.append (left(e));

		L.append (left(e_end));
	}
}

void CompOrder2::get_adj_nodes (node v, list<node>& L)
{
	edge e,
	  //e_start = (v != v1) ? prev_pred[v] : g->cyclic_adj_succ(g->reversal(e21)),
	  e_start = (v != v_left) ? prev_pred[v] : g->cyclic_adj_succ(g->reversal(e_left)),
	  //e_end   = (v != v2) ? next_succ[v] : g->cyclic_adj_pred(e21);
	  e_end   = (v != v_right) ? next_succ[v] : g->cyclic_adj_pred(e_right);

	L.clear();
	L.append ((v != v_left) ? prev(v) : source(e_left));

	if (deg[v] >= 3) {
		for (e = e_start; e != e_end; e = g->cyclic_adj_succ(e))
			L.append (target(e));
		L.append (target(e_end));
	}
	L.append ((v != v_right) ? next(v) : target(e_right));
}

void CompOrder2::remove_next_face (OrderSet &V)
{
	node cl = get_face_cl(next_f), cr, v;
	edge e;
	int i;

	// Berechnung der order2_set-Menge
	V = OrderSet (outv[next_f]-2);
	V.left (cl);
	for (i = 1, cr = next(cl); cr != v_right && deg[cr] == 2; i++, cr = next(cr))
		V [i] = cr ;
	V.right (cr);
	//V.has_left (!virt_edge[cl]);
	V.left_edge (virt_edge[cl]       ? nil : g->reversal(g->cyclic_adj_succ(next_succ[cl])));
	V.right_edge(virt_edge[prev(cr)] ? nil : g->reversal(g->cyclic_adj_pred(prev_pred[cr])));
	//V.has_right(!virt_edge[prev(cr)]);

	if (cutv(next_f) && next(virt_src [next_f]) == cr)
		set_update(cr);

	// aktualisieren der Knoten und Flchen-Variablen
	if (cutv(next_f)) {
		DEC_VAR(cl,cutf)
		DEC_VAR(cr,cutf)
		v = virt_src [next_f];
		if (v != cr) {
			poss_virt.del_item(link_virt[v]);
			link_virt[v] = nil;
		}
	}
	e = next_succ[cl];
	while (true) {
		edge_to_contour(e);

		if (target(e) == cr)
			break;
		else {
			INIT_VAR(target(e),on_outer,true)
		}

		e = g->face_cycle_succ(e);
	}
	DEC_VAR (cl,deg)
	DEC_VAR (cr,deg)

	for (v = cl; v != cr; v = next(v)) {
		INC_VAR(right(v),oute)
		if (v != cl)
			set_outv (v);
	}

	set_seqp (cl, cr);

	// falls virtuelle Kante wegfllt, dann lsche diese wieder
	if (cutv(next_f)) {
		if (virt_src [next_f] == cl) {
			set_update(cl);
			virt_edge [cl] = false;
		}
		virt_src [next_f] = nil;
	}
	del_outer_ref(next_f);
}

void CompOrder2::remove_next_node (OrderSet &V)
{
	node v, w, v_virt = nil, cl, cr;
	nd u;
	edge e, e2;
	face f, f_virt = nil;
	list<node> neighb;

	cl = prev (next_v); cr = next (next_v);

	// Berechnung der order2_set-Menge
	V = OrderSet(1);
	V [1] = next_v;
	if (virt_edge[prev(next_v)] == true) {
		V.left(target(prev_pred[next_v]));
		V.left_edge(prev_pred[next_v]);
	} else {
		V.left(prev(next_v));
		V.left_edge(g->cyclic_adj_pred(prev_pred[next_v]));
	}
	//V.left  ((virt_edge[prev(next_v)] == true) ? target(prev_pred[next_v]) :
	//	prev(next_v));
	//V.left_edge (prev_pred[next_v]);
	if (virt_edge[next_v] == true) {
		V.right(target(next_succ[next_v]));
		V.right_edge(next_succ[next_v]);
	} else {
		V.right(next(next_v));
		V.right_edge(g->cyclic_adj_succ(next_succ[next_v]));
	}
	//V.right_edge(next_succ[next_v]);
	//V.right ((virt_edge[next_v] == true) ? target(next_succ[next_v]) :
	//	next(next_v));

	if (virt_edge[prev(next_v)]) {
		INIT_VAR(prev(next_v), virt_edge, false)
		v_virt = cl;
		f_virt = left(next_v);
		virt_src [f_virt] = nil;
	}
	if (virt_edge[next_v]) {
		if (link_virt[next_v] != nil) {
			poss_virt.del_item(link_virt[next_v]);
			link_virt[next_v] = nil;
		}
		v_virt = cr;
		f_virt = right(next_v);
		virt_src [f_virt] = nil;
	}

	list<face> L;
	get_adj_faces (next_v, L);
	forall (f, L) {
		outv [f]--;
	}
	list<node> L_v;
	get_adj_nodes(next_v, L_v);

	del_outer_node(next_v);
	oute [left (next_v)]--;
	oute [right(next_v)]--;
	dec_seqp (next_v);

	forall (w, L_v) {
		on_outer [w] = true;
		DEC_VAR (w, deg)
	}

	L_v.pop();
	bool first_time = true;
	forall (w, L_v) {

		if (first_time == true) {
			e2 = next_succ [prev(next_v)];
			e = prev_pred[next_v];
			first_time = false;
		} else {
			e2 = g->face_cycle_succ (e);
			e = g->cyclic_adj_succ(e);
		}

		while (true) {
			v = target(e2);

			if (v != w && on_outer[v] == true) {
				f = left(e2);

				// neue virtuelle Kante einfgen
				virt_to_contour(source(e2), w, e2, (w == next(next_v)) ?
					prev_pred[w] : g->cyclic_adj_succ(g->reversal(e)));

				set_update(f);

				INC_VAR(source(e2),deg)
				INC_VAR(w,deg)

				if (f != f_virt) {
					forall(u, outer_nodes[f]) {
						INC_VAR(u.v, cutf);
					}
				}
				virt_src [f] = source(e2);

				break;
			}

			edge_to_contour(e2);

			if (v == w) {
				del_outer_ref (left(e2));
				break;
			}
			INIT_VAR(v,on_outer,true)
			if (source(e2) == cl) {
				list_item it;
				list<nd> &L = outer_nodes[left(e2)];
				forall_items (it, L)
					if (L[it].v == cl) {
						in_out_nodes[cl].del_item (L[it].it);
						L.del_item(it);
						break;
					}
				outv [left(e2)]--;
			}
			e2 = g->face_cycle_succ(e2);
		}

	}

	for (v = cl; v != cr; v = next(v)) {
		INC_VAR(right(v),oute)
		if (v != cl)
			set_outv (v);
	}

	set_seqp (cl,cr);

	//if (v_virt != nil && cutv(f_virt)) {
	if (v_virt != nil && virt_src [f_virt] == nil) {
		DEC_VAR(v_virt,cutf)
	}
}

void CompOrder2::dec_seqp(node v)
{
	face f;
	list<face> L;

	mark(v,true);
	get_adj_faces (next(v), L);
	forall (f, L) {
		if (f_mark [f] == true)
			seqp [f]--;
	}
	get_adj_faces (prev(v), L);
	forall_rev (f, L) {
		if (f_mark [f] == true)
			seqp [f]--;
	}

	mark(v, false);
}

void CompOrder2::del_outer_node(node v)
{
	face_item x;

	forall (x, in_out_nodes[v])
		(outer_nodes[x.f]).del_item (x.it);
}

void CompOrder2::set_outv (node v)
{
	face f;
	list<face> L;

	get_adj_faces (v, L);
	forall (f, L) {
		INC_VAR(f,outv)
		put_on_outer (v, f);
		if (cutv(f) == true) {
			INC_VAR(v, cutf)
		}
		if (is_sf [f]) {
			INC_VAR(v, numsf)
		}
	}
	return;
}

void CompOrder2::set_seqp (node cl, node cr)
{
	node v, w;
	face f;
	list<face> L;

	for (v = cl; v != cr; v = w) {
		w = next(v);
		mark (v, true);
		get_adj_faces (w, L);
		forall (f, L) {
			if (f_mark [f] == true) {
				INC_VAR (f,seqp)
			}
		}
		mark (v, false);
	}
}

void CompOrder2::remove_next_virt (OrderSet &V)
{
	node v, cl = next_e, cr = next(next_e);
	face f;
	int i = 0;

	while (deg[cl] == 2 && cl != v_left)
		{ cl = prev(cl); i++; }
	while (deg[cr] == 2 && cr != v_right)
		{ cr = next(cr); i++; }

	//V = OrderSet (i, !virt_edge[cl], !virt_edge[prev(cr)]);
	V = OrderSet (i, virt_edge[cl] ? nil : prev_pred[next(cl)],
		virt_edge[prev(cr)] ? nil : next_succ[prev(cr)]);
	for (i = 1, v = next(cl); v != cr; v = next(v)) {
		V [i++] = v;
		del_outer_node(v);
	}
	V.left (cl);
	V.right (cr);

	f = right(cl);
	virt_src [f] = cl;

	virt_to_contour(cl, cr);

	INIT_VAR(f,outv,(outv[f] - V.len()))
	INIT_VAR(f,oute,(oute[f] - V.len()))
	INIT_VAR(f,seqp,(seqp [f] - V.len()-1))
	set_seqp(cl,cr);
	set_update(cl);
	set_update(cr);
}

void CompOrder2::set_update (node v)
{
	if (v_update [v] == false) {
		update_nodes.append (v);
		v_update [v] = true;
	}
}

void CompOrder2::set_update (face f)
{
	if (f_update [f] == false) {
		update_faces.append (f);
		f_update [f] = true;
	}
}

void CompOrder2::do_update()
{
	bool possible, sf;
	face f;
	node v;
	nd u;

	while (!update_faces.empty()) {
		f = update_faces.pop();
		f_update [f] = false;
		sf = (outv[f] > seqp[f]+1);
		if (sf != is_sf [f]) {
			forall (u, outer_nodes[f])
				if (sf) {
					INC_VAR(u.v,numsf)
				} else {
					DEC_VAR(u.v,numsf)
				}
			is_sf [f] = sf;
		}
		possible = is_poss_face(f);
		if (possible && f_link [f] == nil)
			f_link [f] = poss_faces.append(f);
		else if (!possible && f_link [f] != nil) {
			poss_faces.del_item(f_link [f]);
			f_link [f] = nil;
		}
	}

	list_item it, it_prev;
	for (it = update_nodes.last(); it != nil; it = it_prev) {
		it_prev = update_nodes.pred(it);
		v = update_nodes[it];
		if (v != v_left && virt_edge[prev(v)] == true) set_update(prev(v));
	}

	while (!update_nodes.empty()) {
		v = update_nodes.pop();
		v_update [v] = false;
		possible = is_poss_node(v);
		if (possible && v_link [v] == nil)
			v_link [v] = poss_nodes.append(v);
		else if (!possible && v_link [v] != nil) {
			poss_nodes.del_item (v_link [v]);
			v_link [v] = nil;
		}
		possible = is_poss_virt(v);
		if (possible && link_virt[v] == nil)
			link_virt[v] = poss_virt.append(v);
		else if (!possible && link_virt[v] != nil) {
			poss_virt.del_item (link_virt[v]);
			link_virt[v] = nil;
		}
	}
}


//
// I/O Operators for struct nd as required by LEDA 3.4 (dummy only)
//


ostream& operator<< (ostream& out, nd)
{
	return out;
}

istream& operator>> (istream& in, nd)
{
	return in;
}


//
// I/O Operators for struct face_item as required by LEDA 3.4 (dummy only)
//


ostream& operator<< (ostream& out, face_item)
{
	return out;
}

istream& operator>> (istream& in, face_item)
{
	return in;
}
