
#include <AGD/Ortho.h>
#include <AGD/compactor.h>
#include <AGD/impl/check_embedding.h>
#include <AGD/PlaneGraphCopy.h>
#include <LEDA/face_map.h>
#include <LEDA/stack.h>
#include <LEDA/node_partition.h>


void random_4_planar_graph(graph& G, int n, int m, bool biconnected) {
  
  grid_graph(G, int(ceil(sqrt(n))));
  while (G.number_of_nodes() > n) G.del_node(G.first_node());

  m = Max(m, n-1);

  list<edge> L = G.all_edges();
  L.permute();

  list_item it; forall_items(it, L) {
    if (G.number_of_edges() <= m) break;
    else {
      G.del_edge(L[it]);
      if (!Is_Connected(G) || (!Is_Biconnected(G) && biconnected))
        G.new_edge(source(L[it]), target(L[it])); 
    }
  }
}


Ortho::Ortho() : GridLayoutModule(true), Ovid() {

  _allow_bends = true;
  _allow_nonpos_deg = false;
  _allow_neg_deg = false;
  _forbid_small_zero_deg = true;
  _forbid_cross_zero_deg = true;
  _restrict_to_L_shape = true;
  _check = true; _debug = false;
  _contains_cages = false; _doorley = false;
  _dissect = true; _optimally = false;

  _output_level = 10;

  add_precondition(key::planar); 
  add_precondition(key::simple);
  add_precondition(key::no_self_loops);
  add_precondition(key::four_graph);
  add_precondition(key::connected);

  add_post_rule(key::no_crossings);
  add_post_rule(key::orthogonal);
  add_post_rule(key::planar_layout, 0);

}



AGDModule* Ortho::clone() const {
  Ortho *O = new Ortho;

  O->_allow_bends = _allow_bends;
  O->_allow_nonpos_deg = _allow_nonpos_deg;
  O->_allow_neg_deg = _allow_neg_deg;
  O->_forbid_small_zero_deg = _forbid_small_zero_deg;
  O->_forbid_cross_zero_deg = _forbid_cross_zero_deg;
  O->_restrict_to_L_shape = _restrict_to_L_shape;
  O->_check = _check; O->_debug = _debug;
  O->_contains_cages = _contains_cages;
  O->_doorley = _doorley;
  O->_dissect = _dissect; O->_optimally = _optimally;
 
  return O;
}


void Ortho::_call_grid(PlaneGraphCopy& G, face f_0, GridLayout& L) {

  if (G.number_of_nodes() <= 1) return; // degenerate cases
  LINE();

  if (!f_0) f_0 = G.max_face();
  
  OrthoShape H(G, f_0); 

  compute_optimal_shape(H);

  H.normalize();

  string msg; if(!H.check(msg)) error_handler(1, msg);

  if (_dissect) H.dissect();

  if(!H.check(msg)) error_handler(1, msg);

  H.orientate();

  if (_output_level > 3) cout << H << endl;

  if (!H.check(msg)) error_handler(1, msg); 

  compact_one_dim(H, L.x(), true);
  compact_one_dim(H, L.y(), false);

  // check the embedding here

  LINE();
}

