
#line 256 "Ovid.lw"
#include <AGD/Ovid.h>

#include <AGD/PlaneGraphCopy.h>
#include <LEDA/face_map.h>
#include <LEDA/graph_alg.h>



#line 269 "Ovid.lw"
Ovid::Ovid() {
  _adopt = true;	
  _number_of_bends = _number_of_zero_degs = 0;

  _k = 2;

  _allow_bends = _allow_nonpos_deg = _allow_neg_deg =
    _forbid_small_zero_deg = _forbid_cross_zero_deg = 
    _restrict_to_L_shape = _check = _debug = true;

  _contains_cages = false; _doorley = false;
}


#line 290 "Ovid.lw"
void Ovid::compute_optimal_shape(OrthoShape& H) {

  _number_of_zero_degs = 0;

  H.init();

  
#line 307 "Ovid.lw"
INF("constructing the network... ");
graph N;

#line 324 "Ovid.lw"
graph G = H.get_graph(); face f_0 = H.get_f_0();	

node_map<node> V2U(H.get_graph()), U2V(N);
node_map<n_type> node_type(N);
node v, i, j; face f; edge e, a;

forall_nodes(v, H) {
  i = N.new_node();
  V2U[v] = i; U2V[i] = v;
  node_type[i] = ((outdeg(v) > 4) ? high : low);
}

face_map<node> F2U(H.get_graph()); node_map<face> U2F(N);

forall_faces(f, H) {
  j = N.new_node();
  F2U[f] = j; U2F[j] = f;
  node_type[j] = ((f == f_0) ? outer : inner);
}

#line 309 "Ovid.lw"
                             ;

#line 349 "Ovid.lw"
node_array<int> b(N, 0);

forall_nodes(i, N) switch (node_type[i]) {
  case high: 
  case low:  b[i] = pow2(_k) - outdeg(U2V[i]); break;
  case inner: b[i] = pow2(_k) - H.size(U2F[i]); break;
  case outer: b[i] = pow2(_k) - H.size(U2F[i]) - pow2(_k+1); break;
  default: error_handler(1, "fatal error 1");
}

if (_check) {
  int checksum = 0; forall_nodes(i, N) checksum += b[i];
  if (checksum != 0) error_handler(1, "supply does not equal demand");
}

#line 310 "Ovid.lw"
                         ;

#line 372 "Ovid.lw"
edge_map<a_type> arc_type(N);
edge_map<edge>   e_corr(N);

if (_allow_bends) { 
#line 384 "Ovid.lw"
forall_faces(f, H) forall_face_edges(e, f) {
  face g = H.face_of(H.reversal(e));
  if (_contains_cages && _is_cage[g]) ;
  else {
    a = N.new_edge(F2U[f], F2U[g]);
    arc_type[a] = (_is_cage[f] ? cage_bend : bend); 
    e_corr[a] = e;
  }
}

#line 375 "Ovid.lw"
                                         }

#line 398 "Ovid.lw"
forall_edges(e, H) {
  v = target(e); f = H.face_of(e);
  a = N.new_edge(V2U[v], F2U[f]);
  arc_type[a] = (_is_cage[f] ? cage_angle : angle); 
  e_corr[a] = e;
}

#line 377 "Ovid.lw"
if (_allow_nonpos_deg) { 
#line 409 "Ovid.lw"
forall_edges(e, H) {
  v = target(e); f = H.face_of(e);
  if (outdeg(v) > 4 || !_forbid_small_zero_deg) {
    a = N.new_edge(F2U[f], V2U[v]);
    arc_type[a] = neg_angle; e_corr[a] = e;
  }
}

#line 377 "Ovid.lw"
                                                        }


#line 311 "Ovid.lw"
                            ;

#line 425 "Ovid.lw"
edge_array<int> u(N), l(N, 0), c(N);

const int u_max = pow2(_k) * H.number_of_nodes(),
  u_neg_angle = (_allow_neg_deg ? pow2(_k) : 1);

forall_edges(a, N) switch(arc_type[a]) {
  case angle: u[a] = pow2(_k) - 1; c[a] = 0; break;
  case cage_angle: u[a] = 1; c[a] = 0; if (_doorley) l[a] = 1; break;
  case neg_angle: u[a] = u_neg_angle; c[a] = 1; break;
  case bend: u[a] = u_max; c[a] = 1; break;
  case cage_bend: u[a] = u_max; c[a] = 0; break;
  default: error_handler(1, "fatal error 2");
}

#line 312 "Ovid.lw"
                                               ;
DONE();

#line 296 "Ovid.lw"
                           ;
  
#line 444 "Ovid.lw"
INF("computing the minimal cost flow... ");

edge_array<int> chi(N, 0); 
bool feasible = MIN_COST_FLOW(N, l, u, c, b, chi);
if (!feasible) error_handler(1, "no feasible flow");

int cost = 0;
forall_edges(a, N) {
  cost += chi[a] * c[a];
  _number_of_bends += (arc_type[a] == bend) ? (chi[a] * c[a]) : 0;
}

cout << string("c(x) = %d ", cost);
//cout << "b(x) = " << _number_of_bends << endl;
//cout << "z(x) = " << _number_of_zero_degs << endl;

DONE();


#line 297 "Ovid.lw"
                                   ;
  
#line 469 "Ovid.lw"
INF("deriving the orthogonal representation... ");

int factor = pow2(2-_k);

forall_edges(a, N) {
  e = e_corr[a];
  switch (arc_type[a]) {
    case neg_angle: H.set_angle(e, H.get_angle(e) - chi[a] * 90 * factor); 
    break;
    case cage_angle:
    case angle:     H.set_angle(e, H.get_angle(e) + (chi[a] + 1) * 90 * factor);
    break;
    case bend:      
    case cage_bend: { 
#line 499 "Ovid.lw"
if (chi[a] > 0) {
  H.set_string(e, '0', chi[a]);
  H.set_string(H.reversal(e), '1', chi[a]);
}


#line 482 "Ovid.lw"
                                             }; break;
    default: error_handler(1, "fatal error 3");
  }
}

DONE();

#line 298 "Ovid.lw"
                               ;
}

