
#include <AGD/Quod.h>
#include <AGD/compactor.h>
#include <AGD/PlaneGraphCopy.h>
#include <LEDA/face_map.h>
#include <LEDA/stack.h>
#include <LEDA/node_partition.h>

#include <AGD/impl/check_embedding.h>


Quod::Quod() : Ortho() {

  _contains_cages = true; _doorley = false;     

  rem_precondition(key::four_graph);
  rem_precondition(key::connected);
  
}

AGDModule* Quod::clone() const {
  Quod *Q = new Quod;

   Q->_allow_bends = _allow_bends;
  Q->_allow_nonpos_deg = _allow_nonpos_deg;
  Q->_allow_neg_deg = _allow_neg_deg;
  Q->_forbid_small_zero_deg = _forbid_small_zero_deg;
  Q->_forbid_cross_zero_deg = _forbid_cross_zero_deg;
  Q->_restrict_to_L_shape = _restrict_to_L_shape;
  Q->_check = _check; Q->_debug = _debug;
  Q->_contains_cages = _contains_cages;
  Q->_doorley = _doorley;
  Q->_dissect = _dissect;

  return Q;
}

void Quod::_call_grid(PlaneGraphCopy& G, face f_0, GridLayout& L) {

  if (G.number_of_nodes() <= 1) return; // degenerate case
  LINE();

  if (!Is_Connected(G)) {
    edge e_ref; if (f_0) e_ref = G.first_face_edge(f_0);
    Make_Connected(G);
    list<edge> R; G.make_map(R); G.compute_faces();
    if (f_0) f_0 = G.face_of(e_ref);
  }

  if (!f_0) f_0 = G.max_face();
  
  
  face_map<node> lion(G);

  _is_cage.init(G, false);

  node v; forall_nodes(v, G) if (outdeg(v) > 4) {
    node l = G.original(v);
    face c = G.expand(v); _is_cage[c] = true;
    lion[c] = l;
  }

  
  OrthoShape H(G, f_0); 

  if (_check) cout << H << endl;

  compute_optimal_shape(H);

  if (_check && _output_level > 3) cout << H << endl;

  H.normalize();

  string msg; if(_check && !H.check(msg)) error_handler(1, msg);

  if (_dissect) H.dissect();

  H.orientate();
  
  if (_check && !H.check(msg)) error_handler(1, msg); 
  
  
  face f; forall_faces(f, H) if (_is_cage[f]) {
    int *dim = new int[4]; dim[0] = dim[1] = dim[2] = dim[3] = 0;
    edge e; forall_face_edges(e, f) dim[(int)H.get_dir(e)]++;
    int height = Max(dim[1], dim[3]),
      width = Max(dim[2], dim[0]);
    if (height == 1) { 
                       forall_face_edges(e, f) if (H.west(e)) break;
                       H.split(e);
 };
    if (width == 1) { 
                      forall_face_edges(e, f) if (H.north(e)) break;
                      H.split(e);

 };
    delete[] dim;
  }
;

  if (_check && _output_level > 3) cout << H << endl;

  compact_one_dim(H, L.x(), true);
  compact_one_dim(H, L.y(), false);

  
  forall_faces(f, H) if (_is_cage[f]) {

    int xmin = MAXINT, ymin = MAXINT;
    int xmax = 0, ymax = 0;
    edge e; forall_face_edges(e, f) {
      node c = source (e);
      if (L.x()[c] < xmin) xmin = L.x()[c]; if (L.x()[c] > xmax) xmax = L.x()[c];
      if (L.y()[c] < ymin) ymin = L.y()[c]; if (L.y()[c] > ymax) ymax = L.y()[c];
    }
    
    v = H.collapse(f, lion[f]);
    L.x()[v] = (xmin + xmax)/2; L.y()[v] = (ymin + ymax)/2;

    forall_out_edges(e, v) {
      node w = target(e);
      if (L.x()[v] != L.x()[w] && L.y()[v] != L.y()[w]) ++_number_of_bends;
    }
    newline;

  }


  // check the embedding here

  LINE(); 
}

