#undef _LEDA_PREFIX

#include <AGD/impl/_sort_edges_double.h>

typedef double NT;


static int sign(NT x)
{ if ( x == 0 ) return 0;
  if (x > 0 ) return 1;
  else return - 1;
}

static int compare_vectors(const NT& x1, const NT& y1, 
                           const NT& x2, const NT& y2)
{ 

  if (x1 == x2 && y1 == y2) return 0;
  if (x1 == 0 && y1 == 0) return -1;
  if (x2 == 0 && y2 == 0) return +1;
  // vectors are distinct and non-zero

  int sy1 = sign(y1);  int sy2 = sign(y2);
  int sx1 = sign(x1);  int sx2 = sign(x2);

  if ( x1*y2 - x2*y1 == 0 && sx1 == sx2 && sy1 == sy2) 
  { // vectors have same direction
    if ( x1 != 0 )
      return ( double(x2)/double(x1) >=1 ? +1 : -1 ); 
    else 
     return ( double(y2)/double(y1) >=1 ? +1 : -1 );
  } 
   
  if (sy1 == 0)
  { if (sy2 < 0)  return -1;
    return -sx1;    
  }
  if (sy1 > 0)
  { if (sy2 < 0) return -1;
    if (sy2 == 0) return sign(x2);
    if (sy2 > 0) 
    { if (sx1 != sx2) return -compare(sx1,sx2);
      assert(sx1 != 0);
      /* same quadrant */
      return compare(y1*x2,x1*y2);
    }
  }
    
  if (sy1 < 0)
  { if (sy2 >= 0) return 1;
    if (sy2 < 0) 
    { if (sx1 != sx2) return compare(sx1,sx2);
      assert(sx1 != 0);
      /* same quadrant */
      return compare(y1*x2,x1*y2);
    }
  }
  /* control never reaches this point, but of course, a 
     compiler cannot know and therefore issues a warning */
  
  assert(false);
  return 0;
}


static const edge_array<NT>* DX; // will be set in SORT_EDGES 
static const edge_array<NT>* DY; // and check_CCW...


static int cmp_edges_by_angle2(const edge& e, const edge& f)
{ 
  const edge_array<NT>& dx = *DX;
  const edge_array<NT>& dy = *DY;
  
  node v = source(e); node w = source(f);
  if ( v != w ) return compare(index(v),index(w));
     
  return compare_vectors(dx[e],dy[e],dx[f],dy[f]);
}



bool check_CCW_order_of_edges(const graph &G, 
                              const edge_array<NT>& dx, 
                              const edge_array<NT>& dy)
{ edge e;

  DX = &dx; DY = &dy;  // required to transport information 
                       // about dx and dy to cmp_edges_by_angle

  forall_edges(e,G)
  { edge f = G.adj_succ(e); 
    if ( f != nil && cmp_edges_by_angle2(e,f) > 0 ) 
      return false;        
  }
  return true;
}


bool SORT_EDGES(graph &G, const edge_array<NT>& dx, 
                          const edge_array<NT>& dy)
{ 
  DX = &dx; DY = &dy;   
      // to transport information to the compare function

  G.sort_edges(&cmp_edges_by_angle2);

  assert(check_CCW_order_of_edges(G,dx,dy));
  return Is_Planar_Map(G);  // war Is_Plane_map...
}



bool check_CCW_order_of_edges(const graph &G, 
                              const node_array<NT>& x, 
                              const node_array<NT>& y)
{ edge e;
  edge_array<NT> dx(G);
  edge_array<NT> dy(G);
  forall_edges(e,G) 
  { node v = G.source(e); node w = G.target(e);
    dx[e] = x[w] - x[v];
    dy[e] = y[w] - y[v];
  }
  return check_CCW_order_of_edges(G, dx, dy);
}

bool SORT_EDGES(graph &G, 
                const node_array<NT>& x, 
                const node_array<NT>& y)
{ edge e;

  edge_array<NT> dx(G);
  edge_array<NT> dy(G);
  forall_edges(e,G) 
  { node v = G.source(e); node w = G.target(e);
    dx[e] = x[w] - x[v];
    dy[e] = y[w] - y[v];
  }
  return SORT_EDGES(G, dx, dy);
}



 
