
#line 43 "check_embedding.lw"
#undef LEDA_PREFIX

#include <AGD/impl/check_embedding.h>
#include <LEDA/plane_alg.h>
#include <LEDA/map.h>

static map<segment, edge> corr;
static list<edgepair> C;

void report(const segment& s, const segment& t) {
  node i = source(corr[s]), j = target(corr[s]),
    k = source(corr[t]), l = target(corr[t]);
  if (i == k || i == l || j == k || j == l) return;
  // i = k ----- j ----- l  missing!!!
  // i ----- k ----- j = l  missing!!!
  cout << s << " and " << t << " cross" << endl;
  C.append(edgepair(corr[s], corr[t]));
}

list<edgepair> check_orthogonal_grid_embedding(const graph& G, 
  const node_map<int>& x, const node_map<int>& y) {

  list<segment> S; C.clear();
 
  edge e; forall_edges(e, G) if (
#line 129 "check_embedding.lw"
x[source(e)] <= x[target(e)] && y[source(e)] <= y[target(e)]

#line 67 "check_embedding.lw"
                                                      ) {
    segment s(x[source(e)], y[source(e)], x[target(e)], y[target(e)]);
    S.append(s); corr[s] = e;
  }

  SEGMENT_INTERSECTION(S, report);

  cout << C.size() << " ... checking for same face... ";
 
  list_item it; forall_items(it, C) {
    face f = G.face_of(C[it].first()),
         g = G.face_of(G.reversal(C[it].first())),
         h = G.face_of(C[it].second()),
         i = G.face_of(G.reversal(C[it].second()));
    if (f != h && f != i && g != h && g != i) C.del(it);
  }
  
  cout << C.size() << " ... checking for doubles... ";

  edge_array<bool> forget(G, false);
  forall_items(it, C) {
    edge e1 = C[it].first(), e2 = C[it].second();
    if (forget[e1] || forget[e2]) C.del(it);
    forget[e1] = forget[e2] = true;
  }
  
  cout << C.size() << " ... only different directions... ";

  forall_items(it, C) {
    edge e1 = C[it].first(), e2 = C[it].second();
    if ((x[source(e1)] == x[target(e1)] == x[source(e2)] == x[target(e2)])
     || (y[source(e1)] == y[target(e1)] == y[source(e2)] == y[target(e2)]))
      C.del(it);
  }

  cout << C.size() << endl;

  

  return C; 

}

list<edgepair> check_orthogonal_grid_embedding_properly(const graph& G, 
  const node_map<int>& x, const node_map<int>& y) {

  list<segment> S; C.clear();
 
  edge e; forall_edges(e, G) if (
#line 129 "check_embedding.lw"
x[source(e)] <= x[target(e)] && y[source(e)] <= y[target(e)]

#line 115 "check_embedding.lw"
                                                      ) {
    segment s(x[source(e)], y[source(e)], x[target(e)], y[target(e)]);
    S.append(s); corr[s] = e;
  }

  SEGMENT_INTERSECTION(S, report);

  return C; 
}


#line 135 "check_embedding.lw"
int compare(const nodepair &p, const nodepair &q) {
  if (p.first() < q.first()) return -1;
  if (p.first() > q.first()) return -1;
  return compare(p.second(), q.second());
}


list<nodepair> check_vertex_positions(const graph& G, 
  const node_map<int>& x, const node_map<int>& y) {

  //node v, w; list<nodepair> S;
  //forall_nodes(v, G) forall_nodes(w, G) if (v < w) 
    //if (x[v] == x[w] && x[w] == y[w]) S.append(nodepair(v, w));

  list<nodepair> S; face f; node v, w;
  forall_faces(f, G) {
    list<node> L = G.adj_nodes(f);
    forall(v, L) forall(w, L) if (index(v) < index(w))
      if (x[v] == x[w] && y[v] == y[w]) S.append(nodepair(v, w));
  }
   
  S.sort(compare);
  S.unique(compare);

  return S;
}
  

