
// #line 52 "compactor.lw"
#include <AGD/compactor.h>

#include <LEDA/node_partition.h>
#include <LEDA/stack.h>

void longest_paths(const graph& D, node_array<int>& l) {

  node_array<int> delta_in(D, 0); stack<node> S;

  node v; forall_nodes(v, D) {
    delta_in[v] = indeg(v);
    if (delta_in[v] == 0) S.push(v);
  }

  while (!S.empty()) {
    v = S.pop();
    edge e; forall_out_edges(e, v) {
      node w = target(e);
      l[w] = Max(l[w], l[v] + 1);
      if (--delta_in[w] == 0) S.push(w);
    }
  }

}

void compact_one_dim(const OrthoShape& H, node_map<int>& c, bool horiz) {

  graph &G = H.get_const_graph();
  c.init(G, 0);
  
  node_partition P(G);

  edge e; forall_edges(e, H) {
    if ((horiz && H.west(e)) || (!horiz && H.north(e))) 
      P.union_blocks(source(e), target(e));
  }

  graph D; node_array<node> link(G, nil);

  node v; forall_nodes(v, H) if (!link[P.find(v)]) 
    link[P.find(v)] = D.new_node();

  forall_edges(e, H) 
    if ((horiz && H.north(e)) || (!horiz && H.west(e)))
      D.new_edge(link[P.find(source(e))], link[P.find(target(e))]);

  node_array<int> l(D, 0);
  longest_paths(D, l);

  forall_nodes(v, H) c[v] = l[link[P.find(v)]];

}

int compact_one_dim(const graph& G, GridLayout& L) {

  PlaneGraphCopy PG(G);
  return 0;
}

