
#include <AGD/FastHierarchyLayout.h>
#include <AGD/Hierarchy.h>
#include <AGD/LayoutInterface.h>

#define ALLOW .00001
#define forallnodes for(actNode=0;actNode<n;actNode++)
#define foralllayers for(actLayer=0;actLayer<k;actLayer++)
#define forallnodesonlayer \
	for(actNode=first[actLayer];actNode<first[actLayer+1];actNode++)

/*** Member functions of kList ***/

bool kList::pop(int& e,double& k) {
  if(empty()) return 0;
  withKey wk=pop_front();
  e=wk.element;
  k=wk.key;
  return 1;
}

double kList::peek() {
  return front().key;
}

bool kList::add(int e,double k) {
  withKey wK;
  wK.element=e;
  wK.key=k;
  return (append(wK) != nil);
}

double kList::median() const {
  int sz=size();
  if(!sz) return 0;
  item it=get_item(sz/2);
  double k=(*this)[it].key;
  if(sz==2*(int)(sz/2)) k=(k+(*this)[pred(it)].key)/2;
  return k;
}

void kList::reduce(kList& newList) {
  if(empty()) return;
  withKey oldWK,newWK;
  newWK=oldWK=pop_front();
  while(!empty()) {
    oldWK=pop_front();
    if(oldWK.key>newWK.key+ALLOW||oldWK.key<newWK.key-ALLOW) {
      if(newWK.element) newList+=newWK;
      newWK=oldWK;
    }
    else newWK.element+=oldWK.element;
  }
  if(newWK.element) newList+=newWK;
}

/*** Member functions of FastHierarchyLayout ***/

FastHierarchyLayout::FastHierarchyLayout()
  :minNodeDist(3),minLayerDist(3),fixedLayerDist(0) {
}

FastHierarchyLayout::~FastHierarchyLayout() {
}

string FastHierarchyLayout::name() const {
  return string("Phase 3");
}

string FastHierarchyLayout::long_name() const {
  return string("Sugiyama Algorithm, Phase 3");
}

string FastHierarchyLayout::author() const {
  return string("");
}

string FastHierarchyLayout::impl_author() const {
  return string("Christoph Buchheim");
}

string FastHierarchyLayout::impl_date() const {
  return string("july 1998");
}

void FastHierarchyLayout::call(const GraphCopy& GC,const Hierarchy& H,
			       LayoutInterface& A) {
  n=GC.number_of_nodes();
  m=GC.number_of_edges();
  k=H.number_of_layers();
  x=new double[n];
  breadth=new double[n];
  layer=new int[n];
  adj[0]=new list<int>[n];
  adj[1]=new list<int>[n];
  virt=new bool[n];
  longEdge=new list<int>*[n];
  height=new double[k];
  y=new double[k];
  first=new int[k+1];
  first[0]=0;
  int actLayer,actNode;
  foralllayers {
    first[actLayer+1]=first[actLayer]+H.length(actLayer);
    height[actLayer]=0;
  }
  node v1,v2;
  edge e1,e2;
  int n1,n2;
  list<int> *newEdge;
  forall_nodes(v1,GC) {
    n1=first[H.rank(v1)]+H.pos(v1);
    virt[n1]=H.is_dummy(v1);
    breadth[n1]=0;
    layer[n1]=H.rank(v1);
    if(!virt[n1]) {
      breadth[n1]=A.get_width(v1);
      incrTo(height[layer[n1]],A.get_height(v1));
      newEdge=new list<int>;
      newEdge->append(n1);
      longEdge[n1]=newEdge;
    }
  }
  forall_edges(e1,GC.original()) if(GC.chain(e1).size()>1) {
    newEdge=new list<int>;
    forall(e2,GC.chain(e1)) {
      v1=target(e2);
      n1=first[H.rank(v1)]+H.pos(v1);
      newEdge->append(n1);
    }
    newEdge->pop_back();
    forall(n1,*newEdge) longEdge[n1]=newEdge;
  }
  forall_edges(e1,GC) {
    v1=source(e1);
    v2=target(e1);
    n1=first[H.rank(v1)]+H.pos(v1);
    n2=first[H.rank(v2)]+H.pos(v2);
    adj[0][n2]+=n1;
    adj[1][n1]+=n2;
  }
  forallnodes {
    adj[0][actNode].sort();
    adj[1][actNode].sort();
  }
  findPlacement();
  forall_nodes(v1,GC) {
    n1=first[H.rank(v1)]+H.pos(v1);
    A.set_x(v1,x[n1]);
    A.set_y(v1,y[layer[n1]]);
  }
  A.make_straight_line();
  /*DPolyline bends;
  forall_edges(e1,GC.original()) if(H.chain(e1).size()>1) {
    v1=target(H.chain(e1).front());
    n1=first[H.rank(v1)]+H.pos(v1);
    forall(e2,H.chain(e1)) {
      v1=target(e2);
      n1=first[H.rank(v1)]+H.pos(v1);
      bends+=DPoint(x[n1],y[layer[n1]]);
    }
    bends.pop_back();
    if(GC.original(v1)==source(e1)) bends.reverse();
    A.original().set_bend_points(e1,bends);
    bends.clear();
  }
  bbox=DRect(0,boxX,0,boxY);*/
  list<int> *toDelete;
  forallnodes if(longEdge[actNode]!=0) {
    toDelete=longEdge[actNode];
    forall(n1,*toDelete) longEdge[n1]=0;
    delete toDelete;
  }
  delete[] y;
  delete[] first;
  delete[] height;
  delete[] x;
  delete[] breadth;
  delete[] layer;
  delete[] adj[0];
  delete[] adj[1];
  delete[] virt;
  delete[] longEdge;
}

double FastHierarchyLayout::node_distance() const {
  return minNodeDist;
}

void FastHierarchyLayout::node_distance(double d) {
  minNodeDist=d;
}

double FastHierarchyLayout::layer_distance() const {
  return minLayerDist;
}

void FastHierarchyLayout::layer_distance(double d) {
  minLayerDist=d;
}

bool FastHierarchyLayout::fixed_layer_distance() const {
  return fixedLayerDist;
}

void FastHierarchyLayout::fixed_layer_distance(bool b) {
  fixedLayerDist=b;
}

AGDModule *FastHierarchyLayout::clone() const {
  FastHierarchyLayout *fhl = new FastHierarchyLayout;
  fhl->node_distance(node_distance());
  fhl->layer_distance(layer_distance());
  fhl->fixed_layer_distance(fixed_layer_distance());
  return fhl;
}

void FastHierarchyLayout::incrTo(double& d,double t) {
  if(d<t) d=t;
}

void FastHierarchyLayout::decrTo(double& d,double t) {
  if(d>t) d=t;
}

bool FastHierarchyLayout::sameLayer(int n1,int n2) const {
  return (n1>=0&&n1<n&&n2>=0&&n2<n&&layer[n1]==layer[n2]);
}

bool FastHierarchyLayout::isFirst(int actNode) const {
  return actNode<0||actNode>=n||actNode==first[layer[actNode]];
}

bool FastHierarchyLayout::isLast(int actNode) const {
  return actNode<0||actNode>=n||actNode==first[layer[actNode]+1]-1;
}

void FastHierarchyLayout::sortLongEdges(int actNode,int dir,double *pos,
					bool& exD,double &dist,
					int *block,bool *marked) {
  if(marked[actNode]) return;
  bool exB=0;
  double best=0;
  int next;
  forall(next,*longEdge[actNode]) marked[next]=1;
  forall(next,*longEdge[actNode]) 
    if(sameLayer(next-dir,next)&&block[next-dir]==block[next]) {
      sortLongEdges(next-dir,dir,pos,exD,dist,block,marked);
      if(!exB||dir*(best-pos[next-dir])<dir*(totalB[next]-totalB[next-dir])) {
	exB=1;
	best=pos[next-dir]+totalB[next]-totalB[next-dir];
      }
    }
  forall(next,*longEdge[actNode]) {
    pos[next]=best;
    if(sameLayer(next+dir,next)&&block[next+dir]!=block[next]
       &&(!exD||
	  dir*(totalB[next+dir]-totalB[next]-pos[next+dir]+pos[next])>dist)) {
      dist=dir*(totalB[next+dir]-totalB[next]-pos[next+dir]+pos[next]);
      exD=1;
    }
  }
}

bool FastHierarchyLayout::placeSingleNode(int leftBnd,int rightBnd,
					  int actNode,double& best,int d) {
  int next;
  kList neighbours;
  forall(next,adj[d][actNode]) neighbours.add(0,x[next]);
  if(neighbours.empty()) return 0;
  best=neighbours.median();
  if(leftBnd!=-1) incrTo(best,x[leftBnd]+mDist[actNode]-mDist[leftBnd]);
  if(rightBnd!=-1) decrTo(best,x[rightBnd]+mDist[actNode]-mDist[rightBnd]);
  return 1;
}

void FastHierarchyLayout::placeNodes(int leftBnd,int rightBnd,
				     int left,int right,int d) {
  if(left==right) placeSingleNode(leftBnd,rightBnd,left,x[left],d);
  else if(left<right) {
    int mdl[2],bnd[2],res[2];
    kList bds,bends[2];
    int actNode,next,resChange,resCh0,resCh1;
    double mD,newBend,diff,diff1,diff2;
    mdl[0]=(right+left)/2; mdl[1]=mdl[0]+1;
    placeNodes(leftBnd,rightBnd,left,mdl[0],d);
    placeNodes(leftBnd,rightBnd,mdl[1],right,d);
    bnd[0]=leftBnd; bnd[1]=rightBnd;
    mD=mDist[mdl[1]]-mDist[mdl[0]];
    int dir;
    for(int i=0;i<2;i++) {
      dir=i?1:-1;
      res[i]=0;
      for(actNode=mdl[i];actNode>=left&&actNode<=right;actNode+=dir) {
	resChange=0;
	forall(next,adj[d][actNode]) {
	  if(dir*(x[next]-x[actNode])<ALLOW) resChange++;
	  else {
	    resChange--;
	    newBend=x[next]+mDist[mdl[i]]-mDist[actNode];
	    if(dir*(x[mdl[i]]-newBend)>-ALLOW) res[i]++;
	    else if((bnd[i]==-1||dir*(newBend-x[bnd[i]]
				      +mDist[bnd[i]]-mDist[mdl[i]])<ALLOW)
		    &&dir*(newBend-x[mdl[1-i]])<mD-ALLOW)
	      bds.add(2,newBend);
	  }
	}
	newBend=x[actNode]+mDist[mdl[i]]-mDist[actNode];
	if(dir*(x[mdl[i]]-newBend)>-ALLOW) res[i]+=resChange;
	else if((bnd[i]==-1||dir*(newBend-x[bnd[i]]
		 +mDist[bnd[i]]-mDist[mdl[i]])<ALLOW)
		&&dir*(newBend-x[mdl[1-i]])<mD-ALLOW)
	  bds.add(resChange,newBend);
      }
      if(bnd[i]!=-1) bds.add(m,x[bnd[i]]-mDist[bnd[i]]+mDist[mdl[i]]);
      bds.sort(&cmpWithKey);
      bds.reduce(bends[i]);
    }
    if(!bends[0].empty()) bends[0].reverse();
    while(x[mdl[1]]-x[mdl[0]]<mD-ALLOW) {
      resCh0=resCh1=0;
      if(res[0]<res[1]) {
	if(!bends[0].pop(resCh0,x[mdl[0]])||x[mdl[1]]-x[mdl[0]]>mD+ALLOW) 
	  x[mdl[0]]=x[mdl[1]]-mD;
      }
      else if(res[1]<res[0]) {
	if(!bends[1].pop(resCh1,x[mdl[1]])||x[mdl[1]]-x[mdl[0]]>mD+ALLOW) 
	  x[mdl[1]]=x[mdl[0]]+mD;
      }
      else {
        diff=(mD-x[mdl[1]]+x[mdl[0]])/2;
	diff1=bends[0].empty()?diff+1:x[mdl[0]]-bends[0].peek();
	diff2=bends[1].empty()?diff+1:bends[1].peek()-x[mdl[1]];
      	if(diff1<diff+ALLOW&&diff1<diff2+ALLOW) 
	  bends[0].pop(resCh0,newBend);
	if(diff2<diff+ALLOW&&diff2<diff1+ALLOW) 
	  bends[1].pop(resCh1,newBend);
	decrTo(diff,diff1);
	decrTo(diff,diff2);
	x[mdl[0]]-=diff; 
	x[mdl[1]]+=diff;
      }
      res[0]+=resCh0;
      res[1]+=resCh1;
    }
    actNode=mdl[0];
    while(--actNode>=left
	  &&x[mdl[0]]-x[actNode]<mDist[mdl[0]]-mDist[actNode])
      x[actNode]=x[mdl[0]]-mDist[mdl[0]]+mDist[actNode];
    actNode=mdl[1];
    while(++actNode<=right
	  &&x[mdl[1]]-x[actNode]>mDist[mdl[1]]-mDist[actNode])
      x[actNode]=x[mdl[1]]-mDist[mdl[1]]+mDist[actNode];
  }
}

void FastHierarchyLayout::moveLongEdge(int actNode,int dir,
				       bool *marked) {
  if(!marked[actNode]&&virt[actNode]) { 
    int next;
    forall(next,*longEdge[actNode]) marked[next]=1;
    int fst=adj[0][longEdge[actNode]->front()].front();
    int lst=adj[1][longEdge[actNode]->back()].front();
    double xFirst,xOpt;
    list<double> toTest;
    if(dir<0) {
      toTest+=x[fst]<x[lst]?x[fst]:x[lst];
      toTest+=x[fst]<x[lst]?x[lst]:x[fst];
    }
    else if(dir>0) {
      toTest+=x[fst]<x[lst]?x[lst]:x[fst];
      toTest+=x[fst]<x[lst]?x[fst]:x[lst];
    }
    else {      
      toTest+=x[fst];
      toTest+=x[lst];
    }
    xFirst=toTest.front();
    bool done=0;
    while(!done&&!toTest.empty()) {
      xOpt=toTest.pop();
      done=1;
      forall(next,*longEdge[actNode]) {
	if(!isFirst(next)) {
	  if(xOpt-x[next-1]<totalB[next]-totalB[next-1]) 
	    moveLongEdge(next-1,-1,marked);
	  done=done&&xOpt-x[next-1]>=totalB[next]-totalB[next-1]-ALLOW;
	}
	if(!isLast(next)) {
	  if(xOpt-x[next+1]>totalB[next]-totalB[next+1]) 
	    moveLongEdge(next+1,1,marked);
	  done=done&&xOpt-x[next+1]<=totalB[next]-totalB[next+1]+ALLOW;
	}
      }
    }
    if(!done) {
      xOpt=xFirst;
      forall(next,*longEdge[actNode]) {
	if(!isFirst(next))
	  incrTo(xOpt,x[next-1]+totalB[next]-totalB[next-1]);
	if(!isLast(next))
	  decrTo(xOpt,x[next+1]+totalB[next]-totalB[next+1]);
      }
    }
    forall(next,*longEdge[actNode]) x[next]=xOpt;
  }
}  

void FastHierarchyLayout::straightenEdge(int actNode,bool *marked) {
  if(!marked[actNode]&&breadth[actNode]<ALLOW
     &&adj[0][actNode].size()==1
     &&adj[1][actNode].size()==1
     &&longEdge[actNode]->size()<2) {
    marked[actNode]=1;
    int fst=adj[0][actNode].front();
    int lst=adj[1][actNode].front();
    double xOpt=x[fst]+(x[lst]-x[fst])
                *(y[layer[actNode]]-y[layer[fst]])
                /(y[layer[lst]]-y[layer[fst]]);
    if(!isFirst(actNode)) {
      if(xOpt-x[actNode-1]<totalB[actNode]-totalB[actNode-1]-ALLOW)
        straightenEdge(actNode-1,marked);
      if(xOpt-x[actNode-1]<totalB[actNode]-totalB[actNode-1]-ALLOW)
        return;
    }
    if(!isLast(actNode)) {
      if(x[actNode+1]-xOpt<totalB[actNode+1]-totalB[actNode]-ALLOW)
        straightenEdge(actNode+1,marked);
      if(x[actNode+1]-xOpt<totalB[actNode+1]-totalB[actNode]-ALLOW)
        return;
    }
    x[actNode]=xOpt;
  }
}

void FastHierarchyLayout::findPlacement() {
  int actNode,next,last,actLayer,dir,leftBnd,leftNxt,rightNxt;
  bool *marked=new bool[n];
  forallnodes if(virt[actNode]
		 &&!virt[adj[0][actNode].front()]
		 &&!virt[adj[1][actNode].front()]) virt[actNode]=0;

  // Compute minimal distances between nodes (totalB)

  totalB=new double[n];
  double toAdd;
  forallnodes {
    if(!actNode||actNode&&layer[actNode-1]<layer[actNode]) totalB[actNode]=0;
    else {
      toAdd=breadth[actNode-1]/2+breadth[actNode]/2;
      incrTo(toAdd,minNodeDist/3
	     *(double)(adj[0][actNode-1].size()+adj[0][actNode].size()));
      incrTo(toAdd,minNodeDist/3
	     *(double)(adj[1][actNode-1].size()+adj[1][actNode].size()));
      toAdd=minNodeDist*(int)(toAdd/minNodeDist+1-ALLOW);
      toAdd+=minNodeDist;
      totalB[actNode]=totalB[actNode-1]+toAdd;
    }
  }

  // Remove crossings of long edges

  list<int> *newEdge,*oldEdge;
  int down,spl;
  foralllayers {
    last=-1;
    forallnodesonlayer if(virt[actNode]) {
      down=adj[1][actNode].front();
      if(virt[down]) {
	if(last!=-1&&last>down) {
	  oldEdge=longEdge[actNode];
	  spl=actLayer-layer[oldEdge->front()]+1;
	  newEdge=new list<int>;
	  oldEdge->split(oldEdge->get_item(spl),*newEdge,*oldEdge);
	  forall(next,*newEdge) longEdge[next]=newEdge;
	}
	else last=down;
      }
    }
  }

  // Place long edges

  int blockCount;
  bool exD;
  int *block=new int[n];
  double *pos=new double[n];
  list<int> *blockNodes;
  kList neighbours;
  double dist;
  forallnodes x[actNode]=0;
  for(dir=1;dir>=-1;dir-=2) {
    blockCount=0;
    blockNodes=new list<int>[n];
    forallnodes block[actNode]=marked[actNode]=0;
    foralllayers {
      actNode=(dir==1?first[actLayer]:first[actLayer+1]-1);
      if(!block[actNode]) {
	forall(next,*longEdge[actNode]) block[next]=blockCount;
	blockCount++;
      }
      actNode+=dir;
      while(actNode>=first[actLayer]&&actNode<first[actLayer+1]) {
	if(!block[actNode]) 
	  forall(next,*longEdge[actNode]) block[next]=block[actNode-dir];
	actNode+=dir;
      }
    }
    foralllayers 
      for(actNode=(dir==1?first[actLayer]:first[actLayer+1]-1);
	  actNode>=first[actLayer]&&actNode<first[actLayer+1];actNode+=dir)
	blockNodes[block[actNode]]+=actNode;
    for(int i=0;i<blockCount;i++) {
      exD=0; dist=0;
      forall(actNode,blockNodes[i]) 
	sortLongEdges(actNode,dir,pos,exD,dist,block,marked);
      if(!exD) {
	actLayer=layer[blockNodes[i].front()];
	forallnodesonlayer forall(next,adj[0][actNode])
	  neighbours.add(0,pos[next]-pos[actNode]);
	if(!neighbours.empty()) {
	  neighbours.sort(&cmpWithKey);
	  dist=-dir*neighbours.median();
	  neighbours.clear();
	}
      }
      forall(actNode,blockNodes[i]) pos[actNode]-=dir*dist;
    }
    forallnodes x[actNode]+=pos[actNode]/2;
    delete[] blockNodes;
  }
  delete[] block;
  delete[] pos;

  // Place nonvirtual nodes

  int *nodeDir=new int[n];
  forallnodes {
    if(virt[actNode]) {
      next=actNode+1;
      while(sameLayer(next,actNode)&&!virt[next]) next++;
      marked[actNode]=sameLayer(next,actNode)
	&&x[next]-x[actNode]<totalB[next]-totalB[actNode]+ALLOW;
    }
    else marked[actNode]=0;
    nodeDir[actNode]=-1;
  }
  forallnodes if(marked[actNode]&&nodeDir[actNode]==-1)
    forall(next,*longEdge[actNode]) if(next!=actNode)
      nodeDir[next]=next<actNode;
  mDist=new double[n];
  forallnodes mDist[actNode]=totalB[actNode];
  for(dir=0;dir<2;dir++) {
    for(actLayer=dir?k-1:0;0<=actLayer&&actLayer<k;actLayer+=dir?-1:1) {
      leftBnd=-1;
      forallnodesonlayer if(virt[actNode]) {
	if(leftBnd==-1) {                    
	  placeNodes(-1,actNode,first[actLayer],actNode-1,dir);
	  for(next=first[actLayer];next<actNode;next++)
	    mDist[next]=mDist[actNode]-x[actNode]+x[next];
	}
	else if(nodeDir[leftBnd]!=!dir) {
	  if(!marked[leftBnd])
	    placeNodes(leftBnd,actNode,leftBnd+1,actNode-1,dir);
	  leftNxt=adj[!dir][leftBnd].front();
	  rightNxt=adj[!dir][actNode].front();
	  if(virt[leftNxt]&&virt[rightNxt])
	    for(next=leftNxt+1;next<rightNxt;next++) 
	      nodeDir[next]=dir;
	}
	leftBnd=actNode;
      }
      if(leftBnd==-1) {
	placeNodes(-1,-1,first[actLayer],first[actLayer+1]-1,dir);
	for(next=first[actLayer];next<first[actLayer+1];next++)
	  mDist[next]=x[next];
      }
      else {
	placeNodes(leftBnd,-1,leftBnd+1,first[actLayer+1]-1,dir);
	for(next=first[actLayer+1]-1;next>leftBnd;next--)
	  mDist[next]=mDist[leftBnd]-x[leftBnd]+x[next];
      }
    }
  }
  delete[] nodeDir;
  delete[] mDist;

  // Make-up

  forallnodes marked[actNode]=0;
  foralllayers {
    for(actNode=(first[actLayer]+first[actLayer+1])/2;
	actNode<first[actLayer+1];actNode++)
      moveLongEdge(actNode,0,marked);
    for(actNode=(first[actLayer]+first[actLayer+1])/2-1;
	actNode>=first[actLayer];actNode--)
      moveLongEdge(actNode,0,marked);
  }

  // Compute ordinates for the layers and boxY

  double boxY=k?height[0]/2:0;
  double minD;
  foralllayers {
    y[actLayer]=boxY;
    minD=minLayerDist;
    if(!fixedLayerDist) {
      forallnodesonlayer forall(next,adj[1][actNode]) {
	incrTo(minD,(x[next]-x[actNode])/3);
	incrTo(minD,(x[actNode]-x[next])/3);
      }
      decrTo(minD,10*minLayerDist);
    }
    boxY+=height[actLayer]/2;
    if(actLayer<k-1) boxY+=minD+height[actLayer+1]/2;
  }

  // Make-up

  forallnodes marked[actNode]=0;
  foralllayers {
    for(actNode=(first[actLayer]+first[actLayer+1])/2;
	actNode<first[actLayer+1];actNode++)
      straightenEdge(actNode,marked);
    for(actNode=(first[actLayer]+first[actLayer+1])/2-1;
	actNode>=first[actLayer];actNode--)
      straightenEdge(actNode,marked);
  }
  delete[] marked;
  delete[] totalB;
/*
  // Compute boxX

  double xMin,xMax;
  xMin=xMax=0;
  foralllayers {
    if(!actLayer||x[first[actLayer]]-breadth[first[actLayer]]/2<=xMin)
      xMin=x[first[actLayer]]-breadth[first[actLayer]]/2;
    if(!actLayer||x[first[actLayer+1]-1]+breadth[first[actLayer+1]-1]/2>=xMax)
      xMax=x[first[actLayer+1]-1]+breadth[first[actLayer+1]-1]/2;
  }
  forallnodes x[actNode]-=xMin;
  boxX=xMax-xMin;
*/
}
