/************************************************************************

         Filename   :   matching.cc

         Version    :   09.1994

         Author     :   Sebastian Leipert

         Language   :   C++


 File containing all procedures for applying the pattern matching
 algorithm of Booth and Lueker in order to reduce the maximal
 pertinent sequence.

************************************************************************/ 



#include <AGD/impl/MaxPlanSub/proto.h>

     
extern  int      verticecount;
extern  int      edgecount;
extern  int      directed;
extern  int      total_num_del_edges;
extern  int      node_count;
extern  pq_node  *root;

extern  int      _printDuringReduction;
extern  int      _printPQTreeIterative;
extern  int      _iterationNumber;
extern  char     _debugInformation[INFOSIZE];


/************************************************************************
                          reduce
************************************************************************/ 

void reduce(
            pq_node **pert_root,
            int i,
            int,
            stack<int> *pertinent_leaves,
            leaves *leave_ptr,
            stack<pq_node*> *pert_nodes
           )

   // Procedure that goes form the pertinent leaves up to the root of
   // the pertinent subtree and applies the template matchings.
{
   pq_node      *node_ptr;
   int          helpnode;
   queue<pq_node*> *Q = new queue<pq_node*>;
                                    // The Queue is used to store the
                                    // nodes, that have to be edited.
   int          max_pert_leaf_count = 0;
                                    // Denotes the number of pertinent
                                    // leaves in the tree.
  
   
   pertinent_leaves->start_at_bottom();
   while (!pertinent_leaves->read_last())
                                    // First store the pertinent leaves
                                    // in the Queue.
   {
      helpnode = pertinent_leaves->read_next();
      node_ptr = leave_ptr->get_ptr(helpnode,i);
      if (node_ptr != NULL)
      {
         max_pert_leaf_count++;
         node_ptr->label = FULL;
         node_ptr->pert_leaf_count = 1;
         Q->enqueue(node_ptr);
      }// if
   }// while

   
   node_ptr = Q->front();
   while (node_ptr != NULL && Q->queue_size() > 0)
                                    // While there are still nodes in
                                    // Q that have to be edited, we dequeue
                                    // the first one out of Q and apply the
                                    // template matchings to it.
   {
      node_ptr = Q->dequeue();     
           
      if (node_ptr->pert_leaf_count < max_pert_leaf_count)
                                    // node_ptr is not the root of the
                                    // pertinent subtree.
      {
         node_ptr->parent->pert_leaf_count = node_ptr->parent->pert_leaf_count
                                            + node_ptr->pert_leaf_count;
         node_ptr->parent->pert_child_count--;
         if (!node_ptr->parent->pert_child_count)
            Q->enqueue(node_ptr->parent);
         if (!template_L1(node_ptr,FALSE))
         if (!template_PQ1(node_ptr,FALSE))
         if (!template_P3(node_ptr,pert_nodes))
         if (!template_P5(node_ptr,pert_nodes))
         if (!template_Q2(node_ptr,FALSE))
            node_ptr= NULL;
      }// if 

      else
                                    // node_ptr is the root of the
                                    // pertinent subtree.
      {
         if (!template_L1(node_ptr,TRUE))
         if (!template_PQ1(node_ptr,TRUE))
         if (!template_P2(&node_ptr,pert_nodes))
         if (!template_P4(&node_ptr,pert_nodes))
         if (!template_P6(&node_ptr,pert_nodes))
         if (!template_Q2(node_ptr,TRUE))
         if (!template_Q3(node_ptr))
            node_ptr = NULL;
      }// else
   }// while
   (*pert_root) = node_ptr;

   delete Q;
}// reduce



/************************************************************************
                          template_L1
************************************************************************/ 

int template_L1(
                pq_node *node_ptr,
                int is_root
               )

   // Template matching for leaves. It basically updates the full_children
   // stack of its parent as long as node_ptr is not the root.
{
   if (node_ptr->typ == LEAF && node_ptr->label == FULL)
   {
      if (!is_root)
         node_ptr->parent->full_children->push(node_ptr);
      return TRUE;
   }// if

   else
      return FALSE;
}// template_L1



/************************************************************************
                          template_PQ1
************************************************************************/ 

int template_PQ1(
                 pq_node *node_ptr,
                 int is_root
                )

    // Template matching for P- an Q-nodes, with only full children.
    // If the P- or Q-node is not the root of the pertinent subtree,
    // the full_children stack of its parent is updated.
{
   if (node_ptr->full_children->count() != node_ptr->child_count)
      return FALSE;

   else
   {
      node_ptr->label = FULL;
      if (node_ptr->typ == Q_NODE)
      {
         node_ptr->endmost_children->start_at_bottom();
         node_ptr->first_full = node_ptr->endmost_children->read_next();
      }// if
      if (!is_root)
         node_ptr->parent->full_children->push(node_ptr);
      return TRUE;
   }// else
}// template_PQ1



/************************************************************************
                          template_P2
************************************************************************/ 

int template_P2(
                pq_node **node_ptr,
                stack<pq_node*> *pert_nodes
               )

   // Template Matching for a P-node with full and empty children,
   // which is the root of the pertinent subtree. Creates a new full
   // P-node, which will be the root of the pertinent subtree, and copies
   // the full children from the old root to the new one.
{
   pq_node  *new_node = NULL;
   
   if ((*node_ptr)->typ != P_NODE   ||
       (*node_ptr)->partial_children->count() > 0)
      return FALSE;

   else
   {
      (*node_ptr)->child_count = (*node_ptr)->child_count -
                              (*node_ptr)->full_children->count() + 1;
                                     // Gather all full children of node_ptr
                                     // as children of the new P-node.
                                     // Delete them from node_ptr.
      new_node = copy_full_children_from_P_to_P_node
                  ((*node_ptr)->full_children,pert_nodes);
                                     // Correct parent-pointer and
                                     // sibling-pointers of the new P-node.
      new_node->parent = (*node_ptr);
      new_node->sib_next = (*node_ptr)->child_sib_list->sib_next;
      new_node->sib_pre = new_node->sib_next->sib_pre;
      new_node->sib_pre->sib_next = new_node;
      new_node->sib_next->sib_pre = new_node;
                                     // The new P-node now is the root of
                                     // the pertinent subtree.
      (*node_ptr) = new_node;
      return TRUE;
   }// else
}// template_P2

                                       
         
/************************************************************************
                          template_P3
************************************************************************/ 
         
int template_P3(
                pq_node *node_ptr,
                stack<pq_node*> *pert_nodes
               )

   // Template Matching for a P-node with full and empty children,
   // which is NOT the root of the pertinent subtree. Creates a new full
   // P-node and copies the full children of node_ptr to the new
   // P-node. node_ptr keeps all empty_children and will be labeled empty.
   // A new partial Q-node is created, and will be set at the position of
   // node_ptr in the PQ-tree. node_ptr and the new P-node will be the
   // children of the new Q-node.
{
   pq_node  *new_Qnode;               // pointer to the new Q-node
   //   pq_node  *new_Pnode;               // pointer to the new P-node
   pq_node  *father;                  // pointer to the parent of node_ptr
   pq_node  *empty_node;              // pointer to an empty child of node_ptr
                                      

   
   if (node_ptr->typ != P_NODE || node_ptr->partial_children->count() > 0)
      return FALSE;

   else
                                      // Create a new Q-node.
   {
      new_Qnode = new pq_node(0,0,node_count++,Q_NODE,PARTIAL); 
      pert_nodes->push(new_Qnode);
                                      // Set Q-node into position of node_ptr
                                      // by updating parent-pointers and
                                      // sibling-pointers. Keep node_ptr as a
                                      // child of the new Q-node.
      father = node_ptr->parent;
      new_Qnode->parent = node_ptr->parent;  
      node_ptr->parent = new_Qnode;
                                      // Change sibling-pointers.
      if (node_ptr->imm_siblings->count() > 0) 
                                      // node_ptr was son of a Q-node.
         change_Q_imm_sib(node_ptr,new_Qnode,father);
      else
                                      // node_ptr was son of a P-node.
         change_P_imm_sib(node_ptr,new_Qnode);

      new_Qnode->endmost_children->push(node_ptr);
      new_Qnode->child_count = 1;
                                      // Create a new P-node and copy
                                      // the full children to it.
      P3_copy_full_ch_to_partial(node_ptr,new_Qnode,pert_nodes);
                                      // Check if node_ptr contains
                                      // only one son. If so, node_ptr
                                      // will be deleted from the tree.
      empty_node = node_ptr->child_sib_list;
      check_if_one_son(node_ptr,empty_node);
                                      // Update partial_children stack of
                                      // the parent of the new Q-node.
      new_Qnode->parent->partial_children->push(new_Qnode);
      return TRUE;
   }// else
}// template_P3



/************************************************************************
                          template_P4
************************************************************************/ 

int template_P4(
                pq_node **node_ptr,
                stack<pq_node*> *pert_nodes
               )

   // Template Matching for a P-node with full and empty children  and 
   // exactly one partial child, which is the root of the pertinent
   // subtree. Creates a new full P-node if necessary and copies the full
   // children of node_ptr to this P-node. The new P-node is made an endmost
   // child of the partial Q-node, which is a child of node_ptr.
{
   pq_node  *partial_child;           // pointer to the partial_child
   //   pq_node  *new_Pnode;               // pointer to the new P-node

   
   if ((*node_ptr)->typ != P_NODE   ||
       (*node_ptr)->partial_children->count() != 1)
      return FALSE;
      
   else
   {
      partial_child = (*node_ptr)->partial_children->pop();
      copy_full_ch_to_partial(*node_ptr,partial_child,pert_nodes);
                                      // If node_ptr does not have any
                                      // empty children, then it has to
                                      // be deleted and the partial node
                                      // is occupying its place in the tree.
      check_if_one_son(*node_ptr,partial_child);
                                      // The partial child now is
                                      // root of the pertinent subtree.
      *node_ptr = partial_child;
      return TRUE;
   }// else
}// template_P4



/************************************************************************
                          template _P5
************************************************************************/ 

int template_P5(pq_node *node_ptr,stack<pq_node*> *pert_nodes)

   // Template Matching for a P-node with full and empty children and
   // exactly one partial child which is NOT the root of the pertinent
   // subtree. It creates a new full P-node if necessary, and copies all
   // the full children of node_ptr to the new full P-node,
   // whereas all empty children of node_ptr stay children of node_ptr.
   // The new empty P-node and node_ptr both will be the two new endmost
   // children of the partial child. The partial child then occupies the
   // possition of node_ptr.
{
   pq_node  *partial_child;            // pointer to the partial node
   pq_node  *helpptr;                  // pointer to the endmost
                                       // empty child of partial child
   pq_node  *new_empty_child = NULL;   // pointer to the empty node beeing
                                       // copied to partial_child
   int      empty_child_count;         // counts the number of empty
                                       // children of node_ptr
   int      found;
   

   if (node_ptr->typ != P_NODE || node_ptr->partial_children->count() != 1)
      return FALSE;
      
   else   
   {
      empty_child_count = node_ptr->child_count -
                          node_ptr->full_children->count() - 1;
                                       // Remove partial child from node_ptr.
      partial_child = node_ptr->partial_children->pop();
      partial_child->parent = node_ptr->parent;
      node_ptr->parent->partial_children->push(partial_child);
      remove_P_imm_sib(partial_child);
                                       // Get partial_child into the position
                                       // of node_ptr in the tree.
      if (node_ptr->imm_siblings->count() == 0)
         change_P_imm_sib(node_ptr,partial_child);
      else
         change_Q_imm_sib(node_ptr,partial_child,node_ptr->parent);
                                       // Copy all full children from
                                       // node_ptr to partial_child.
      copy_full_ch_to_partial(node_ptr,partial_child,pert_nodes);
                                       // Now node_ptr just has empty
                                       // children. Copy node_ptr as
                                       // endmost child to partial_child.

      if (empty_child_count > 0)
                                       // node_ptr does have empty children.
      {
         partial_child->endmost_children->start_at_bottom();
                                       // Check at which side of the Q-node
                                       // the empty children hide.
         found = FALSE;   
         while (!partial_child->endmost_children->read_last() && !found)
         {
            helpptr = partial_child->endmost_children->read_next();
            if (helpptr->label == EMPTY)
               found = TRUE;
         }// while
                                       // helpptr points to an empty
                                       // endmost child of partial_child.
                                       
         if (empty_child_count == 1)
                                       // node_ptr does have only one
                                       // empty child. So just its empty
                                       // child is copied to partial_child.
             new_empty_child = node_ptr->child_sib_list;
             
         else
                                       // node_ptr does have more than one
                                       // empty child, so copy node_ptr to
                                       // partial_child.
         {
            new_empty_child = node_ptr;
            new_empty_child->child_count = empty_child_count;
         }// else

                                       // new_empty_child now contains
                                       // the node to be copied. Update the
                                       // pointers.
         new_empty_child->parent = partial_child;
         partial_child->endmost_children->search_and_change
                                    (helpptr,new_empty_child);
         partial_child->child_count++;
         helpptr->imm_siblings->push(new_empty_child);
         new_empty_child->imm_siblings->push(helpptr);
      }// if
      
                                       // If node_ptr did not have any empty
                                       // children it has to be deleted.
      if (empty_child_count <= 1)
         destroy(node_ptr);
      return TRUE;
   }// else
}// template_P5




/************************************************************************
                          template_P6
************************************************************************/ 

int template_P6(
                pq_node **node_ptr,
                stack<pq_node*> *pert_nodes
               )

   // Template Matching for a P-node with full and empty children and
   // exactly two partial children which is the root of the pertinent
   // subtree. It creates a new full P-node if necessary, and copies all
   // the full children of node_ptr to the new full P-node
   // whereas all empty children of node_ptr stay children of node_ptr.
   // The new P-node will be copied to one partial child as endmost
   // child of this node. Then the children of the second partial node
   // are copied to the first one, such that the pertinent nodes form
   // a consective sequence.
{
   pq_node  *partial_1 = NULL;       // pointer to the first partial child
   pq_node  *partial_2 = NULL;       // pointer to the second partial child
   pq_node  *helpptr = NULL;
   pq_node  *fullend_1 = NULL;       // pointer denoting a full endmost
                                     // child of partial_1
   pq_node  *fullend_2 = NULL;       // pointer denoting a full endmost
                                     // child of partial_2
   pq_node  *emptyend_2 = NULL;      // pointer denoting an empty endmost
                                     // child of partial_2
   int      found;
   

   if ((*node_ptr)->typ != P_NODE   ||
       (*node_ptr)->partial_children->count() != 2)
      return FALSE;
      
   else
   {
      partial_1 = (*node_ptr)->partial_children->pop();
      partial_2 = (*node_ptr)->partial_children->pop();
                                     // We keep partial_1 as son of
                                     // node_ptr and copy the children
                                     // of partial_2 to it.
      remove_P_imm_sib(partial_2);
      (*node_ptr)->child_count--;
                                     // Copy all full children of
                                     // node_ptr to partial_1, if there
                                     // are any.
      copy_full_ch_to_partial(*node_ptr,partial_1,pert_nodes);

      partial_1->endmost_children->start_at_bottom();
                                     // Get endmost child of partial_1 at the
                                     // side where the full_children hide.
      found = FALSE;
      while (!partial_1->endmost_children->read_last() && !found)
      {
         fullend_1 = partial_1->endmost_children->read_next();
         if (fullend_1->label == FULL)
            found = TRUE;
         else
            fullend_1 = NULL;
      }//while
      
      partial_2->endmost_children->start_at_bottom();
      while (!partial_2->endmost_children->read_last())
                                     // Get the endmost children of partial_2.
                                     // Differ between their labels.
      {
         helpptr = partial_2->endmost_children->read_next();
         if (helpptr->label == FULL)
            fullend_2 = helpptr;
         else
            emptyend_2 = helpptr;
      }// while

      while (!partial_2->full_children->stack_empty())
                                    // Copy full children in stack.
      {
         helpptr = partial_2->full_children->pop();
         partial_1->full_children->push(helpptr);
      }// while

                                    // Link the siblings.
      fullend_1->imm_siblings->push(fullend_2);
      fullend_2->imm_siblings->push(fullend_1);
      
      partial_1->endmost_children->search_and_change(fullend_1,emptyend_2);
      partial_1->child_count = partial_1->child_count +
                                  partial_2->child_count;
                                    // Remove partial_2 from tree.
                                    // Keep in mind that the children
                                    // of partial_2 now do have partial_1
                                    // as a parent.
      mark_killed(partial_2,partial_1);
                                    // partial_1 is now root of the
                                    // pertinent subtree.
      *node_ptr = partial_1;
      return TRUE;
   }// else
}// template_P6



/************************************************************************
                          template_Q2
************************************************************************/ 

int template_Q2(
                pq_node *node_ptr,
                int is_root
               )

   // Template Matching for a Q-node with empty children on one side and
   // full children on the other and at a maximum one partial child
   // between them. The Q-node may be the root of the pertinent subtree.
{
   int       found = FALSE;
   int       cons_sequence = FALSE;
   pq_node   *helpptr = NULL;
   pq_node   *seq_start = NULL;
   pq_node   *seq_end = NULL;
   pq_node   *partial_child = NULL;  // pointer to the partial child
   
   

   if (node_ptr->typ != Q_NODE || node_ptr->partial_children->count() > 1)
      return FALSE;
      
   else
                                     // node_ptr does have at a maximum
                                     // one partial child.
   {
      if (node_ptr->full_children->count() > 0)
                                     // node_ptr does have full children.
                                     // Check if full children and partial
                                     // child form a consecutive sequence. If
                                     // not, the tree is not reducible. Check
                                     // further, if the sequence is on one
                                     // side of the Q-node, if not template
                                     // Q3 has to be applied.
      {
         node_ptr->endmost_children->start_at_bottom();
                                     // Check the endmost children for a full
                                     // one.
         while (!node_ptr->endmost_children->read_last() && !found)
         {
            helpptr = node_ptr->endmost_children->read_next();
            if (helpptr->label == FULL)
               found = TRUE;
         }// while
         
         if (found) 
                                     // There is a full endmost child. Check
                                     // whether there is a consecutive
                                     // sequence of the full nodes.
            cons_sequence = checkchain(node_ptr,helpptr,&seq_start,&seq_end);

         if (cons_sequence && node_ptr->partial_children->count() ==1)
                                      // Check if the consecutive sequence
                                      // of full children is adjacent to
                                      // the partial node.
         {
            node_ptr->partial_children->start_at_bottom();
            partial_child = node_ptr->partial_children->read_next();
            seq_end->imm_siblings->start_at_bottom();
            cons_sequence = FALSE;
            while (!seq_end->imm_siblings->read_last())
            {
               helpptr = seq_end->imm_siblings->read_next();
               if (helpptr == partial_child)
                                       // partial_child and full children
                                       // are adjacent.
                  cons_sequence = TRUE;
            }// while
         }// if
      }// if

      
      else
                                       // node_ptr does not have any full
                                       // children.
      {
         if (!node_ptr->partial_children->stack_empty())
                                       // node_ptr has a partial child.
                                       // Check if the partial child
                                       // is an endmost child of node_ptr.
         {
            node_ptr->partial_children->start_at_bottom();
            partial_child = node_ptr->partial_children->read_next();
            node_ptr->endmost_children->start_at_bottom();
            while (!node_ptr->endmost_children->read_last())
            {
               helpptr = node_ptr->endmost_children->read_next();
               if (helpptr == partial_child)
                                       // partial_child is endmost child.
                  cons_sequence = TRUE; 
            }// while
         }// if
      }// else
      if (cons_sequence)
         removeblock(node_ptr,is_root,seq_start,seq_end);
      return cons_sequence;
   }// else
}// template_Q2



/************************************************************************
                          template_Q3
************************************************************************/ 

int template_Q3(
                pq_node *node_ptr
               )

   // Template Matching for a Q-node with empty children at both
   // ends and full children in the middle, forming a consecutive sequence.
   // There may be at most two partial children, one at each side of the
   // consecutive sequence of the full children.
   // The Q-node has to be the root of the pertinent subtree.
{
   pq_node   *full_child = NULL;     // arbitrary full child of node_ptr
   pq_node   *full_start = NULL;     // denotes one end of the consecutive
                                     // sequence of full children
   pq_node   *full_end = NULL;       // denotes the other end of the
                                     // consecutive sequence of full children
   pq_node   *partial_1 = NULL;
   pq_node   *partial_2 = NULL;
   int       cons_sequence = FALSE;  // True, if all pertinent children form
                                     // a consecutive sequence.
   int       found = FALSE;
   

   if (node_ptr->typ != Q_NODE || node_ptr->partial_children->count() >= 3)
      return FALSE;
      
   else
                                     // node_ptr does have at most
                                     // two partial children.
   {
      if (!node_ptr->full_children->stack_empty())
                                     // node_ptr does have full children.
                                     // Check if full children and partial
                                     // child form a consecutive sequence. If
                                     // not, the tree is not reducible.
      {
         node_ptr->full_children->start_at_bottom();
         full_child = node_ptr->full_children->read_next();
         cons_sequence = checkchain(node_ptr,full_child,&full_start,&full_end);

         if (cons_sequence)
         {
            node_ptr->partial_children->start_at_bottom();
            while (!node_ptr->partial_children->read_last())
                                      // Check if the consecutive sequence
                                      // of full children is adjacent to
                                      // the partial nodes, if there are any.
            {
               partial_1 = node_ptr->partial_children->read_next();
               full_start->imm_siblings->start_at_bottom();
               full_end->imm_siblings->start_at_bottom();
               found = FALSE;
               while (!full_start->imm_siblings->read_last())
                  if (full_start->imm_siblings->read_next() == partial_1)
                                      // Partial node is adjacent to the
                                      // consecutive sequence of full children.
                     found = TRUE;

               while (!full_end->imm_siblings->read_last())
                  if (full_end->imm_siblings->read_next() == partial_1)
                                      // Partial node is adjacent to the
                                      // consecutive sequence of full children.
                     found = TRUE;
               if (!found)
                  cons_sequence = found;
            }// while
         }// if
      }// if

      
      else if (node_ptr->partial_children->count() == 2)
                                      // node_ptr does not have any full
                                      // children, but 2 partial children.
                                      // Check if they are adjacent.
      {
         node_ptr->partial_children->start_at_bottom();
         partial_1 = node_ptr->partial_children->read_next();
         partial_2 = node_ptr->partial_children->read_next();
         partial_1->imm_siblings->start_at_bottom();
         while (!partial_1->imm_siblings->read_last())
            if (partial_1->imm_siblings->read_next() == partial_2)
               found = TRUE;
         cons_sequence = found;
      }// else if
      int is_root = TRUE;
      if (cons_sequence)
         removeblock(node_ptr,is_root,full_start,full_end);
      return cons_sequence;
   }// else
}// template_Q3
         
                     

/************************************************************************
                     P3_copy_full_ch_to_partial
************************************************************************/ 
                      
void P3_copy_full_ch_to_partial(
                                pq_node *node_ptr,
                                pq_node *partial_child,
                                stack<pq_node*> *pert_nodes
                               )

   // Procedure only used by template P3. Creates a new full P-node
   // and copies the full children of node_ptr to the new P-node.
   // The new P-node will be included into the tree as child of the
   // partial_child, pointing to the newly introduced Q-node of template P3.
{
   pq_node  *new_node = NULL;
   pq_node  *empty_child;

   if (node_ptr->full_children->count() > 0)
                                     // There are full children to be copied.
   {
      node_ptr->child_count = node_ptr->child_count -
                              node_ptr->full_children->count();
                              

      new_node = copy_full_children_from_P_to_P_node(node_ptr->full_children,
                                          pert_nodes);

                                     // Update partial_child.
      partial_child->child_count++;
      partial_child->full_children->push(new_node);
                                     // Update sibling pointers.
      partial_child->endmost_children->start_at_bottom();
      empty_child = partial_child->endmost_children->read_next();
      empty_child->imm_siblings->push(new_node);
      new_node->imm_siblings->push(empty_child);
      partial_child->endmost_children->push(new_node);
      new_node->parent = partial_child;
   }// if
}// P3_copy_full_ch_to_partial
     


/************************************************************************
                     copy_full_ch_to_partial
************************************************************************/ 

void copy_full_ch_to_partial(
                             pq_node *node_ptr,pq_node *partial_child,
                             stack<pq_node*> *pert_nodes
                            )

   // Copies the full children of node_ptr to a new P-node, if necessary,
   // and adds this new P-node as an endmost child to the partial node
   // partial_child on the side, where the pertinent nodes are.
{
   pq_node  *new_node = NULL;
   pq_node  *helpnode = NULL;
   int      found = FALSE;

   if (node_ptr->full_children->count() > 0)
                                        // There are some full children to
                                        // be copied.
   {
      node_ptr->child_count = node_ptr->child_count -
                              node_ptr->full_children->count();

      new_node = copy_full_children_from_P_to_P_node(node_ptr->full_children,
                                             pert_nodes);

                                         // Introduce new_node as endmost
                                         // child of the partial Q-node.
      partial_child->child_count++;
      partial_child->full_children->push(new_node);
      partial_child->endmost_children->start_at_bottom();
      while (!partial_child->endmost_children->read_last() && !found)
                                          // Check at which side the
                                          // pertinet nodes hide.
      {
         helpnode = partial_child->endmost_children->read_next();
         if (helpnode->label == FULL)
            found = TRUE;
      }// while
      if(found)
      {
         partial_child->endmost_children->search_and_change
                                         (helpnode,new_node);
         helpnode->imm_siblings->push(new_node);
         new_node->imm_siblings->push(helpnode);
         new_node->parent = partial_child;
      }// if
      else
         printf("copy_full_ch_to_partial: ERROR Endmost_child not found\n");
   }// else
}// copy_full_ch_to_partial



/************************************************************************
                  copy_full_children_from_P_to_P_node
************************************************************************/

pq_node* copy_full_children_from_P_to_P_node(
                                             stack<pq_node*> *S,
                                             stack<pq_node*> *pert_nodes
                                            )

   // Copies the full children of a P-node that are stored in the stack S
   // to a P-node new_node, if there is more than one full child.
   // If there is just one child, that child will be the new_node.
{
   pq_node  *new_node = NULL;
   pq_node  *oldson = NULL;
   pq_node  *firstson = NULL;
   pq_node  *aktual_son = NULL;


   if (S->count() == 1)
                                     // There is just one full child. So no
                                     // no new P-node is created. The full
                                     // child is copied as child to the
                                     // partial_child.
   {
      new_node = S->pop();
      remove_P_imm_sib(new_node);
   }// if
   
   else
                                     // There is more than one full child,
                                     // so a new P-node is created.
   {
      new_node = new pq_node(0,0,node_count++,P_NODE,FULL);
      pert_nodes->push(new_node);
                                    // Pop the nodes out of the stack S
                                    // and introduce them as sons of new_node.
                                    // Popping them out of the stack implies
                                    // at the same time that they are removed
                                    // from the full_children stack of the
                                    // P-node, that they used to belong to.
      new_node->pert_child_count = S->count();
      new_node->child_count = S->count();
                                    // The first node is copied separately,
                                    // since we need the pointer to it for
                                    // setting the pointers to the siblings
                                    // of the next full nodes.
      firstson = S->pop();
      remove_P_imm_sib(firstson);
      new_node->full_children->push(firstson);
      firstson->parent = new_node;
      oldson = firstson;
      while (!S->stack_empty())
      {
         aktual_son = S->pop();
         remove_P_imm_sib(aktual_son);
         new_node->full_children->push(aktual_son);
         oldson->sib_next = aktual_son;
         aktual_son->sib_pre = oldson;
         aktual_son->parent = new_node;
         oldson = aktual_son;
      }// while
                                    // Update the last sibling pointers 
                                    // and update the father pointer to the
                                    // doubly-linked list of children.
      firstson->sib_pre = oldson;
      oldson->sib_next = firstson;
      new_node->child_sib_list = firstson;
      firstson->sib_list_father = new_node;
   }// else
   return new_node;
}// copy_full_children_from_P_to_P_node
   


/************************************************************************
                          check_chain
************************************************************************/ 

int checkchain(
               pq_node *node_ptr,
               pq_node *first_full,
               pq_node **seq_start,
               pq_node **seq_end
              )

   // Checks whether all full nodes of node_ptr form a consecutive
   // sequence. If so, the procedure gives TRUE as a result, otherwise FALSE.
   // first_full is just a full child, being detected by
   // the procedures template_Q2 and template_Q3, that gives a
   // position to start searching for a consecutive sequence.
   // The two pointers seq_start and seq_end are used to denote
   // the first and the last node of the consecutive sequence, if
   // there is any.
{
   int      full_count = 0;         // Counts the number of full children
                                    // that are discovered. This is necessary
                                    // since this procedure is used by
                                    // Q2 and Q3, where in Q3 first_full
                                    // can point to any full child in
                                    // the front.
   int      not_full = FALSE;       // TRUE when encountering an empty child
   pq_node  *aktual_ptr;            // aktual node which is examined
   pq_node  *left_next = NULL;      // next node to be examined on the
                                    // left side of first_full
   pq_node  *left_old = NULL;       // node that was examined right befor
                                    // aktual_ptr on left side
   pq_node  *right_next = NULL;     // next node to be examined on the right
                                    // side of first_full
   pq_node  *right_old = NULL;      // node that was examined right befor
                                    // aktual_ptr on the right side. right_next
                                    // and right_old are not necessary
                                    // for template Q2.

   
   full_count = node_ptr->full_children->count();
   full_count--;                    // first_full does have a FULL label.
   first_full->imm_siblings->start_at_bottom();

   
   if (!first_full->imm_siblings->read_last())
                                    // Check the left side of first_full.
   {
      left_next = first_full->imm_siblings->read_next();
      if (left_next->label == FULL)
         full_count--;
      else
         not_full = TRUE;
      if (not_full)
         (*seq_end) = first_full;
      else
      {
         left_old = first_full;
         aktual_ptr = left_next;

         while (full_count > 0 && !not_full)
                                    // There are still full children to be
                                    // counted, and no empty child has been
                                    // encountered on this side.
         {
            aktual_ptr->imm_siblings->start_at_bottom();
            left_next = aktual_ptr->imm_siblings->read_next();
            if (left_next == left_old)
               left_next = aktual_ptr->imm_siblings->read_next();
            if (left_next->label == FULL)
               full_count--;
            else
               not_full = TRUE;
            left_old = aktual_ptr;
            aktual_ptr = left_next;
         }// while
         if (aktual_ptr->label == FULL)
            (*seq_end)  = aktual_ptr;
         else if (left_old->label == FULL)
            (*seq_end) = left_old;
         else
            printf("checkchain: ERROR searching con seq in Q2 or Q3.\n");
      }// else
   }// if


   not_full = FALSE;
   if (!first_full->imm_siblings->read_last())
                                     // Check right side of first_full.
   {
      right_next = first_full->imm_siblings->read_next();
      if (right_next->label == FULL)
         full_count--;
      else
         not_full = TRUE;
      if (not_full)
         (*seq_start) = first_full;
      else
      {
         right_old = first_full;
         aktual_ptr = right_next;

         while (full_count > 0 && !not_full)
                                    // There are still full children to be
                                    // counted, and no empty child has been
                                    // encountered on this side.
         {
            aktual_ptr->imm_siblings->start_at_bottom();
            right_next = aktual_ptr->imm_siblings->read_next();
            if (right_next == right_old)
               right_next = aktual_ptr->imm_siblings->read_next();
            if (right_next->label == FULL)
               full_count--;
            else
               not_full = TRUE;
            right_old = aktual_ptr;
            aktual_ptr = right_next;
         }// while
         if (aktual_ptr->label == FULL)
            (*seq_start) = aktual_ptr;
         else if (right_old->label == FULL)
            (*seq_start) = right_old;
         else
            printf("checkchain: ERROR searching con seq in Q2 or Q3.\n");
      }// else
   }// if

   if (full_count == 0)
                                    // All full children occupy a consecutive
                                    // sequence.
      return TRUE;

   else
      return FALSE;
}// checkchain

   

/************************************************************************
                          remove_P_imm_sib
************************************************************************/ 

void remove_P_imm_sib(
                      pq_node *node_ptr
                     )

   // Removes node_ptr from the doubly linked list of his siblings,
   // if his parent is a P-node.
{
   if (node_ptr->sib_list_father != NULL)
                                   // Check first if node_ptr is connected
                                   // with its parent via the extra pointer
                                   // of the doubly linked list.
                                   // If so, link the next sibling of
                                   // node_ptr with the parent.
   {
      node_ptr->sib_list_father->child_sib_list = node_ptr->sib_next;
      node_ptr->sib_next->sib_list_father = node_ptr->sib_list_father;
      node_ptr->sib_list_father = NULL;
   }// if

                                   // Now remove node_ptr from the list.
   if (node_ptr->sib_next != NULL)
      node_ptr->sib_next->sib_pre = node_ptr->sib_pre;
   if (node_ptr->sib_pre != NULL)
      node_ptr->sib_pre->sib_next = node_ptr->sib_next;
   node_ptr->sib_next = NULL;
   node_ptr->sib_pre = NULL;
}// remove_P_imm_sib



/************************************************************************
                          remove_Q_imm_sib
************************************************************************/ 

void remove_Q_imm_sib(
                      pq_node *node_ptr
                     )

   // Removes node_ptr from the linked list of its immediate siblings
   // when its parent is a Q-node. This is done by emptying the imm_siblings
   // stack of node_ptr and connecting the two immediate siblings
   // as adjacent siblings or, if node_ptr was an endmost child of the
   // Q-node, making the next sibling endmost.
{
   pq_node  *helpptr = NULL;
   pq_node  *sibling_1 = NULL;     // first sibling of node_ptr
   pq_node  *sibling_2 = NULL;     // second sibling of node_ptr,
                                   // if it exists
   int      found = FALSE;
   
   stack<pq_node*>  *helpstack = new stack<pq_node*>;

   node_ptr->parent->endmost_children->start_at_bottom();
                                   // Check if node_ptr is endmost child.
   while (!node_ptr->parent->endmost_children->read_last())
   {
      helpptr = node_ptr->parent->endmost_children->read_next();
      if (helpptr == node_ptr)
         found = TRUE;            
   }// while

   helpptr = NULL;
   if (found)
                                   // node_ptr is endmost child.
                                   // Make sibling_1 be endmost child.
   {  
      sibling_1 = node_ptr->imm_siblings->pop();
                                   // node_ptr has only one sibling.
                                   // imm_sibling stack of node_ptr now
                                   // empty.
      node_ptr->parent->endmost_children->search_and_change
                                          (node_ptr,sibling_1);
                                   // sibling_1 now is endmost child.
                                   // Remove node_ptr from imm_sibling
                                   // stack of sibling_1.
      while (!sibling_1->imm_siblings->stack_empty())
      {
         helpptr = sibling_1->imm_siblings->pop();
         helpstack->push(helpptr);
      }// while
      while (!helpstack->stack_empty())
      {
          helpptr = helpstack->pop();
          if (helpptr != node_ptr)
             sibling_1->imm_siblings->push(helpptr);
      }// while
   }// if
   
   else
                                    // node_ptr is not endmost child.
                                    // Get the two siblings of node_ptr
                                    // and make them siblings.
   {
      sibling_1 = node_ptr->imm_siblings->pop();
      sibling_2 = node_ptr->imm_siblings->pop();
      sibling_1->imm_siblings->search_and_change(node_ptr,sibling_2);
      sibling_2->imm_siblings->search_and_change(node_ptr,sibling_1);
   }// else

   delete helpstack;
}// remove_Q_imm_sib

   

/************************************************************************
                          change_Q_imm_sib
************************************************************************/ 

void change_Q_imm_sib(
                      pq_node *old_node,
                      pq_node *new_node,
                      pq_node *parent
                     )

   // Replaces old_node by new_node in the immediate siblings lists of
   // old_nodes adjacent siblings, when its parent is a Q-node.
{
   pq_node   *sibling;                // Pointer for the siblings of old_node.

                                      // Check if old_node is endmost child
                                      // of the Q-node. If so replace it
                                      // by new_node.
   parent->endmost_children->search_and_change(old_node,new_node);

                                      // Get the siblings of old_node
                                      // and make them siblings of new_node.
   new_node->imm_siblings->set_empty();
   while (!old_node->imm_siblings->stack_empty())
   {                                  // Get sibling
      sibling = old_node->imm_siblings->pop();
                                      // Replace in the imm_siblings stack
                                      // of sibling old_node by new_node.
      sibling->imm_siblings->search_and_change(old_node,new_node);
                                      // Make sibling neighbor of new_node.
      new_node->imm_siblings->push(sibling);
   }// while
}// change_Q_imm_sib


     

/************************************************************************
                          change_P_imm_sib
************************************************************************/ 

void change_P_imm_sib(
                      pq_node *old_node,
                      pq_node *new_node
                     )

   // Replaces old_node by new_node in the doubly linked list of old_nodes
   // siblings, when its parent is a P-node.
{

   if (old_node->sib_list_father != NULL)
                                      // old_node is connected to its parent
                                      // via the extra pointer of the doubly
                                      // linked list. Link new_node by this
                                      // pointer to the P-node.
   {
      old_node->sib_list_father->child_sib_list = new_node;
      new_node->sib_list_father = old_node->sib_list_father;
      old_node->sib_list_father = NULL;
   }// if

                                       // Predecessor of old_node in the list  
   old_node->sib_pre->sib_next = new_node;
   new_node->sib_pre = old_node->sib_pre;
   old_node->sib_pre = NULL;

                                       // Successor of old_node in the list
   old_node->sib_next->sib_pre = new_node;
   new_node->sib_next = old_node->sib_next;
   old_node->sib_next = NULL;
}// change_P_imm_sib



/************************************************************************
                          removeblock
************************************************************************/ 

void removeblock(
                 pq_node *node_ptr,
                 int is_root,
                 pq_node *,
                 pq_node *
                )

   // Procedure used by template_Q2 and template Q_3.
   // Includes the children of a partial child into the sequence of
   // children of node_ptr.
{
   pq_node  *partial_1 = NULL;        // 1. partial child
   pq_node  *end_full_1= NULL;        // full endmost child of partial_1
   pq_node  *end_empty_1= NULL;       // empty endmost child of partial_1

   pq_node  *partial_2 = NULL;        // 2. partial child
   pq_node  *end_full_2 = NULL;       // full endmost child of partial_2
   pq_node  *end_empty_2 = NULL;      // empty endmost_child of partial_2
   
   pq_node  *sib_full_1 = NULL;       // full sibling of partial_1
   pq_node  *sib_partial_1 = NULL;    // partial sibling of partial_1
   pq_node  *sib_empty_1 = NULL;      // empty sibling of partial_1

   pq_node  *sib_full_2 = NULL;       // full sibling of partial_2
   pq_node  *sib_partial_2 = NULL;    // partial sibling of partial_2
   pq_node  *sib_empty_2 = NULL;      // empty sibling of partial_2 if
                                      // partial_2 exists, else second
                                      // empty sibling of partial_1, if there
                                      // are no full children of node_ptr.
   int   endmost_check = FALSE;       // True if partial_1 is endmost child
                                      // of node_ptr.

   pq_node  *helpptr = NULL;
   


   
   
   node_ptr->label = PARTIAL;
   if (!is_root)
      node_ptr->parent->partial_children->push(node_ptr);
      
   if (!node_ptr->partial_children->stack_empty())
                                        // Get a partial child.
   {
      partial_1 = node_ptr->partial_children->pop();
      partial_1->endmost_children->start_at_bottom();
      while (!partial_1->endmost_children->read_last())
                                        // Check on which side the full
                                        // children hide.
      {
         helpptr = partial_1->endmost_children->read_next();
         if (helpptr->label == FULL)
            end_full_1 = helpptr;
         else if (helpptr->label == EMPTY)
            end_empty_1 = helpptr;
      }// while

                                        // Check what kind of immediate
                                        // siblings partial_1 has.
      partial_1->imm_siblings->start_at_bottom();
      helpptr = NULL;
      while (!partial_1->imm_siblings->read_last())
      {
         helpptr = partial_1->imm_siblings->read_next();
         if (helpptr->label == FULL)
            sib_full_1 = helpptr;
         else if (helpptr->label == EMPTY)
            sib_empty_1 = helpptr;
         else if (helpptr->label == PARTIAL)
            sib_partial_1 = helpptr;
      }// while
   }// if


   if (!node_ptr->partial_children->stack_empty())
                                       // There is a second partial child.
   {
      partial_2 = node_ptr->partial_children->pop();
      partial_2->endmost_children->start_at_bottom();
      
      helpptr = NULL;
      while (!partial_2->endmost_children->read_last())
                                       // Check at which end the full
                                       // children hide.
      {
         helpptr = partial_2->endmost_children->read_next();
         if (helpptr->label == FULL)
            end_full_2 = helpptr;
         else if (helpptr->label == EMPTY)
            end_empty_2 = helpptr;
      }// while

      partial_2->imm_siblings->start_at_bottom();
      helpptr = NULL;

                                      // Check what kind of immediate
                                      // siblings partial_2 has.
      while (!partial_2->imm_siblings->read_last())
      {
         helpptr = partial_2->imm_siblings->read_next();
         if (helpptr->label == FULL)
            sib_full_2 = helpptr;
         else if (helpptr->label == EMPTY)
            sib_empty_2 = helpptr;
         else if (helpptr->label == PARTIAL)
            sib_partial_2 = helpptr;
      }// while
   }// if


   

   if (partial_1 != NULL && partial_2 != NULL)
                                       // There are two partial children.
                                       // Make a type A child of it with
                                       // full children inbetween.
                                       
   {
      if (sib_full_1 != NULL && sib_full_2 != NULL)
                                       // There are full children  between
                                       // the 2 partial nodes.
                                       // Connect the full children
                                       // between the 2 partial children
                                       // with the full endmost children
                                       // of the 2 partial nodes.
      {
         sib_full_1->imm_siblings->search_and_change(partial_1,end_full_1);
         end_full_1->imm_siblings->push(sib_full_1);
         sib_full_2->imm_siblings->search_and_change(partial_2,end_full_2);
         end_full_2->imm_siblings->push(sib_full_2);
      }// if

      else if (sib_partial_1 == partial_2)
                                       // There are no full children between
                                       // the 2 partial nodes. Connect the
                                       // full endmost children of the
                                       // partial nodes as siblings.
      {
         end_full_1->imm_siblings->push(end_full_2);
         end_full_2->imm_siblings->push(end_full_1);
      }// else if
                                      // Include the children of the 
                                      // partial children with their
                                      // full nodes inbetween into
                                      // the sequence of the children of
                                      // Q-node node_ptr.
      if (sib_empty_1 == NULL)
                                      // partial_1 is endmost child of
                                      // node_ptr. Make the empty endmost
                                      // child of partial_1 be the new endmost
                                      // child of node_ptr.
         node_ptr->endmost_children->search_and_change(partial_1,end_empty_1);
                                
      else
                                      // partial_1 is not endmost child.
      {
         sib_empty_1->imm_siblings->search_and_change(partial_1,end_empty_1);
         end_empty_1->imm_siblings->push(sib_empty_1);
      }// else

      if (sib_empty_2 == NULL)
                                      // partial_2 is endmost child of
                                      // node_ptr. Make the empty endmost
                                      // child of partial_2 be the new endmost
                                      // child of node_ptr.
         node_ptr->endmost_children->search_and_change(partial_2,end_empty_2);
                                
      else
                                      // partial_2 is not endmost child.
      {
         sib_empty_2->imm_siblings->search_and_change(partial_2,end_empty_2);
         end_empty_2->imm_siblings->push(sib_empty_2);
      }// else

      while (!partial_2->full_children->stack_empty())
      {
         helpptr = partial_2->full_children->pop();
         node_ptr->full_children->push(helpptr);
      }// while
      node_ptr->child_count = node_ptr->child_count +partial_2->child_count - 1;
                              
      mark_killed(partial_2,node_ptr);

      while (!partial_1->full_children->stack_empty())
      {
         helpptr = partial_1->full_children->pop();
         node_ptr->full_children->push(helpptr);
      }// while
      node_ptr->child_count = node_ptr->child_count +partial_1->child_count - 1;
                               
      mark_killed(partial_1,node_ptr);

   }// if




   else if (partial_1 != NULL)
                                      // There is one partial child.
   {
      node_ptr->endmost_children->start_at_bottom();
                                      // Check if partial_1 is endmost child
                                      // of node_ptr.
      while (!node_ptr->endmost_children->read_last())
      {
         helpptr = node_ptr->endmost_children->read_next();
         if (helpptr == partial_1)
                                      // partial_1 is endmost child.
            endmost_check = TRUE;
      }// while
      
   
      if (sib_full_1 != NULL)
                                      // There are full children on one
                                      // side of the partial node.
                                      // Connect the full children with
                                      // the full endmost child of partial_1.
      {
         sib_full_1->imm_siblings->search_and_change(partial_1,end_full_1);
         end_full_1->imm_siblings->push(sib_full_1);
      }// if

   
      else if (!endmost_check)
                                      // There are not any full children
                                      // and partial_1 is not endmost.
                                      // So get the 2nd empty sibling
                                      // of partial_1 and connect it
                                      // to the full endmost child
                                      // of partial_1.
      {
         partial_1->imm_siblings->start_at_bottom();
         while (!partial_1->imm_siblings->read_last())
         {
            helpptr = partial_1->imm_siblings->read_next();
            if (helpptr != sib_empty_1)
               sib_empty_2 = helpptr;
         }// while
      
         sib_empty_2->imm_siblings->search_and_change(partial_1,end_full_1);
         end_full_1->imm_siblings->push(sib_empty_2);
      }// else if
   

      else
                                      // partial_1 is endmost child
                                      // and there are no full children.
                                      // Make the full endmost child of 
                                      // partial_1 be the endmost_child
                                      // of node_ptr.
         node_ptr->endmost_children->search_and_change(partial_1,end_full_1);
         


      if (sib_empty_1 == NULL)
                                      // There are no empty children.
                                      // partial_1 is endmost child of
                                      // node_ptr. Make the empty endmost
                                      // child of partial_1 be the new endmost
                                      // child of node_ptr.
         node_ptr->endmost_children->search_and_change(partial_1,end_empty_1);
                                
      else
                                      // There are empty children. So connect
                                      // the empty endmost child of partial_1
                                      // with sib_empty_1.
      {
         sib_empty_1->imm_siblings->search_and_change(partial_1,end_empty_1);
         end_empty_1->imm_siblings->push(sib_empty_1);
      }// else   
                                                              
      while (!partial_1->full_children->stack_empty())
      {  helpptr = partial_1->full_children->pop();
         node_ptr->full_children->push(helpptr);
      }// while
      node_ptr->child_count = node_ptr->child_count +
                               partial_1->child_count - 1;
      mark_killed(partial_1,node_ptr);

   }// else if
   
   // else node_ptr does not have partial children. Then nothing is to do.
   
}// removeblock

      
      
/************************************************************************
                          check_if_one_son
************************************************************************/ 

void check_if_one_son(
                      pq_node *father,
                      pq_node *son
                     )

   // Checks, if father has only one child. If so, the only
   // child is connected to its grandparent, as long as father is
   // not the root of the tree. The father is completely removed
   // from the tree and destroyed.
{
   if (father->child_count == 1)
   {
      son->parent = father->parent;
      son->sib_list_father = NULL;
      if (father->parent != NULL)
                                      // father is not the root.
      {
         if (father->imm_siblings->stack_empty())
            change_P_imm_sib(father,son);
         else
            change_Q_imm_sib(father,son,father->parent);
      }// if
      else
         root = son;
      destroy(father);
   }// if
}// check_if_one_son



/************************************************************************
                          mark_killed
************************************************************************/ 

void mark_killed(
                 pq_node *node_ptr,
                 pq_node *father
                )

   // Marks node_ptr killed. This means node_ptr is removed from
   // the PQ-tree but not destroyed, since we need the information about
   // this node when finding the father pointers of the children of a Q-node.
{
   node_ptr->parent = father;
   node_ptr->label = KILLED;
}// mark_killed

   

/************************************************************************
                          destroy
************************************************************************/ 

void destroy(
             pq_node *node_ptr
            )
{
   node_ptr->label = TO_BE_DELETED;
}// destroy



