#include <LEDA/graph.h>
#include <LEDA/node_array.h>

double rand_double() {
  return (double)rand_int(0,999)/1000;
}

void random_hierarchy(graph& G,
		      node_array<int>& rank,node_array<int>& pos,
		      int numberOfNodes,int numberOfEdges,
                      bool planar,bool singleSource) {

  for(int i=0;i<numberOfNodes;i++) G.new_node();
  rank.init(G);
  pos.init(G);
  node *nodes=new node[numberOfNodes];
  int *first=new int[numberOfNodes+1];
  int *leftN=new int[numberOfNodes];
  int *rightN=new int[numberOfNodes];

  /** Place nodes **/

  node v;
  double r;
  int numberOfLayers=0,count=0;
  first[0]=0;
  forall_nodes(v,G) {
    nodes[count]=v;
    rank[v]=numberOfLayers;
    pos[v]=count-first[numberOfLayers];
    r=rand_double();
    if(!count&&singleSource||count==numberOfNodes-1||r*r*numberOfNodes<1) {
      numberOfLayers++;
      first[numberOfLayers]=count+1;
    }
    count++;
  }

  /** Determine allowed neighbours **/

  int l,n1,n2;
  double rat;
  for(l=1;l<numberOfLayers;l++) {
    if(planar) {
      rat=(double)(first[l]-first[l-1])/(double)(first[l+1]-first[l-1]);
      n1=first[l-1]; n2=first[l];
      leftN[n2]=n1;
      while(n1<first[l]&&n2<first[l+1]) {
        if(n1!=first[l]-1&&(n2==first[l+1]-1||rand_double()<rat)) 
	  n1++;
        else {
          rightN[n2]=n1;
          if(++n2<first[l+1]) leftN[n2]=n1;
        }
      }
    }
    else for(n2=first[l];n2<first[l+1];n2++) {
      leftN[n2]=first[l-1];
      rightN[n2]=first[l]-1;
    }
  }

  /** Insert edges **/

  if(numberOfLayers) {
    int maxEdges=0;
    bool connected;
    for(n2=first[1];n2<numberOfNodes;n2++) maxEdges+=rightN[n2]-leftN[n2]+1;
    double density=(double)numberOfEdges/(double)maxEdges;
    for(n2=first[1];n2<numberOfNodes;n2++) {
      connected=!singleSource;
      for(n1=leftN[n2];n1<=rightN[n2];n1++) if(rand_double()<density) {
        connected=1;
        G.new_edge(nodes[n1],nodes[n2]);
      }
      if(!connected) 
        G.new_edge(nodes[rand_int(leftN[n2],rightN[n2])],nodes[n2]);
    }
  }

  delete[] rightN;
  delete[] leftN;
  delete[] first;
  delete[] nodes;

}


