/************************************************************************

         Filename   :   st_num.cc

         Version    :   09.1994

         Author     :   Sebastian Leipert

         Language   :   C++


 File containing all procedures for computing an st-numbering of
 a biconnected component of the graph.

************************************************************************/ 


#include <AGD/impl/MaxPlanSub/proto.h>

     
extern  int      verticecount;
extern  int      edgecount;
extern  pq_node  *root;

extern  int      _printDuringReduction;
extern  int      _printPQTreeIterative;
extern  int      _iterationNumber;
extern  char     _debugInformation[INFOSIZE];


/************************************************************************
                          st_number
************************************************************************/ 

int st_number(
              adjlists *block,
              int **st_no,
              int **no_st
             )

  // Procedure for computing an st-numbering of a biconnected component.
{


  int    count;
  //  int    node;
  int    sink;        
  int    source;      
  int    top_entry;
  int    helpnode;
  int    *father;     // array containing the fathers of the vertices
  int    *low;        // array containing the LOW-Numbers of the vertices
  int    *dfn;        // array containing the DFN-Numbers of the vertices
  int    *marked;
  
  adjlists tree(verticecount); // stores the depth-first-search tree
  adjlists back(verticecount); // stores the back-edges of the depth-first-search tree
  stack<int> path;  // stores the path vertices, which have not been
                    // visited during the computation of the st-numbering

  count = sink = 1;
  father = (int *) calloc ((verticecount+1),sizeof(int));
  low = (int *) calloc ((verticecount+1),sizeof(int));
  dfn = (int *) calloc ((verticecount+1),sizeof(int));
  marked = (int *) calloc ((verticecount+1),sizeof(int));

                                        // Find first node of block.
                                        // Call is sink.
  while ((*block).node_degree(sink) == 0)
    sink++;

                                        // Compute the DFN and LOW numbers
                                        // of the block.
  st_search(block, &tree, &back, sink, &count, marked,
            father, low, dfn);
                                        // Find source.
  tree.start_at_first_node(sink);
  source = tree.read_next_node(sink);
  tree.delete_old_edge(sink);
  for ( int i = 0; i <= verticecount; i++)
     marked[i] = FALSE;
  marked[source] = TRUE;
  marked[sink] = TRUE;

  stack<int> nodestack;   // nodestack stores the vertices during the computation
                          // of the st-numbering.
  nodestack.push(sink);
  nodestack.push(source);
  count = 1;
  top_entry = nodestack.pop();
  while (top_entry != sink)
  {
     if (!st_path(&tree, &back, &path, top_entry, marked, father, dfn, low))
     {
        (*st_no)[top_entry] = count;
        (*no_st)[count] = top_entry;
        count++;
     }// if
     else
     {
        helpnode = path.pop();
        while (!path.stack_empty())
        {
           helpnode = path.pop();
           nodestack.push(helpnode);
        }// while
     }// else
     top_entry = nodestack.pop();
  }// while
  (*st_no)[sink] = count;
  (*no_st)[count] = sink;

  if (_printPQTreeIterative)
  {
     FILE      *output;
     output = fopen(_debugInformation,"a");
     for (int j = 1; j <= verticecount; j++)
     fprintf(output,"Knoten %d : ST-Nummer %d\n",j,(*st_no)[j]);
     fclose(output);
  }
  
				    // change the numbers of the vertices
				    // according the st-numbering
  change_numbers(block,*st_no,count);

  delete[] father;
  delete[] low;
  delete[] dfn;
  delete[] marked;

  return count;

}// st_number



/************************************************************************
                          st_search
************************************************************************/ 

void st_search(
               adjlists *block,
               adjlists *tree,
               adjlists *back,
               int node,
               int *count,
               int *marked,
               int *father,
               int *low,
               int *dfn
              )

   // Procedure for computing the DFN and LOW numbers of a biconnected
   // component. Those numbers are needed for computing the st-numbering.
{


  int   neighbor;
  

  marked[node] = TRUE;
  dfn[node] = (*count);
  (*count)++;
  low[node] = dfn[node];
  (*block).start_at_first_node(node);
  while(!(*block).end_of_list(node))
  {
     neighbor = (*block).read_next_node(node); 
     if (!marked[neighbor])
     {
        (*tree).insert_edge(node,neighbor);
        (*block).mark_old_edge(node);
        father[neighbor] = node;
        st_search(block,tree,back,neighbor,count,marked,father,low,dfn);
        if (low[node] > low[neighbor])
           low[node] = low[neighbor];
     } // if
     else if (neighbor != father[node])
     {
        if (!(*block).old_edge_marked(node))
        {
           (*back).insert_edge(node,neighbor);
           (*block).mark_old_edge(node);
        }// if
        if (low[node] > dfn[neighbor])
           low[node] = dfn[neighbor];
     }// else if        
  }// while
}// st_search
        


/************************************************************************
                          st_path
************************************************************************/ 

int st_path(
            adjlists *tree,
            adjlists *back,
            stack<int> *path,
            int node,
            int *marked,
            int *father,
            int *dfn,
            int *low
           )

   // Procedure for computing a paht of egdes, which have not been visited
   // yet within the depth-first-search tree of the biconnected tree.
   // This path is put into a stack called path.

{


   int    backedge_up = FALSE;
   int    tree_edge = FALSE;
   int    backnode;
   int    new_treenode;
   int    old_treenode;
   int    neighbor;
   int    vater;
   int    helpnode;
   

   (*path).empty_stack();
   (*back).start_at_first_node(node);
   while (!(*back).end_of_list(node) && !backedge_up)
   {
      neighbor = (*back).read_next_node(node);
      if (dfn[neighbor] <= dfn[node])
                                        // A new backedge up was found.
         backedge_up = TRUE;
   }// while

                                        // If a backedge up was not 
                                        // found and (*back) is not empty,
                                        // get first edge. This must be a
                                        // backedge down. 

   (*tree).start_at_first_node(node);
   if ((*tree).node_degree(node) > 0)
      if ( (*tree).node_degree(node) > 1 ||
           (*tree).read_next_node(node) != father[node] )
                                         // Find out, if new tree-edge exists.
         tree_edge = TRUE;   

   if (backedge_up)
                                         // New backedge up exists.
   {
      (*back).delete_old_edge(node);
      (*path).push(node);
      (*path).push(neighbor);
      return TRUE;
   }// if
   else if (tree_edge)
                                        // New tree-edge exists.
   {  
      (*tree).start_at_first_node(node);
      neighbor = (*tree).read_next_node(node);
      if (neighbor == father[node])
         neighbor = (*tree).read_next_node(node);
      new_treenode = neighbor;
      old_treenode = node;
      marked[new_treenode] = TRUE;
      marked[old_treenode] = TRUE;
      (*path).push(old_treenode);
      (*path).push(new_treenode);
      (*tree).delete_old_edge(node);    // edge in path
      int found = FALSE;
      while (!found)
                                        // The backedge, that defines
                                        // LOW(neighbor) is not found.
      {
         (*back).start_at_first_node(new_treenode);
         while (!(*back).end_of_list(new_treenode) && !found)
         {
            backnode = (*back).read_next_node(new_treenode);
            if (dfn[backnode] == low[neighbor])
               found = TRUE;
         }// while
         if (found)
         {
            (*path).push(backnode);
            marked[backnode] = TRUE;
            (*back).delete_old_edge(new_treenode);  
                                        // edge in path 
         }// if
         else
         {
            old_treenode = new_treenode;
            (*tree).start_at_first_node(old_treenode);
            tree_edge = FALSE;
            while (!(*tree).end_of_list(old_treenode) && !tree_edge)
            {
               helpnode = (*tree).read_next_node(old_treenode);
               if (helpnode != father[old_treenode] &&
                  low[helpnode] == low[neighbor])
               {
                  tree_edge = TRUE;
                  new_treenode = helpnode;
               }// if
            }// while
            if (!tree_edge)
               printf("st_path: Error while computing the ST-Numbering\n");
            marked[new_treenode] = TRUE;
            (*tree).delete_old_edge(old_treenode);  
            (*path).push(new_treenode);
                                         // edge in path
         }// else
      }// while
      return TRUE;
   }// else if
   else if ((*back).node_degree(node) > 0)
                                         // New backedge down exists.
   {
      (*back).start_at_first_node(node);
      neighbor = (*back).read_next_node(node);
      (*path).push(node);
      (*path).push(neighbor);
      (*back).delete_old_edge(node);     // edge in path
      if (!marked[neighbor])
      {
         vater = father[neighbor];
         marked[neighbor] = TRUE;
         (*path).push(vater);
         (*tree).start_at_first_node(neighbor);
         while ((*tree).read_next_node(neighbor) !=  vater);
         (*tree).delete_old_edge(neighbor);
         while (!marked[vater])
         {
            neighbor = vater;
            marked[neighbor] = TRUE;
            vater = father[neighbor];
            (*path).push(vater);
            (*tree).start_at_first_node(neighbor);
            while ((*tree).read_next_node(neighbor) !=  vater);
            (*tree).delete_old_edge(neighbor);
         }// while
      }// if
      return TRUE;
   }// else if
   else
      return FALSE;
}// st_path
          
         

/************************************************************************
                          change_numbers
************************************************************************/ 

void change_numbers(
                    adjlists *block,
                    int *st_no,
                    int nodenmbr
                   )

   // After computing the st-numbering of a biconnected component,
   // the numbers of the nodes of the block have to be changed. 
   // This is exactly done by this procedure.
{


   int     node, neighbor;
   adjlists helpblock(nodenmbr);
   
   for ( node = 1; node <= verticecount; node++)
   {
      (*block).start_at_first_node(node);
      while (!(*block).end_of_list(node))
      {
         neighbor = (*block).read_next_node(node);
         helpblock.insert_edge(st_no[node],st_no[neighbor]);
         (*block).delete_old_edge(node);
      }// while
   }// for
   for ( node = 1; node <= nodenmbr; node++)
   {
      helpblock.start_at_first_node(node);
      while (!helpblock.end_of_list(node))
      {
         neighbor = helpblock.read_next_node(node);
         (*block).insert_edge(node,neighbor);
         helpblock.delete_old_edge(node);
      }// while
   }// for
}// change_numbers
