/*****************************************************************

            Filename    :  vbc_node.cc

	    Version     :  01.1997

	    Author      :  Sebastian Leipert

	    Language    :  C++

            Purpose     :  See .h File.
	    
******************************************************************/

#include <AGD/impl/TreeLayout/vbc_node.h>
#include <AGD/impl/TreeLayout/def.glb>
#include <string.h>


vbc_node::vbc_node(int num, double h, double w, double rad, int color, int under_color,
           double xco, double yco, double px, double mx,
           vbc_node *par,vbc_node *fch,vbc_node *lsi,vbc_node *rsi,vbc_node *lnei)
{
   nmber = num;
   radius = rad;
   height = h;
   width = w;
   displayed = FALSE;
   highlighted = FALSE;
   colour = color;
   underlying_colour = under_color;
   
   x = xco;
   y = yco;
   prelim_x = px;
   mod = mx;


   parent = par;
   firstchild = fch;
   leftsib =lsi;
   rightsib = rsi;
   leftneighb = lnei;
   information[0] = '\0';
   shortinfo[0] = '\0';
}


vbc_node::~vbc_node()
{

}



vbc_node*  vbc_node::get_parent() const
{
   return parent;
}

vbc_node*  vbc_node::get_firstchild() const
{
   return firstchild;
}

vbc_node*  vbc_node::get_leftsibling() const
{
   return leftsib;
}

vbc_node*  vbc_node::get_rightsibling() const
{
   return rightsib;
}

vbc_node*  vbc_node::get_leftneighbor() const
{
   return leftneighb;
}

int   vbc_node::get_nmber() const
{
   return nmber;
}

int   vbc_node::get_colour() const
{
   return colour;
}

int   vbc_node::get_underlying_colour() const
{
   return underlying_colour;
}

double   vbc_node::get_radius() const
{
   return radius;
}

double   vbc_node::get_height() const
{
   return height;
}

double   vbc_node::get_width() const
{
   return width;
}

double  vbc_node::get_Xcoord() const
{
   return x;
}

double  vbc_node::get_Ycoord() const
{
   return y;
}

double  vbc_node::get_prelim() const
{
   return prelim_x;
}

double  vbc_node::get_modifier() const
{
   return mod;
}

char*   vbc_node::get_information()
{
   return information;
}

char*   vbc_node::get_shortinfo()
{
   return shortinfo;
}

void   vbc_node::set_parent(vbc_node *put)
{
   parent = put;
}

void   vbc_node::set_firstchild(vbc_node *put)
{
   firstchild = put;
}

void   vbc_node::set_leftsibling(vbc_node *put)
{
   leftsib = put;
}

void   vbc_node::set_rightsibling(vbc_node *put)
{
   rightsib = put;
}

void   vbc_node::set_leftneighbor(vbc_node *put)
{
   leftneighb = put;
}

void  vbc_node::set_nmber(int m)
{
   nmber = m;
}

void  vbc_node::set_colour(int m)
{
   colour = m;
}

void  vbc_node::set_underlying_colour(int m)
{
   underlying_colour = m;
}

void  vbc_node::set_height(double m)
{
   height = m;
}

void  vbc_node::set_width(double m)
{
   width = m;
}

void  vbc_node::set_Xcoord(double m)
{
   x = m;
}

void  vbc_node::set_Ycoord(double m)
{
   y = m;
}

void  vbc_node::set_prelim(double m)
{
   prelim_x = m;
}

void  vbc_node::set_modifier(double m)
{
   mod = m;
}

void  vbc_node::set_information(char* info)
{
   strcpy(information,info);
}

void  vbc_node::set_shortinfo(char* info)
{
   strcpy(shortinfo,info);
}

void  vbc_node::add_information(char* info)
{
   strcat(information,info);
}

void  vbc_node::add_shortinfo(char* info)
{
   strcat(shortinfo,info);
}

int   vbc_node::is_leaf() const
{
   return ( firstchild == NULL ? TRUE : FALSE );
}

int   vbc_node::has_left_sib() const
{
   return ( leftsib != NULL ? TRUE : FALSE );
}

int   vbc_node::has_right_sib() const
{
   return ( rightsib != NULL ? TRUE : FALSE );
}

int   vbc_node::has_child() const
{
   return ( firstchild != NULL ? TRUE : FALSE);
}


int   vbc_node::is_displayed()const
{
   return displayed;
}

void  vbc_node::display(int number)
{
   displayed = number;
}


int   vbc_node::is_highlighted()const
{
   return highlighted;
}

void  vbc_node::highlight(int boolean)
{
   highlighted = boolean;
}


ostream &operator<<(ostream & output, const vbc_node &n)
{
   output << "x = " << n.x << endl
          << "y = " << n.y << endl
	  << "prelim_x = " << n.prelim_x << endl
	  << "mod = " << n.mod << endl
          << "parent = " << ((n.parent != NULL) ? n.parent : 0) << endl
          << "firstchild = " << ((n.firstchild != NULL) ? n.firstchild : 0) << endl
          << "leftsib = " << ((n.leftsib != NULL) ? n.leftsib : 0) << endl
          << "rightsib = " << ((n.rightsib != NULL) ? n.rightsib : 0) << endl
          << "leftneighb = " << ((n.leftneighb != NULL) ? n.leftneighb : 0) << endl;

   return output;
}
