#include <AGD/PlanAugOpt.h>
#include <AGD/impl/plaugment.h>


int MAKE_CONNECTED (graph&, bctree&, list<edge>&);

//***************************************************************
PlanAugOpt::PlanAugOpt() : PlanAug(),
heurist_optim_ (true),only_bac (false) { }

//***************************************************************
void PlanAugOpt::_call (graph& G, list<edge>& L)
{
//simple, preconditions checken ??
 bctree BC (&G, true, true); //BC is Block and Cutpoint tree of G
 if ( MAKE_CONNECTED (G, BC, L))
    BC.update (true);

 if ( BC.biconnected ()) // G a) was biconnected or 
                        //b) contains only biconnected components
	return;

//compute the lower bound (general augmentation) for the planar augmentation problem
  int AA_Optimum = Max (degree (BC.root ()) - 1,
               (int)ceil((float)BC.number_blocks_with_degree(1)/2));

//Try to solve the problem approximately

 list<edge> solution;
 list<node> node_solution;//I will save my first solution as a list of pairs of nodes

 if ( !only_bac ) {
   PlanAug::_call (G, solution);

   if ( solution.size () == AA_Optimum) {//The solution is optimal
      L.conc (solution); // L has edges added by MAKE_CONNECTED
      return;
   }
   else {// Save solution
      edge e;
      while ( !solution.empty ()) {
         e = solution.pop ();
         node_solution.push (source (e));
         node_solution.push (target (e));
         G.del_edge (e);
      }
   }
	
   streng_cond (false);
   PlanAug::_call (G, solution);

  if ( solution.size () == AA_Optimum) {//The solution is optimal
    L.conc (solution);
    return;    
  }
  else {
    edge e;
    if ( solution.size () < node_solution.size ()/2) {// Second try was better
      node_solution.clear ();

      forall (e, solution) {
        node_solution.push (source (e));
        node_solution.push (target (e));
      } 
    }
    while ( !solution.empty ()) //Reset G for Branch-and-Cut
       G.del_edge (solution.pop ());
  } 
 }

//Branch-and-Cut
  PLAU_MASTER OptAug (G, node_solution.size ()/2);
  OptAug.optimize ();
  
  if ( OptAug.found ())  //better solution found
  {
  	list<edge> OptSolution = OptAug.solution ();
        L.conc (OptSolution);
        heurist_optim_ = false;
  }
  else
  	while ( !node_solution.empty ()) //augment G with approximate solution
	{
           node v = node_solution.pop ();
	   L.push ( G.new_edge (v, node_solution.pop ()));
	}
}
/********************************************************************************/

