#include <LEDA/sortseq.h>
#include <LEDA/node_pq.h>
#include <AGD/impl/planarisator.h>
#include <abacus/active.h>

//****************************************************************************
PLANAR_SUB::PLANAR_SUB (ABA_MASTER *master)
        :ABA_SUB (master, 50.0, 1000.0, 50.0, true), branch (false), 
		vI(planarisator()->nVars ())
{
 neutralized = new h_array<PLANAR_CONSTRAINT*, bool> (false);
}
//*****************************************************************************
PLANAR_SUB::PLANAR_SUB (ABA_MASTER *master, ABA_SUB *father, ABA_BRANCHRULE *brule)
	 :ABA_SUB (master, father, brule ), branch (false), vI(((PLANAR_SUB*)father)->vI),
	  neutralized(((PLANAR_SUB*)father)->neutralized)
{
 neutralized = new h_array<PLANAR_CONSTRAINT*, bool> (*((PLANAR_SUB*)father)->neutralized);
}
//****************************************************************************
// 1) Finde alle Ungleichungen, die durch das letzte Setzen Variable auf 0 "neutralisiert" wurden
// 2) Entferne die Ungl. aus dem set der aktiven und markiere Sie als neutralisiert (sie knnen
//    bei der Suche im Pool nicht berprft werden).
// 3) Fr jede Variable, die in diesen neutralisierten Ungleichungen vorkommt, reduziere 
//    die "Wichtigkeit".
void PLANAR_SUB::initializeCons(int maxCon) {
  int k;
  ABA_SUB::initializeCons (maxCon);

  if ( master_->outLevel () == ABA_MASTER::Full )  {
 // Count and output the numbers of variables set to 1 and to 0
	int nSetTo1 = 0, nSetTo0 = 0;

	for (k = 0; k < nVar (); k++) {
		if ( fsVarStat (k)->status () == ABA_FSVARSTAT::SetToLowerBound) 
			nSetTo0++;
		if ( fsVarStat (k)->status () == ABA_FSVARSTAT::SetToUpperBound) 
			nSetTo1++;     
	}
	master()->out () << "# of set to 1.0 Variables: " << nSetTo1 << "\n";
	master()->out () << "# of set to 0.0 Variables: " << nSetTo0 << "\n";
  }
  
  if (((ABA_SETBRANCHRULE*)branchRule())->setToUpperBound())  return;
  
  master()->out () << "The last variable has been set to 0\n";
  int num = ((ABA_SETBRANCHRULE*)branchRule())->variable ();
  list<int> to_remove; 
	// Push into this list numbers of constraints that have been
	// neutralized due to last setting to 0

  for (int i = 0; i < nCon(); i++)  { // ??
	PLANAR_CONSTRAINT *con = (PLANAR_CONSTRAINT*)constraint(i);
	if (con->row()->nnz () - con->rhs () != 1 || (*neutralized)[con] )
		continue; //this is either Euler constraint or already neutralized constraint
	if (con->coeff (num)) {//if the last set variable (number num) is present in con
		to_remove.push (i);
		(*neutralized)[con] = true; //con can be ignored during the pool separation
		
		for (int j = 0; j < con->row()->nnz(); j++)
		//reduce the importance of all variables present in con.
			vI.reduceImportance (con->row()->support(j));
	}
  }
  if ( vI.Importance (num) < 0) {
  	cerr << "The Importance of a variable isn't correct\n";
	cerr <<  vI.Importance (num) << "  #" << num << "\n";
	exit (Fatal);
  }

  ABA_BUFFER<int> to_del (planarisator (), to_remove.size ());	
  to_remove.sort (); 

  while (!to_remove.empty ())
	to_del.push (to_remove.pop ());
  removeCons (to_del);
}
//*****************************************************************************
// 1) Check whether solution is integer. If not, return false
// 2) Remove edges with 0-value
// 3) Check whether graph is planar
// 4) Reset G (add the removed edges to the graph)

bool PLANAR_SUB::feasible ()
{
//  check_importances ();
  if ( !integerFeasible ())
    return false;

  list<int> solution;

//Remove from G all edges with value 0 in last solution.
  int i;
  for ( i=0; i < nVar (); i++)
    if ( master_->equal (xVal (i), 0.0)) {
      solution.push (i);
      remove_edge (i);
    }
  
  if ( !PLANAR (*Graph (), false)) {
    forall (i, solution)
      restore_edge (i);
    return false;
  }

  planarisator ()->update_solution (solution);

  forall (i, solution)
      restore_edge (i);

  return true;
}

//***************************************************************************
//*****************************************************************************
void PLANAR_SUB::filter_solution ( list<int>& large, list<int>& small )
{
  int i;
  if ( planarisator()->MoreSeparate ) { //then select in large all variables with non-zeros values
       for ( i = 0; i < nVar (); i++ ) {
	 if ( master_->equal (xVal (i), 0))		
	    small.push (i);
	 else
	    large.push (i);
       }
    return;
  }

  p_queue <double, int> vars;//Sort variables according to their values

  for ( i = 0; i < nVar (); i++ ) {
	if ( master_->equal (xVal (i), 0))
	  small.push (i);
	else
	  vars.insert (-(xVal (i)), edge_var(i)->number ());
  }
 
  double sum = 0;
  pq_item maxi;
  i = 0;

// Sum the greatest values as long as their sum is greater or equal then 
// the number_of_variables - 1.
  do {
     maxi = vars.find_min ();
     sum -= vars.prio (maxi);
     large.push (vars.inf (maxi));
     vars.del_item (maxi);
     i++;
  }  while ( sum > i - 1 + master_->eps () && !vars.empty ());

  if ( sum < i - 1 ) //The last variable exceeeds the limit.
	small.push ( large.pop ());
  
  while ( !vars.empty ()) {
  	small.push (vars.inf ( vars.find_min ()));
        vars.del_min ();
  }
}
  
//*************************************************************************
int PLANAR_SUB::separate () {
//Partition solution into small and large Variables (values)
  list<int> great, small;
  filter_solution (great, small);

  great.sort ();

  int i;

  //separate from Pool
  planarisator()->searchInPool->start();//Timer
  ABA_BUFFER<ABA_CONSTRAINT*> zero_slack (master_, planarisator()->CutPool->size ());

  int nCuts = SeparationFromCutsPool (great, zero_slack);
  planarisator()->searchInPool->stop();

  if ( nCuts ) {// Violated constraints were found
        planarisator()->separatedFromPool += nCuts;
	master()->out () << "Pool separation: " << nCuts << " constraints\n";
	return nCuts;
  }
  else
      if ( zero_slack.number ())  { //Destroy found nonviolated constraints
         planarisator()->destroyedFromPool += zero_slack.number ();
	 master()->out () << "Pool separation: " << zero_slack.number () << " equal constraints found\n";
	 for (int j = 0; j < zero_slack.number (); j++) {
	    if ( zero_slack[j] == nil )
	       continue;

	    int m = DestroyKuratowskiG ((PLANAR_CONSTRAINT*)zero_slack[j], great);
	    
	    if ( m >= 0) {
	       if ( MoreSeparate()) //remove all constraints from zero_slack, which have the
	                            // Variable number m
	           for ( i = j+1; i < zero_slack.number (); i++)
		      if (zero_slack[i] && zero_slack[i]->coeff (edge_var(m))==1)
		         zero_slack[i] = nil;
	       small.push (m);
	    }
	 }
  }

  forall ( i, small )
    remove_edge (i);

  list<edge> edges;
  list<node> nodes;
  node_array<int> deg;
  ABA_BUFFER <ABA_CONSTRAINT*> newCuts (master_, master_->maxConAdd());
  
  double val, sum = 0;
  bool violated = true;

 if ( !Graph()->make_map ()) {
     cerr << "The Graph isn't bidirected\n";
     exit (Fatal);
 }
 edge e;

 while ( !PLANAR ( *Graph(), edges, false ) && !newCuts.full () )  {
	planarisator()->nKuratowski++;

        PLANAR_CONSTRAINT* con = planarisator()->createCon (edges);
	incrImportances (con);
	(*neutralized)[con] = false;
	newCuts.push (con);
	small.push (DestroyKuratowskiG (con, great));
        remove_edge (small.head ());
	edges.clear ();
  }
  forall (i, small)
  	restore_edge(i);
  
  branch = true;

  for ( i = 0; i < newCuts.number (); i++)
      if ( newCuts[i]->violated (actVar (), xVal_)) {
        branch = false; //ueberpruefen
	break;
  }

  addCons (newCuts, planarisator ()->CutPool);
//  check_importances ();
  return newCuts.number ();
}
//*******************************************************************
// This little auxilliar function destroys a Kuratowski subdivision
// described by constraint con by removing from G an edge present in con 
// and having minimal value in last LP. The number of corresponding 
// KURATOWSKI_EDGE will be also removed from list l and returned.
int PLANAR_SUB::DestroyKuratowskiG (PLANAR_CONSTRAINT* con, list<int>& l)
{
   if ( level () == 1 && nIter_ == 1 ) {// first separation has not yet a LP-solution
	int res = con->row()->support (0);//remove first edge in con
	l.remove (res); 
        return res;
   }

   int min = con->find_min (xVal_);
   list_item li = l.search (min);
   if (li != nil )
       return l.del (li);
   else 
       return -1;//noch umschreiben
}
//*******************************************************************
bool PLANAR_SUB::exceptionBranch()
 {
    if ( !branch )
    	return false;
    	
    branch = false;
    return true;
 }
//******************************************************************
// This function tries to separate from Pool. First, we separate violated
// constraints and add them to addConbuffer_. Second we also search for
// constraints, that are non-violated but have zero slack (lhs = rhs).
// Such constrains we add to buffer zero_slack. 
// Return value: the number of !violated! constraints.

int PLANAR_SUB::SeparationFromCutsPool (list<int>& edges,ABA_BUFFER<ABA_CONSTRAINT*>& zero_slack) 
{
  int j, i, num = addConBuffer_->number ();
  zero_slack.clear ();
  ABA_STANDARDPOOL<ABA_CONSTRAINT, ABA_VARIABLE> *pool = planarisator()->CutPool;

  for (i = 0; i < pool->size (); i++)
  {
      PLANAR_CONSTRAINT *con = (PLANAR_CONSTRAINT*)(pool->slot(i)->conVar());
      if (con && ! ((*neutralized)[con]) && addConBufferSpace() > 0 ) {
     // neutralized means one of variables present in constraint con is set to 0
          PLANAR_CONSTRAINT::VIOLATE_STATUS stat = con->violated_or_equal (xVal_, edges);
          if (stat == PLANAR_CONSTRAINT::NOT_VIOLATED ) {
	     if (planarisator ()->MoreSeparate  &&  con->is_subset_of (edges))
		   zero_slack.push (con);
	     continue;
	  }
	  if ( stat == PLANAR_CONSTRAINT::VIOLATED )  {
		addConBuffer_->insert (pool->slot(i), true);
		(*neutralized)[con] = false;
		incrImportances (con);
	  }
	  else // stat = PLANAR_CONSTRAINT::EQUAL : slack = 0
	     zero_slack.push (con);
      }
  }

  return addConBuffer_->number () - num;
}
//******************************************************************************
int PLANAR_SUB::selectBranchingVariable(int &variable) { 
//   check_importances ();
   if (!planarisator ()->BranchMoreReferences )
       return closeHalf(variable, ABA_VARTYPE::Binary);

   variable = -1;
   int i, Position = 0, maxFound = 0;

   for (i = 0; i < nVar (); i++) 
   {
	if (master_->isInteger (xVal_[i]) || IsLocalSet (i)) 
		continue;
	if (vI.Importance (i) > maxFound) {
	//select a variable,which belongs to the most constraints
		variable = i;
		maxFound = vI.Importance (i);
	 }
  }
  if ( variable == -1 ) {
       cerr << "Error in selectBranchingVariable() !!!\n";
       exit (Fatal);
  }
  master()->out() << "BranchVar is varible #" << variable << " and it has " << maxFound << " References" << endl;

  return 0;
}
//******************************************************************************
int PLANAR_SUB::check_importances ()
{
  array<int> vars (nVar ());
  int i;
  for ( i = 0; i < nVar (); i++)
  	vars[i] = 0;
  
  for (i = 0; i < nCon (); i++) {
  	PLANAR_CONSTRAINT *con = (PLANAR_CONSTRAINT*)constraint(i);
	if (con->row()->nnz () - con->rhs () != 1 || (*neutralized)[con] ) 
	       continue; //this is either Euler constraint or already neutralized constraint
	for (int j = 0; j < con->row()->nnz(); j++)
		vars[con->row()->support(j)]++;
  }
  
  for ( i = 0; i < nVar (); i++)
  	if ( vars[i] != vI.Importance (i) ) {
  		vars.print ();
  		cout << "\n";
  		vI.print ();
  		cout << "\n";
  		cerr << "Die Var. Importance stimmt nicht!\n";
  		exit (Fatal);
  	}
}
/********************************************************************************
int PLANAR_SUB::check_neutralized ()  {
  for (int i = 0; i < nCon(); i++) 
	  if ((*neutralized)[(PLANAR_CONSTRAINT*)constraint(i)])  {
		  cerr "Eine neutralisierte Ungleichung ist aktiv\n";
		  exit (Fatal);
	  }
}*/
//********************************************************************************
void PLANAR_SUB::incrImportances (PLANAR_CONSTRAINT* con)  {
  for ( int j = 0; j < con->row()->nnz (); j++)
	vI.incrImportance (con->row()->support (j));
}
//*******************************************************************************
void PLANAR_SUB::fathom ( bool reoptimize )  {
  ABA_SUB::fathom (reoptimize);
  delete neutralized;
}
